#=========================================================================================================
#                                      AddIndexes
#=========================================================================================================
#
# Author(s)         :  Clover Point Cartographics (Mike Neal)
#
# Date              :  2009-04-08
#
# Purpose           :  This script creates any missing field indexes on items in feature classes and
#                      tables within the TEIS Environment (indexes can improve overall file
#                      geodatabase performance). The file geodatabases are then compacted. This tool
#                      should be run periodically after major changes are made, such as the loading of
#                      additional data.
#
# Arguments         :  None
#
# Example Input     :  X:\fullpath\AddIndexes.py
#
# Outputs           :  No new outputs are created. Indexes are created on existing feature classes and
#                      tables.
#
# Dependencies      :  None
#
# Expected Duration :  8+ hours (mostly for compacting Operational_Data.gdb)
#
# History           :  2012-10-12 (JK): Update to CPMessage routine: only append messages to PROCESS_LOG
#                                       if the e-mail function is being used (otherwise it can fill memory)
#
#                   :  2013-02-04 (JK): Remove references to TEIS_Master_Project_Details table (retired)
#
#                   :  2013-03-26 (JK): Set up to read TEIS_Metadata_Items Item_Indexed field to
#                                       determine fields to index; previously it was all just hard-coded.
#
#=========================================================================================================
#=========================================================================================================
#  Process Status and Return Codes:
#      -1   - Unexpected error
#       0   - Completed Successful Run
#       1   - In progress
#       100 - Invalid Arguments
#       101 - ESRI License Error
#
#=========================================================================================================

#*********************************************************************************************************
#  Program Initialization
#*********************************************************************************************************

#---------------------------------------------------------------------------------------------------------
#  Python and System Modules
#---------------------------------------------------------------------------------------------------------

from __future__ import division

import sys
import string
import os
import time
import random

#---------------------------------------------------------------------------------------------------------
#  Arguement Validation
#---------------------------------------------------------------------------------------------------------

# Capture/Validate Parameters
#  Test that the right number of paramters were provided and validated
#  Note that 1 must be added because the Python script is an arguement (0) supplied to the
#  Python Environment.


# If long text parameters are required, use:  any_value = gp.GetParameterAsText(0)

if len(sys.argv) < 1 or len(sys.argv) > 1 :

    print "\nInvalid number of arguments (" + str(len(sys.argv) - 1) + " supplied) to:"
    print "    " + sys.argv[0] + "\n"
    print " Usage:   " + sys.argv[0] 
    sys.exit(100)

PYTHON_SCRIPT      = sys.argv[0]


#---------------------------------------------------------------------------------------------------------
#  Standard/Global Variables/Constants
#---------------------------------------------------------------------------------------------------------

# Global Constants
COMPUTERNAME     = os.environ['COMPUTERNAME']
USERNAME         = os.environ['USERNAME']
START_TIME       = time.ctime(time.time())
START_TIME_SEC   = time.time()
START_TIME_SQL   = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
BEEP             = chr(7)


# Global Variables
#   The Processing_Variables is the collection of messaging and script specific information needed
#   throughout this program.  It is constantly updated, and generally includes all paths and configuration
#   information.  For parameters that are passed in to the script, they should generally be assigned
#   to a Processing_Variable through the Initialize() routine.

Processing_Variables = {}



#---------------------------------------------------------------------------------------------------------
#  CPC Variables/Constants
#---------------------------------------------------------------------------------------------------------




#---------------------------------------------------------------------------------------------------------
#  Set up the ArcGIS Geoprocessing Environment
#---------------------------------------------------------------------------------------------------------



# For ArcGIS 9.3:
try:
    import arcgisscripting
    gp = arcgisscripting.create(9.3)

# Get any required extensions
# if gp.CheckExtension("spatial") == "available":
#     gp.CheckOutExtension("spatial")
# else:
#     sys.exit(101)

except:
    print '**** ArcGIS Failed to initialize a GeoProcessor Object'
    sys.exit(101)



#---------------------------------------------------------------------------------------------------------
#  Script Specific Variables/Constants
#---------------------------------------------------------------------------------------------------------



#*********************************************************************************************************
#  Routines
#*********************************************************************************************************

#---------------------------------------------------------------------------------------------------------
# TemplateRoutine(Input_FC)
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def TemplateRoutine(Input_FC):
    CPMessage('    - Template Routine', 2)
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "TemplateRoutine" 

    result = gp.GetCount_management(Input_FC)
    Number_of_Features = int(result.GetOutput(0))
    Number_Processed = 0
    
    CPMessage('        - Calculating individual values for' + str(Number_of_Features) + ' features.', 4)
    rows = gp.SearchCursor(Input_FC)
    row = rows.Next()
    
    while row:
        if Number_Processed % 1000 == 0:
            CPMessage('            - Processed ' + str(Number_Processed), 4)

        Number_Processed = Number_Processed + 1
        row = rows.Next()
    del row
    del rows

    return(Processing_Variables['PROCESS_STATUS'])

#=========================================================================================================
#  Standard Routines
#=========================================================================================================

#---------------------------------------------------------------------------------------------------------
# Initialize()
#
#       Set up all of the paths and variables that we need to run this script.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def Initialize():
    print '    - Initializing Script'
    
    Processing_Variables['Script_Directory_Path'] = os.path.split(PYTHON_SCRIPT)[0]
    Processing_Variables['Script_Name'] = os.path.split(PYTHON_SCRIPT)[1]
    Processing_Variables['Log_Directory_Path'] = Processing_Variables['Script_Directory_Path'] + r'\log'

    Processing_Variables['Project_Base_Path']      = os.path.split(Processing_Variables['Script_Directory_Path'])[0]

    
    Processing_Variables['ToolBoxRun'] = 'FALSE'
    # Processing_Variables['SendRunLogToEmail'] = ['mneal@cloverpoint.com', 'chaos_mike@hotmail.com']
    Processing_Variables['Log_Level'] = 4
    Processing_Variables['PROCESS_INFO']   = "RUNNING " + PYTHON_SCRIPT
    Processing_Variables['PROCESS_STATUS'] = 1
    Processing_Variables['PROCESS_LOG'] = []
    Processing_Variables['MessageQueueFile']   = Processing_Variables['Log_Directory_Path'] + '\\' + Processing_Variables['Script_Name'] + '.' + COMPUTERNAME + '.log'

    #### Adding a FGDB Log will significantly slow down performance.
    # Processing_Variables['Log_FGDB_Table'] = Processing_Variables['Project_Base_Path'] + r"\Operational.gdb\Processing_Log"

    if not gp.Exists(Processing_Variables['Log_Directory_Path']):
        os.mkdir(Processing_Variables['Log_Directory_Path'])
        CPMessage(r'Created \Scripts\log folder', 1)
    
    CPMessage('Executing: ' + PYTHON_SCRIPT, 1)
    CPMessage('    Start Time: ' + START_TIME, 2)
    CPMessage('    Python ' + sys.version, 4)

    # Load required toolboxes...
    # gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Data Management Tools.tbx")
    
    # Set the Geoprocessing environment...
    gp.OverwriteOutput = 1
    # gp.cellSize = "MINOF"
    # gp.workspace = INPUT_LOCATION
    
    
    
    
    # CPC Naming Standard: {variable}_Path is the fully qualified path, otherwise it is just the name of the item.
  
    # Processing Variables
   
    Processing_Variables['OperationalFGDB_Path']           = Processing_Variables['Project_Base_Path'] + r'\Operational_Data.gdb'
    Processing_Variables['Scratch_Path']                   = Processing_Variables['Project_Base_Path'] + r'\Scratch'
    Processing_Variables['ScratchFGDB_Path']               = Processing_Variables['Scratch_Path'] + r'\Scratch.gdb'
    gp.scratchworkspace = Processing_Variables['ScratchFGDB_Path']

    Processing_Variables['Tools_and_Templates_FGDB_Path']  = Processing_Variables['Project_Base_Path'] + r'\Tools_and_Templates.gdb'
    Processing_Variables['TEIS_Metadata_Items']            = Processing_Variables['Tools_and_Templates_FGDB_Path'] + r'\TEIS_Metadata_Items'

    Processing_Variables['STE_TEI_BCGW_PEM_FGDB_Path']     = Processing_Variables['Project_Base_Path'] + r'\STE_TEI_BCGW_PEM.gdb'
    Processing_Variables['STE_TEI_BCGW_TEI_FGDB_Path']     = Processing_Variables['Project_Base_Path'] + r'\STE_TEI_BCGW_TEI.gdb'

    Processing_Variables['TEI_Long_Tbl']                   = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Long_Tbl'
    Processing_Variables['TEI_Project_Boundaries']         = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Project_Boundaries'
    Processing_Variables['TEI_Project_Details']            = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Project_Details'
    Processing_Variables['TEI_Sample_Site_Points']         = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Sample_Site_Points'
    Processing_Variables['TEI_Sample_Site_Lines']          = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Sample_Site_Lines'
    Processing_Variables['TEI_Long_Tbl_Arcs']              = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Long_Tbl_Arcs'
    Processing_Variables['TEI_Symbols_Points']             = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Symbols_Points'
    Processing_Variables['TEI_Symbols_Lines']              = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Symbols_Lines'
    Processing_Variables['TEI_Symbols_Polys']              = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Symbols_Polys'
    Processing_Variables['TEI_Symbols_Polys_Arcs']         = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Symbols_Polys_Arcs'
    Processing_Variables['TEI_Short_Tbl']                  = Processing_Variables['OperationalFGDB_Path'] + r'\TEI_Short_Tbl'
    Processing_Variables['TEIS_Master_Project_Boundaries'] = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Project_Boundaries'
    Processing_Variables['TEIS_Master_Long_Tbl_Arcs']      = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Long_Tbl_Arcs'
    Processing_Variables['TEIS_Master_Symbols_Points']     = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Symbols_Points'
    Processing_Variables['TEIS_Master_Symbols_Lines']      = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Symbols_Points'
    Processing_Variables['TEIS_Master_Symbols_Polys']      = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Symbols_Polys'
    Processing_Variables['TEIS_Master_Symbols_Polys_Arcs'] = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Symbols_Polys_Arcs'
    Processing_Variables['TEIS_Master_Sample_Site_Points'] = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Sample_Site_Points'
    Processing_Variables['TEIS_Master_Sample_Site_Lines']  = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Sample_Site_Lines'
    Processing_Variables['TEIS_Master_Long_Tbl']           = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Long_Tbl'
    Processing_Variables['TEIS_Master_Short_Tbl']          = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Short_Tbl'
    Processing_Variables['TEIS_Master_Project_Boundaries'] = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Master_Project_Boundaries'

    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# CPMessage(Message)
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def CPMessage(Message, MessageLevel):
    # Based on the logging level, add messages to the message queues.
    #   Text Run Log   : Implimented
    #   Console        : Implimented
    #   GeoProcessor   : Implimented
    #   Processing Log : Implimented (with e-mail)
    #   FGDB           : Implimented
    
    # Message Levels:
    #   0    : Fatal Errors
    #   1    : Top Level Program
    #   2    : Main routines and events
    #   3    : Warnings
    #   4    : General Information
    #   5    : Debug level detail
    
    # Send to the screen
    if MessageLevel <= Processing_Variables['Log_Level']:
        WriteOutputToScreen(Message, MessageLevel)    
    
    # Send to the Processing Log
    if 'SendRunLogToEmail' in Processing_Variables:
        Processing_Variables['PROCESS_LOG'].append((Message, MessageLevel))
    
    # Send to the Log File
    if 'MessageQueueFile' in Processing_Variables and MessageLevel <= Processing_Variables['Log_Level']:
        WriteOutputToFile(Message, Processing_Variables['MessageQueueFile'])

    # Send to the GeoProcessor Message Queue
    #### At ArcGIS 9.3 SP1 there is a bug that also causes any GeoProcessor messages to be echoed to the console.
    #    Added the ToolBoxRun parameter to allow the script to add messages to the GeoProcessor Messaging properly
    if MessageLevel < 4 and MessageLevel <= Processing_Variables['Log_Level'] and Processing_Variables['ToolBoxRun'] == 'TRUE':
        if MessageLevel == 0:
            gp.AddError(Message)
        if MessageLevel == 1 or MessageLevel == 2:
            gp.AddMessage(Message)
        if MessageLevel == 3:
            gp.AddWarning(Message)

    # Send to the FGDB Processing Log
    if MessageLevel < 4 and MessageLevel <= Processing_Variables['Log_Level'] and 'Log_FGDB_Table' in Processing_Variables:
        rows = gp.InsertCursor(Processing_Variables['Log_FGDB_Table'])

        row = rows.NewRow()
        row.SetValue("Computer_Name", COMPUTERNAME)
        row.SetValue("User_Name", USERNAME)
        row.SetValue("Script_Name", PYTHON_SCRIPT)
        row.SetValue("Event_Time", time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
        row.SetValue("Message_Level", MessageLevel)
        row.SetValue("Message", Message)
        row.SetValue("Start_Time", START_TIME_SQL)
        
        rows.InsertRow(row)
        del row
        del rows
    

    return(1)


#---------------------------------------------------------------------------------------------------------
# WriteOutputToScreen(Output_Comment, Header_Style):
#
#   Prints the Output_Comment to the screen with the appropriate formatting
#       0 : "******" Used for Error Sections
#       1 : "======" Used for the Program's Header and Footer
#       2 : "------" Used for New Sections
#       3+:          No delimitation
#   
#---------------------------------------------------------------------------------------------------------
def WriteOutputToScreen(Output_Comment, Header_Style):
    if Header_Style == 0:
        # Errors get beeps, too!
        print BEEP * 3 + "\n" * 2 + "*" * 79 + "\n" * 2
    if Header_Style == 2:
        print "\n" + "-" * 79 + "\n"
    if Header_Style == 1:
        print "\n" + "=" * 79 + "\n"
        
    #print Output_Comment
    gp.AddMessage(Output_Comment)
    
    if Header_Style == 0:
        print "\n" * 2 + "*" * 79 + "\n" * 2
    if Header_Style == 1:
        print "\n" + "=" * 79 + "\n"


#---------------------------------------------------------------------------------------------------------
# WriteOutputToFile(Output_Comment, Run_Log)
#   Record to Log 
#
#
#---------------------------------------------------------------------------------------------------------
def WriteOutputToFile(Output_Comment, Run_Log):
# Run logs are not compatible with concurrency - only one process is expected in debug mode at a time!
# First try to append to an existing file - if that fails create a new file.
    ReturnValue = 1
    try:
        if os.path.isfile(Run_Log) == 1:
            # try to open up the fileName 
            file = open(Run_Log,'a') 

            # write String to the fileName
            file.write(COMPUTERNAME + " " + START_TIME_SQL + ' (' + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()) + ') : ' + Output_Comment + '\n')

            # Close the file
            file.close()
            Processing_Variables['PROCESS_STATUS'] = 1
            Processing_Variables['PROCESS_INFO']   = "Appended comment to the file"
        else:
            # try to open up the fileName 
            file = open(Run_Log,'w') 

            # write String to the fileName
            file.write(COMPUTERNAME + " " + START_TIME_SQL + ' (' + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()) + ') : ' + Output_Comment + '\n')

            # Close the file
            file.close()
            Processing_Variables['PROCESS_STATUS'] = 1
            Processing_Variables['PROCESS_INFO']   = "Created new file and put in the comment"
    except:
        # All failures are unexpected in this case...  
        # Maybe an invalid drive name, read only file or something?
        Processing_Variables['PROCESS_STATUS'] = -1
        Processing_Variables['PROCESS_INFO']   = "**ERROR: Can't write to file: " + Run_Log   
        print BEEP + BEEP + BEEP
        ReturnValue = -1
    return(ReturnValue)
        

#---------------------------------------------------------------------------------------------------------
# SetupWorkspace()
#
#       Test if we need to create the workspace file geodatabases, and then do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def SetupWorkspace():
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "SetupWorkspace"  

    CPMessage('    - Setting Up Workspace', 2)
    
    if gp.Exists(Processing_Variables['OperationalFGDB_Path']):
        CPMessage('        -- Operational FGDB exists.', 4)
    else:
        CPMessage(' **** Operational FGDB does not exist: ' + Processing_Variables['OperationalFGDB_Path'], 0)
        sys.exit(100)

    if gp.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path']):
        CPMessage('        -- Tools and Templates FGDB exists.', 4)
    else:
        CPMessage(' **** Tools and Templates FGDB does not exist: ' + Processing_Variables['Tools_and_Templates_FGDB_Path'], 0)
        sys.exit(100)


    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# SendRunLogToEmail()
#
#       Test if we need to create the workspace file geodatabases, and then do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def SendRunLogToEmail(mailto):
    import smtplib
    CPMessage('  * Sending log to ' + mailto, 2)
    mailfrom = 'GIS_Python@cloverpoint.com'
    # mailto = 'mneal@cloverpoint.com'
    mailsubject = 'Run Log: ' + os.path.split(PYTHON_SCRIPT)[1] + ' (' + COMPUTERNAME + '\\' + USERNAME + ')'
    mailbody = "This is an automatic run log.\n\n"
    
    for LogEntry in Processing_Variables['PROCESS_LOG']:
        mailbody = mailbody + '\n' + LogEntry[0]

    mailmsg = ("From: " + mailfrom + "\r\nTo: " + mailto + "\r\nSubject: " + mailsubject + "\r\n" + mailbody)

    mailserver = smtplib.SMTP('shawmail.gv.shawcable.net')

    # SMTP.login(user, password)

    mailserver.sendmail(mailfrom, mailto, mailmsg)
    mailserver.quit()
    return(Processing_Variables['PROCESS_STATUS'])



#=========================================================================================================
#  Special Routines
#=========================================================================================================

#---------------------------------------------------------------------------------------------------------
# ListIndexedAttributes(Input_FC)
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def ListIndexedAttributes(Input_FC):
    CPMessage('        - Listing Indexed Attributes', 5)
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "ListIndexedAttributes" 

   
    CPMessage('            - Searching Indexes for Fields that are Indexed.', 5)
    Index_List = gp.ListIndexes(Input_FC)
    
    IndexedAttributeList = []
    
    for Index in Index_List:
        CPMessage('                - Index: ' + Index.Name, 5)
        # print Index.IsAscending
        # print Index.IsUnique
        for IndexFields in Index.Fields:
            CPMessage('                    - Field: ' + IndexFields.Name, 5)
            IndexedAttributeList.append(str(IndexFields.Name))
    CPMessage('            - Fields already indexed: ' + str(IndexedAttributeList), 4)
    
    return(IndexedAttributeList)



    

#*********************************************************************************************************
#  Main Program
#*********************************************************************************************************

#---------------------------------------------------------------------------------------------------------
# Run Main
#
#   Main Method manages the overall program
#---------------------------------------------------------------------------------------------------------
def runMain():

    Initialize()
    SetupWorkspace()


    CPMessage('    - Adding Indexes to Domains', 2)
    OriginalWorkspace = gp.Workspace
    
    # Get a list of all the domains in the Tools and Templates FGDB based on naming convention.
    
    gp.Workspace = Processing_Variables['Tools_and_Templates_FGDB_Path']
    Domain_List = gp.ListTables('Domain_*')
    
    # All domains must have a "Code" field, and that must be indexed.
    for Domain in Domain_List:
        CPMessage('        - Checking: ' + Domain, 4)
        IndexedAttributes = ListIndexedAttributes(Domain)
        if 'Code' not in IndexedAttributes:
            CPMessage('            - Adding Index to Code', 4)
            #### Create a new, unique random name.  May not be random across the whole FGDB, though.  ESRI Bug Case #101499
            #### Could use ListIndexes to get the comlete list, but this should work for now, and until the bug is resolved.
            IndexName = 'Index' + str(random.randint(0,1000000))
            while IndexName in IndexedAttributes:
                IndexName = 'Index' + str(random.randint(0,1000000))
            gp.AddIndex_management(Domain, "Code", IndexName, "UNIQUE", "NON_ASCENDING")
        else:
            CPMessage('            - Index already exists for Code.', 5)


    CPMessage('    - Adding Indexes to Templates and Operational Tables.', 2)
    # Each template and operational table has its own unique set of items to index:
    
    fields_to_index = []

    CPMessage('        - Looking up fields to index from TEIS_Metadata_Items table', 4)
    rows = gp.SearchCursor(Processing_Variables['TEIS_Metadata_Items'])
    row = rows.Next()
    
    while row:
        if str(row.GetValue('Item_Indexed')) == 'Y':
            fields_to_index.append(str(row.GetValue('FGDB_Name')))
            CPMessage('            - ' + str(row.GetValue('FGDB_Name')), 4)
        row = rows.Next()
    del row
    del rows

    feature_class_list = ['TEI_Long_Tbl']
    feature_class_list.append('TEI_Project_Boundaries')
    feature_class_list.append('TEI_Project_Details')
    feature_class_list.append('TEI_Sample_Site_Points')
    feature_class_list.append('TEI_Sample_Site_Lines')
    feature_class_list.append('TEI_Long_Tbl_Arcs')
    feature_class_list.append('TEI_Symbols_Points')
    feature_class_list.append('TEI_Symbols_Lines')
    feature_class_list.append('TEI_Symbols_Polys')
    feature_class_list.append('TEI_Symbols_Polys_Arcs')
    feature_class_list.append('TEI_Short_Tbl')
    feature_class_list.append('TEIS_Master_Project_Boundaries')
    feature_class_list.append('TEIS_Master_Long_Tbl_Arcs')
    feature_class_list.append('TEIS_Master_Symbols_Points')
    feature_class_list.append('TEIS_Master_Symbols_Points')
    feature_class_list.append('TEIS_Master_Symbols_Polys')
    feature_class_list.append('TEIS_Master_Symbols_Polys_Arcs')
    feature_class_list.append('TEIS_Master_Sample_Site_Points')
    feature_class_list.append('TEIS_Master_Sample_Site_Lines')
    feature_class_list.append('TEIS_Master_Long_Tbl')
    feature_class_list.append('TEIS_Master_Short_Tbl')
    feature_class_list.append('TEIS_Master_Project_Boundaries')

    for feature_class in feature_class_list:
        if gp.Exists(Processing_Variables[feature_class]):
            CPMessage('        - Feature Class Found: ' + feature_class + '.' , 3)
            field_list = [str(f.name) for f in gp.ListFields(Processing_Variables[feature_class])]
            indexed_field_list = ListIndexedAttributes(Processing_Variables[feature_class])
            for field_to_index in fields_to_index:
                if field_to_index in field_list:
                    if field_to_index not in indexed_field_list:
                        index_name_list = [str(idx.name) for idx in gp.ListIndexes(Processing_Variables[feature_class])]
                        CPMessage('            - Adding Index to ' + field_to_index + ' in feature class ' + feature_class + '.', 4)
                        #### Create a new, unique random name.  May not be random across the whole FGDB, though.  ESRI Bug Case #101499
                        #### Could use ListIndexes to get the comlete list, but this should work for now, and until the bug is resolved.
                        index_name = 'Index' + str(random.randint(0,1000000))
                        while index_name in index_name_list:
                            index_name = 'Index' + str(random.randint(0,1000000))
                        gp.AddIndex_management(Processing_Variables[feature_class], field_to_index, index_name, "NON_UNIQUE", "NON_ASCENDING")
                    else:
                        CPMessage('            - Index already exists for ' + field_to_index + '.' , 5)
        else:
            CPMessage('        - Feature Class Not Found: ' + feature_class + '.' , 3)

    CPMessage('    - Compacting FGDBs.' , 2)
    if gp.Exists(Processing_Variables['OperationalFGDB_Path']):
        CPMessage('        - Operational FGDB.' , 4)
        gp.Compact_management(Processing_Variables['OperationalFGDB_Path'])

    if gp.Exists(Processing_Variables['ScratchFGDB_Path']):
        CPMessage('        - Scratch FGDB.' , 4)
        gp.Compact_management(Processing_Variables['ScratchFGDB_Path'])
    
    if gp.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path']):
        CPMessage('        - Tools and Templates FGDB.' , 4)
        gp.Compact_management(Processing_Variables['Tools_and_Templates_FGDB_Path'])
    
    if gp.Exists(Processing_Variables['STE_TEI_BCGW_PEM_FGDB_Path']) and gp.Exists(Processing_Variables['STE_TEI_BCGW_TEI_FGDB_Path']):
        CPMessage('        - LRDW Staging FGDBs.' , 4)
        gp.Compact_management(Processing_Variables['STE_TEI_BCGW_PEM_FGDB_Path'])
        gp.Compact_management(Processing_Variables['STE_TEI_BCGW_TEI_FGDB_Path'])
    
    return Processing_Variables['PROCESS_STATUS']

#*********************************************************************************************************
#  Execution
#*********************************************************************************************************


runMain()



#---------------------------------------------------------------------------------------------------------
#  Clean-up
#---------------------------------------------------------------------------------------------------------


END_TIME       = time.ctime(time.time())
END_TIME_SEC   = time.time()
END_TIME_SQL   = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
EXECUTION_TIME = END_TIME_SEC - START_TIME_SEC
if EXECUTION_TIME < 120:
    EXECUTION_TIME_STRING = str(int(EXECUTION_TIME)) + ' seconds.'
elif EXECUTION_TIME < 3600:
    EXECUTION_TIME_STRING = str(round(EXECUTION_TIME / 60, 1)) + ' minutes.'
else:
    EXECUTION_TIME_STRING = str(round(EXECUTION_TIME / 3600, 2)) + ' hours.'


CPMessage('    End Time: ' + END_TIME, 2)
CPMessage('    Execution Time: ' + EXECUTION_TIME_STRING, 4)
CPMessage('Complete: ' + PYTHON_SCRIPT, 1)

# gp.CheckInExtension("spatial")
del gp

# Report Program Execution Status back to the calling program


if 'SendRunLogToEmail' in Processing_Variables:
    for Email in Processing_Variables['SendRunLogToEmail']:
        SendRunLogToEmail(Email)

#*********************************************************************************************************
#  End of Program
#*********************************************************************************************************
