#=========================================================================================================
#                                      CheckUniqueTEIS_ID
#=========================================================================================================
#
# Author(s)         :  Clover Point Cartographics (Mike Neal, Jeff Kruys)
#
# Date              :  2012-10-05
#
# Purpose           :  This script reports and optionally renumbers duplicate or null values found in the
#                      TEIS_ID field of the input feature class. If the "Renumber" option is used, it will
#                      determine the current highest number in the TEIS_ID field and renumber the duplicates
#                      beginning at a number one higher than that highest number. If the "NoRenumber" option
#                      is specified, the script will simply report on screen that duplicates were detected,
#                      and list the first ten duplicate values found.
#              
# Arguments         :  argv(1) = INPUT_FC              (Feature class to be checked) 
#                   :  argv(2) = {Renumber|NoRenumber} (Option to renumber any duplicate/null values found) 
#
# Example Input     :  X:\fullpath\CheckUniqueTEIS_ID.py X:\fullpath\TEIS_Master_Long_Tbl {Renumber|NoRenumber}
#
# Outputs           :  No new outputs created. The input feature class will be modified only if the Renumber 
#                      option was specified and duplicates are found.
#
# Dependencies      :  None. Input feature class must have a TEIS_ID field.
#              
# Expected Duration :  < 1 minute for approximately 200,000 records
#                   :  < 2 hours for approximately 24,000,000 records
#
# History           :  2012-10-12 (JK): Update to CPMessage routine: only append messages to PROCESS_LOG
#                                       if the e-mail function is being used (otherwise it can fill memory)
#
#                   :  2012-10-26 (JK): Replace references to 'gp' with 'arcpy' so WriteOutputToScreen 
#                                       doesn't have to check every time for Arc version
#
#                   :  2013-01-02 (JK): Minor changes to on-screen messages displayed during execution
#
#                   :  2013-06-19 (JK): Fixed mistakes in header comments
#
#=========================================================================================================
#=========================================================================================================
#  Process Status and Return Codes:
#      -1   - Unexpected error
#       0   - Completed Successful Run
#       1   - In progress
#       100 - Invalid Arguments
#       101 - ESRI License Error
#
#=========================================================================================================

#*********************************************************************************************************
#  Program Initialization
#*********************************************************************************************************

#---------------------------------------------------------------------------------------------------------
#  Python and System Modules
#---------------------------------------------------------------------------------------------------------

from __future__ import division

import sys
import string
import os
import time


#---------------------------------------------------------------------------------------------------------
#  Arguement Validation
#---------------------------------------------------------------------------------------------------------

# Capture/Validate Parameters
#  Test that the right number of paramters were provided and validated
#  Note that 1 must be added because the Python script is an arguement (0) supplied to the
#  Python Environment.


# If long text parameters are required, use:  any_value = arcpy.GetParameterAsText(0)

if len(sys.argv) <> 3:
    print "\nInvalid number of arguments (" + str(len(sys.argv) - 1) + " supplied) to:"
    print "    " + sys.argv[0] + "\n"
    print "Usage:   " + sys.argv[0] + " [INPUT_FC] {Renumber|NoRenumber}"
    print " [INPUT_FC]            : Feature class to be validated (long table feature class in an FGDB)."
    print " {Renumber|NoRenumber} : Specify 'Renumber' if you wish to renumber any duplicate TEIS_ID values found."
    sys.exit(100)
if sys.argv[2] not in ['Renumber','NoRenumber','true','false']:
    print "\nInvalid number of arguments (" + str(len(sys.argv) - 1) + " supplied) to:"
    print "    " + sys.argv[0] + "\n"
    print "Usage:   " + sys.argv[0] + " [INPUT_FC] {Renumber|NoRenumber}"
    print " [INPUT_FC]            : Feature class to be validated (long table feature class in an FGDB)."
    print " {Renumber|NoRenumber} : Specify 'Renumber' if you wish to renumber any duplicate TEIS_ID values found."
    sys.exit(100)
    
PYTHON_SCRIPT       = sys.argv[0]
INPUT_FC   = sys.argv[1]
if sys.argv[2] in ['Renumber','true']:
    renumber = True
else:
    renumber = False

#---------------------------------------------------------------------------------------------------------
#  Standard/Global Variables/Constants
#---------------------------------------------------------------------------------------------------------

# Global Constants
COMPUTERNAME     = os.environ['COMPUTERNAME']
USERNAME         = os.environ['USERNAME']
START_TIME       = time.ctime(time.time())
START_TIME_SEC   = time.time()
START_TIME_SQL   = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
BEEP             = chr(7)


# Global Variables
#   The Processing_Variables is the collection of messaging and script specific information needed
#   throughout this program.  It is constantly updated, and generally includes all paths and configuration
#   information.  For parameters that are passed in to the script, they should generally be assigned
#   to a Processing_Variable through the Initialize() routine.

Processing_Variables = {}

#---------------------------------------------------------------------------------------------------------
#  CPC Variables/Constants
#---------------------------------------------------------------------------------------------------------

#---------------------------------------------------------------------------------------------------------
#  Set up the ArcGIS Geoprocessing Environment
#---------------------------------------------------------------------------------------------------------

try:
    import arcgisscripting
    arcpy = arcgisscripting.create(93)
except:
    print '**** ArcGIS Failed to initialize a GeoProcessor Object'
    sys.exit(101)

d = arcpy.GetInstallInfo('desktop')
if float(d['Version'][:4]) >= 10.1:
    del arcpy
    import arcpy

#---------------------------------------------------------------------------------------------------------
#  Script Specific Variables/Constants
#---------------------------------------------------------------------------------------------------------

#*********************************************************************************************************
#  Routines
#*********************************************************************************************************

#---------------------------------------------------------------------------------------------------------
# TemplateRoutine93(Input_FC)
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def TemplateRoutine93(Input_FC):
    CPMessage('    - Template Routine', 2)
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "TemplateRoutine" 

    result = arcpy.GetCount_management(Input_FC)
    Number_of_Features = int(result.GetOutput(0))
    Number_Processed = 0
    
    CPMessage('        - Calculating individual values for' + str(Number_of_Features) + ' features.', 4)
    rows = arcpy.SearchCursor(Input_FC)
    row = rows.Next()
    
    while row:
        if Number_Processed % 1000 == 0:
            CPMessage('            - Processed ' + str(Number_Processed), 4)

        Number_Processed = Number_Processed + 1
        row = rows.Next()
    del row
    del rows

    return(Processing_Variables['PROCESS_STATUS'])

#=========================================================================================================
#  Standard Routines
#=========================================================================================================

#---------------------------------------------------------------------------------------------------------
# Initialize93()
#
#       Set up all of the paths and variables that we need to run this script.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def Initialize93():
    print '    - Initializing Script'
    
    Processing_Variables['Script_Directory_Path'] = os.path.split(PYTHON_SCRIPT)[0]
    Processing_Variables['Script_Name'] = os.path.split(PYTHON_SCRIPT)[1]
    Processing_Variables['Log_Directory_Path'] = Processing_Variables['Script_Directory_Path'] + r'\log'

    Processing_Variables['Project_Base_Path']      = os.path.split(Processing_Variables['Script_Directory_Path'])[0]

    
    Processing_Variables['ToolBoxRun'] = 'FALSE'
    # Processing_Variables['SendRunLogToEmail'] = ['mneal@cloverpoint.com', 'chaos_mike@hotmail.com']
    Processing_Variables['Log_Level'] = 4
    Processing_Variables['PROCESS_INFO']   = "RUNNING " + PYTHON_SCRIPT
    Processing_Variables['PROCESS_STATUS'] = 1
    Processing_Variables['PROCESS_LOG'] = []
    Processing_Variables['MessageQueueFile']   = Processing_Variables['Log_Directory_Path'] + '\\' + Processing_Variables['Script_Name'] + '.' + COMPUTERNAME + '.log'

    #### Adding a FGDB Log will significantly slow down performance.
    # Processing_Variables['Log_FGDB_Table'] = Processing_Variables['Project_Base_Path'] + r"\Operational.gdb\Processing_Log"

    if not arcpy.Exists(Processing_Variables['Log_Directory_Path']):
        os.mkdir(Processing_Variables['Log_Directory_Path'])
        CPMessage(r'Created \Scripts\log folder', 1)

    CPMessage('Executing: ' + PYTHON_SCRIPT, 1)
    CPMessage('    Start Time: ' + START_TIME, 2)
    CPMessage('    Python ' + sys.version, 4)

    # Load required toolboxes...
    # arcpy.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Data Management Tools.tbx")
    
    # Set the Geoprocessing environment...
    arcpy.OverwriteOutput = 1
    # arcpy.cellSize = "MINOF"
    # arcpy.workspace = INPUT_LOCATION
    
    
    
    # CPC Naming Standard: {variable}_Path is the fully qualified path, otherwise it is just the name of the item.
  
    # Processing Variables
   
    Processing_Variables['OperationalFGDB_Path']   = Processing_Variables['Project_Base_Path'] + r'\Operational_Data.gdb'
   
    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# Initialize101()
#
#       Set up all of the paths and variables that we need to run this script.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def Initialize101():
    print '    - Initializing Script'
    
    Processing_Variables['Script_Directory_Path'] = os.path.split(PYTHON_SCRIPT)[0]
    Processing_Variables['Script_Name'] = os.path.split(PYTHON_SCRIPT)[1]
    Processing_Variables['Log_Directory_Path'] = Processing_Variables['Script_Directory_Path'] + r'\log'

    Processing_Variables['Project_Base_Path']      = os.path.split(Processing_Variables['Script_Directory_Path'])[0]

    
    Processing_Variables['ToolBoxRun'] = 'FALSE'
    # Processing_Variables['SendRunLogToEmail'] = ['mneal@cloverpoint.com', 'chaos_mike@hotmail.com']
    Processing_Variables['Log_Level'] = 4
    Processing_Variables['PROCESS_INFO']   = "RUNNING " + PYTHON_SCRIPT
    Processing_Variables['PROCESS_STATUS'] = 1
    Processing_Variables['PROCESS_LOG'] = []
    Processing_Variables['MessageQueueFile']   = Processing_Variables['Log_Directory_Path'] + '\\' + Processing_Variables['Script_Name'] + '.' + COMPUTERNAME + '.log'

    #### Adding a FGDB Log will significantly slow down performance.
    # Processing_Variables['Log_FGDB_Table'] = Processing_Variables['Project_Base_Path'] + r"\Operational.gdb\Processing_Log"

    if not arcpy.Exists(Processing_Variables['Log_Directory_Path']):
        os.mkdir(Processing_Variables['Log_Directory_Path'])
        CPMessage(r'Created \Scripts\log folder', 1)

    CPMessage('Executing: ' + PYTHON_SCRIPT, 1)
    CPMessage('    Start Time: ' + START_TIME, 2)
    CPMessage('    Python ' + sys.version, 4)

    # Load required toolboxes...
    # arcpy.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Data Management Tools.tbx")
    
    # Set the Geoprocessing environment...
    arcpy.OverwriteOutput = 1
    # arcpy.cellSize = "MINOF"
    # arcpy.env.workspace = INPUT_LOCATION
    
    
    
    # CPC Naming Standard: {variable}_Path is the fully qualified path, otherwise it is just the name of the item.
  
    # Processing Variables
   
    Processing_Variables['OperationalFGDB_Path']   = Processing_Variables['Project_Base_Path'] + r'\Operational_Data.gdb'
   
    return(Processing_Variables['PROCESS_STATUS'])

#---------------------------------------------------------------------------------------------------------
# CPMessage(Message)
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def CPMessage(Message, MessageLevel):
    # Based on the logging level, add messages to the message queues.
    #   Text Run Log   : Implimented
    #   Console        : Implimented
    #   GeoProcessor   : Implimented
    #   Processing Log : Implimented (with e-mail)
    #   FGDB           : Implimented
    
    # Message Levels:
    #   0    : Fatal Errors
    #   1    : Top Level Program
    #   2    : Main routines and events
    #   3    : Warnings
    #   4    : General Information
    #   5    : Debug level detail
    
    # Send to the screen
    if MessageLevel <= Processing_Variables['Log_Level']:
        WriteOutputToScreen(Message, MessageLevel)    
    
        # Send to the Processing Log
        #### The log is extremely large with this script and may overflow memory if all levels are appended.
        if 'SendRunLogToEmail' in Processing_Variables:
            Processing_Variables['PROCESS_LOG'].append((Message, MessageLevel))
    
    # Send to the Log File
    if 'MessageQueueFile' in Processing_Variables and MessageLevel <= Processing_Variables['Log_Level']:
        WriteOutputToFile(Message, Processing_Variables['MessageQueueFile'])

    # Send to the GeoProcessor Message Queue
    #### At ArcGIS 9.3 SP1 there is a bug that also causes any GeoProcessor messages to be echoed to the console.
    #    Added the ToolBoxRun parameter to allow the script to add messages to the GeoProcessor Messaging properly
    if MessageLevel < 4 and MessageLevel <= Processing_Variables['Log_Level'] and Processing_Variables['ToolBoxRun'] == 'TRUE':
        if MessageLevel == 0:
            arcpy.AddError(Message)
        if MessageLevel == 1 or MessageLevel == 2:
            arcpy.AddMessage(Message)
        if MessageLevel == 3:
            arcpy.AddWarning(Message)

    # Send to the FGDB Processing Log
    if MessageLevel < 4 and MessageLevel <= Processing_Variables['Log_Level'] and 'Log_FGDB_Table' in Processing_Variables:
        rows = arcpy.InsertCursor(Processing_Variables['Log_FGDB_Table'])

        row = rows.NewRow()
        row.SetValue("Computer_Name", COMPUTERNAME)
        row.SetValue("User_Name", USERNAME)
        row.SetValue("Script_Name", PYTHON_SCRIPT)
        row.SetValue("Event_Time", time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
        row.SetValue("Message_Level", MessageLevel)
        row.SetValue("Message", Message)
        row.SetValue("Start_Time", START_TIME_SQL)
        
        rows.InsertRow(row)
        del row
        del rows
    

    return(1)


#---------------------------------------------------------------------------------------------------------
# WriteOutputToScreen(Output_Comment, Header_Style):
#
#   Prints the Output_Comment to the screen with the appropriate formatting
#       0 : "******" Used for Error Sections
#       1 : "======" Used for the Program's Header and Footer
#       2 : "------" Used for New Sections
#       3+:          No delimitation
#   
#---------------------------------------------------------------------------------------------------------
def WriteOutputToScreen(Output_Comment, Header_Style):
    if Header_Style == 0:
        # Errors get beeps, too!
        print BEEP * 3 + "\n" * 2 + "*" * 79 + "\n" * 2
    if Header_Style == 2:
        print "\n" + "-" * 79 + "\n"
    if Header_Style == 1:
        print "\n" + "=" * 79 + "\n"
        
    #print Output_Comment
    arcpy.AddMessage(Output_Comment)
    
    if Header_Style == 0:
        print "\n" * 2 + "*" * 79 + "\n" * 2
    if Header_Style == 1:
        print "\n" + "=" * 79 + "\n"


#---------------------------------------------------------------------------------------------------------
# WriteOutputToFile(Output_Comment, Run_Log)
#   Record to Log 
#
#
#---------------------------------------------------------------------------------------------------------
def WriteOutputToFile(Output_Comment, Run_Log):
# Run logs are not compatible with concurrency - only one process is expected in debug mode at a time!
# First try to append to an existing file - if that fails create a new file.
    ReturnValue = 1
    try:
        if os.path.isfile(Run_Log) == 1:
            # try to open up the fileName 
            file = open(Run_Log,'a') 

            # write String to the fileName
            file.write(COMPUTERNAME + " " + START_TIME_SQL + ' (' + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()) + ') : ' + Output_Comment + '\n')

            # Close the file
            file.close()
            Processing_Variables['PROCESS_STATUS'] = 1
            Processing_Variables['PROCESS_INFO']   = "Appended comment to the file"
        else:
            # try to open up the fileName 
            file = open(Run_Log,'w') 

            # write String to the fileName
            file.write(COMPUTERNAME + " " + START_TIME_SQL + ' (' + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()) + ') : ' + Output_Comment + '\n')

            # Close the file
            file.close()
            Processing_Variables['PROCESS_STATUS'] = 1
            Processing_Variables['PROCESS_INFO']   = "Created new file and put in the comment"
    except:
        # All failures are unexpected in this case...  
        # Maybe an invalid drive name, read only file or something?
        Processing_Variables['PROCESS_STATUS'] = -1
        Processing_Variables['PROCESS_INFO']   = "**ERROR: Can't write to file: " + Run_Log   
        print BEEP + BEEP + BEEP
        ReturnValue = -1
    return(ReturnValue)
        

#---------------------------------------------------------------------------------------------------------
# SetupWorkspace93()
#
#       Test if we need to create the workspace file geodatabases, and then do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def SetupWorkspace93():
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "SetupWorkspace93"  

    CPMessage('    - Setting Up Workspace', 2)
    
    if arcpy.Exists(INPUT_FC):
        CPMessage('        -- User-specified input feature class exists.', 4)
    else:
        CPMessage(' **** User-specified input feature class does not exist: ' + INPUT_FC, 0)
        sys.exit(100)

    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# SetupWorkspace101()
#
#       Test if we need to create the workspace file geodatabases, and then do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def SetupWorkspace101():
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "SetupWorkspace101"  

    CPMessage('    - Setting Up Workspace', 2)
    
    if arcpy.Exists(INPUT_FC):
        CPMessage('        -- User-specified input feature class exists.', 4)
    else:
        CPMessage(' **** User-specified input feature class does not exist: ' + INPUT_FC, 0)
        sys.exit(100)

    return(Processing_Variables['PROCESS_STATUS'])

#---------------------------------------------------------------------------------------------------------
# SendRunLogToEmail()
#
#       Test if we need to create the workspace file geodatabases, and then do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def SendRunLogToEmail(mailto):
    import smtplib
    CPMessage('  * Sending log to ' + mailto, 2)
    mailfrom = 'GIS_Python@cloverpoint.com'
    # mailto = 'mneal@cloverpoint.com'
    mailsubject = 'Run Log: ' + os.path.split(PYTHON_SCRIPT)[1] + ' (' + COMPUTERNAME + '\\' + USERNAME + ')'
    mailbody = "This is an automatic run log.\n\n"
    
    for LogEntry in Processing_Variables['PROCESS_LOG']:
        mailbody = mailbody + '\n' + LogEntry[0]

    mailmsg = ("From: " + mailfrom + "\r\nTo: " + mailto + "\r\nSubject: " + mailsubject + "\r\n" + mailbody)

    mailserver = smtplib.SMTP('shawmail.gv.shawcable.net')

    # SMTP.login(user, password)

    mailserver.sendmail(mailfrom, mailto, mailmsg)
    mailserver.quit()
    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# SanitizeElapsedTime(number_of_seconds)
#
# routine to convert a raw number of seconds into a human-friendly statement of duration
#---------------------------------------------------------------------------------------------------------

def SanitizeElapsedTime(dtInput):

    if dtInput < 120:
        strElapsedTime = str(int(dtInput)) + ' seconds.'
        
    elif dtInput < 3600:
        strElapsedTime = str(int(dtInput)) + ' seconds (' + str(round(dtInput / 60, 1)) + ' minutes).'
        
    else:
        strElapsedTime = str(int(dtInput)) + ' seconds (' + str(round(dtInput / 3600, 2)) + ' hours).'
        
    return strElapsedTime


#=========================================================================================================
#  Special Routines
#=========================================================================================================



#---------------------------------------------------------------------------------------------------------
# CheckUniqueTEISID93()
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def CheckUniqueTEISID93():
    CPMessage('    - Checking uniqueness of TEIS_ID', 2)
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "CheckUniqueTEISID93"

    # check if TEIS_ID field exists
    if "TEIS_ID" not in [f.name for f in arcpy.ListFields(INPUT_FC)]:
        CPMessage(' **** Input feature class does not contain a TEIS_ID field.', 0)
        sys.exit(100)
        
    Cursor_Field_List = ["TEIS_ID"]
    
    TEIS_ID_Dict = {}
    Highest_TEIS_ID = 0
    Duplicate_List = []
    Duplicate_Counter = 0
    
    result = arcpy.GetCount_management(INPUT_FC)
    Number_of_Features = int(result.GetOutput(0))
    Number_Processed = 0
    
    CPMessage('        - Reading individual values for ' + str(Number_of_Features) + ' features.', 4)
    rows = arcpy.UpdateCursor(INPUT_FC)
    row = rows.Next()

    dtCalcStart = time.time()
    while row:

        Number_Processed = Number_Processed + 1
        
        TEIS_ID = row.getValue("TEIS_ID")
        
        if str(TEIS_ID) == 'None':
            if renumber:
                Duplicate_Counter += 1
            else:
                if len(Duplicate_List) < 10:
                    Duplicate_List.append(TEIS_ID)  ## just store the first 10 duplicate values and list them at the end if there are any

        else:
        
            if TEIS_ID > Highest_TEIS_ID:
                Highest_TEIS_ID = TEIS_ID

            dict_key = int(str(TEIS_ID).zfill(3)[:-2])        ## lists of values stored in groups of 100, stored in dictionary indexed on key value int(TEIS_ID / 100). Makes checking for duplicates faster.
            if dict_key not in TEIS_ID_Dict:
                TEIS_ID_Dict[dict_key] = []

            if TEIS_ID not in TEIS_ID_Dict[dict_key]:
                TEIS_ID_Dict[dict_key].append(TEIS_ID)
            else:
                if renumber:
                    Duplicate_Counter += 1
                    row.setNull("TEIS_ID")
                    rows.updateRow(row)           ## set duplicate value to Null for now; will renumber all Nulls in next step once we know the current highest value to start numbering at.
                else:
                    if len(Duplicate_List) < 10:
                        Duplicate_List.append(TEIS_ID)  ## just store the first 10 duplicate values and list them at the end if there are any

        if (0 < Number_Processed <= 10000 and Number_Processed % 1000 == 0) or (10000 < Number_Processed <= 100000 and Number_Processed % 10000 == 0) or (100000 < Number_Processed and Number_Processed % 100000 == 0):
            dtCalcNow = time.time()
            dtCalcElapsed = dtCalcNow - dtCalcStart
            dtCalcRemaining = (dtCalcElapsed / Number_Processed) * (Number_of_Features - Number_Processed)
            CPMessage('            - Processed ' + str(Number_Processed) + ' of ' + str(Number_of_Features) + ' (' + SanitizeElapsedTime(dtCalcElapsed)[:-1] + ' elapsed, ' + SanitizeElapsedTime(dtCalcRemaining)[:-1] + ' remaining)', 4)

        row = rows.Next()

    CPMessage('            - Processed all ' + str(Number_Processed), 4)
    del row
    del rows

    if renumber: 
        if Duplicate_Counter > 0:

            Number_Processed = 0
            CPMessage('        - Renumbering ' + str(Duplicate_Counter) + ' duplicate/null TEIS_ID value(s); updated TEIS_ID value(s) will start at ' + str(Highest_TEIS_ID + 1), 4)
            rows = arcpy.UpdateCursor(INPUT_FC)
            row = rows.Next()
            dtCalcStart = time.time()

            while row:

                Number_Processed = Number_Processed + 1

                TEIS_ID = row.getValue("TEIS_ID")

                if TEIS_ID == None:
                    Highest_TEIS_ID += 1
                    row.setValue("TEIS_ID",Highest_TEIS_ID)
                    rows.updateRow(row)

                if (0 < Number_Processed <= 10000 and Number_Processed % 1000 == 0) or (10000 < Number_Processed <= 100000 and Number_Processed % 10000 == 0) or (100000 < Number_Processed and Number_Processed % 100000 == 0):
                    dtCalcNow = time.time()
                    dtCalcElapsed = dtCalcNow - dtCalcStart
                    dtCalcRemaining = (dtCalcElapsed / Number_Processed) * (Number_of_Features - Number_Processed)
                    CPMessage('            - Processed ' + str(Number_Processed) + ' of ' + str(Number_of_Features) + ' (' + SanitizeElapsedTime(dtCalcElapsed)[:-1] + ' elapsed, ' + SanitizeElapsedTime(dtCalcRemaining)[:-1] + ' remaining)', 4)

                row = rows.Next()

            CPMessage('            - Processed all ' + str(Number_Processed), 4)
            del row
            del rows

        else:
            CPMessage('        - No duplicate or null TEIS_ID values found.', 4)

    else:
        if len(Duplicate_List) == 0:
            CPMessage('        - No duplicate or null TEIS_ID values found.', 4)
        else:
            CPMessage('    **** Duplicate and/or null TEIS_ID values found. Listing up to the first ten duplicate/null values found: ' + str(Duplicate_List).replace('[','').replace(']','').replace(' ','').replace('None','<Null>'), 0)

    return(Processing_Variables['PROCESS_STATUS'])

#---------------------------------------------------------------------------------------------------------
# CheckUniqueTEISID101()
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def CheckUniqueTEISID101():
    CPMessage('    - Checking uniqueness of TEIS_ID', 2)
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "CheckUniqueTEISID101"

    # check if TEIS_ID field exists
    if "TEIS_ID" not in [f.name for f in arcpy.ListFields(INPUT_FC)]:
        CPMessage(' **** Input feature class does not contain a TEIS_ID field.', 0)
        sys.exit(100)
        
    Cursor_Field_List = ["TEIS_ID"]
    
    TEIS_ID_Dict = {}
    Highest_TEIS_ID = 0
    Duplicate_List = []
    Duplicate_Counter = 0
    
    Number_of_Features = int(arcpy.GetCount_management(INPUT_FC).getOutput(0))
    Number_Processed = 0
    
    CPMessage('        - Reading individual values for ' + str(Number_of_Features) + ' features.', 4)

    dtCalcStart = time.time()
    exec_string = '''with arcpy.da.UpdateCursor(INPUT_FC, Cursor_Field_List) as cursor:

    for row in cursor:

        Number_Processed = Number_Processed + 1
        
        if str(row[0]) == 'None':
            if renumber:
                Duplicate_Counter += 1
            else:
                if len(Duplicate_List) < 10:
                    Duplicate_List.append(row[0])  ## just store the first 10 duplicate values and list them at the end if there are any

        else:

            if row[0] > Highest_TEIS_ID:
                Highest_TEIS_ID = row[0]

            dict_key = int(str(row[0]).zfill(3)[:-2])        ## lists of values stored in groups of 100, stored in dictionary indexed on key value int(TEIS_ID / 100). Makes checking for duplicates faster.
            if dict_key not in TEIS_ID_Dict:
                TEIS_ID_Dict[dict_key] = []

            if row[0] not in TEIS_ID_Dict[dict_key]:
                TEIS_ID_Dict[dict_key].append(row[0])
            else:
                if renumber:
                    Duplicate_Counter += 1
                    row[0] = None
                    cursor.updateRow(row)           ## set duplicate value to Null for now; will renumber all Nulls in next step once we know the current highest value to start numbering at.
                else:
                    if len(Duplicate_List) < 10:
                        Duplicate_List.append(row[0])  ## just store the first 10 duplicate values and list them at the end if there are any

            if (0 < Number_Processed <= 10000 and Number_Processed % 1000 == 0) or (10000 < Number_Processed <= 100000 and Number_Processed % 10000 == 0) or (100000 < Number_Processed and Number_Processed % 100000 == 0):
                dtCalcNow = time.time()
                dtCalcElapsed = dtCalcNow - dtCalcStart
                dtCalcRemaining = (dtCalcElapsed / Number_Processed) * (Number_of_Features - Number_Processed)
                CPMessage('            - Processed ' + str(Number_Processed) + ' of ' + str(Number_of_Features) + ' (' + SanitizeElapsedTime(dtCalcElapsed)[:-1] + ' elapsed, ' + SanitizeElapsedTime(dtCalcRemaining)[:-1] + ' remaining)', 4)

CPMessage('            - Processed all ' + str(Number_Processed), 4)
del cursor

if renumber:
    if Duplicate_Counter > 0:

        Number_Processed = 0
        CPMessage('        - Renumbering ' + str(Duplicate_Counter) + ' duplicate/null TEIS_ID value(s); updated value(s) will start at ' + str(Highest_TEIS_ID + 1), 4)
        dtCalcStart = time.time()

        with arcpy.da.UpdateCursor(INPUT_FC, Cursor_Field_List) as cursor:

            for row in cursor:

                Number_Processed = Number_Processed + 1

                if row[0] == None:
                    Highest_TEIS_ID += 1
                    row[0] = Highest_TEIS_ID
                    cursor.updateRow(row)

                if (0 < Number_Processed <= 10000 and Number_Processed % 1000 == 0) or (10000 < Number_Processed <= 100000 and Number_Processed % 10000 == 0) or (100000 < Number_Processed and Number_Processed % 100000 == 0):
                    dtCalcNow = time.time()
                    dtCalcElapsed = dtCalcNow - dtCalcStart
                    dtCalcRemaining = (dtCalcElapsed / Number_Processed) * (Number_of_Features - Number_Processed)
                    CPMessage('            - Processed ' + str(Number_Processed) + ' of ' + str(Number_of_Features) + ' (' + SanitizeElapsedTime(dtCalcElapsed)[:-1] + ' elapsed, ' + SanitizeElapsedTime(dtCalcRemaining)[:-1] + ' remaining)', 4)
        CPMessage('            - Processed all ' + str(Number_Processed), 4)
        del cursor

    else:
        CPMessage('        - No duplicate or null TEIS_ID values found.', 4)
    
elif not renumber:
    if len(Duplicate_List) == 0:
        CPMessage('        - No duplicate or null TEIS_ID values found.', 4)
    else:
        CPMessage('    **** Duplicate and/or null TEIS_ID values found. Listing up to the first ten duplicate/null values found: ' + str(Duplicate_List).replace('[','').replace(']','').replace(' ','').replace('None','<Null>'), 0)'''

    exec(exec_string)   ## code for Arc 10.1 cursors needs to be wrapped in a string this way because the word "with" trips up the Python 2.6 (Arc 9.3.1) preliminary syntax checker

    return(Processing_Variables['PROCESS_STATUS'])

#*********************************************************************************************************
#  Main Program
#*********************************************************************************************************

#---------------------------------------------------------------------------------------------------------
# Run Main for ArcGIS 9.3 and 10.0
#
#   Main Method manages the overall program
#---------------------------------------------------------------------------------------------------------
def runMain93():

    Initialize93()
    SetupWorkspace93()
    CheckUniqueTEISID93()

    return Processing_Variables['PROCESS_STATUS']

#---------------------------------------------------------------------------------------------------------
# Run Main for ArcGIS 10.1
#
#   Main Method manages the overall program
#---------------------------------------------------------------------------------------------------------
def runMain101():

    Initialize101()
    SetupWorkspace101()
    CheckUniqueTEISID101()
    
    return Processing_Variables['PROCESS_STATUS']

#*********************************************************************************************************
#  Execution
#*********************************************************************************************************

d = arcpy.GetInstallInfo('desktop')
if len(d['Version'].split('.')) > 2:
    dv = d['Version'].rsplit('.',len(d['Version'].split('.'))-2)[0]
    del arcpy
    import arcpy
    runMain101()
else:
    dv = d['Version']
    if float(dv) <10.1:
        runMain93()
    else:
        print '** Unsupported ArcGIS version (' + str(d['Version']) + ') running on this machine.'
#---------------------------------------------------------------------------------------------------------
#  Clean-up
#---------------------------------------------------------------------------------------------------------


END_TIME       = time.ctime(time.time())
END_TIME_SEC   = time.time()
END_TIME_SQL   = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
EXECUTION_TIME = END_TIME_SEC - START_TIME_SEC
if EXECUTION_TIME < 120:
    EXECUTION_TIME_STRING = str(int(EXECUTION_TIME)) + ' seconds.'
elif EXECUTION_TIME < 3600:
    EXECUTION_TIME_STRING = str(round(EXECUTION_TIME / 60, 1)) + ' minutes.'
else:
    EXECUTION_TIME_STRING = str(round(EXECUTION_TIME / 3600, 2)) + ' hours.'


CPMessage('    End Time: ' + END_TIME, 2)
CPMessage('    Execution Time: ' + EXECUTION_TIME_STRING, 4)
CPMessage('Complete: ' + PYTHON_SCRIPT, 1)
CPMessage('Next Script: StandardizeNoData',2)
CPMessage('',4)
CPMessage ('Preconditions for next step: None',4)

if 'SendRunLogToEmail' in Processing_Variables:
    for Email in Processing_Variables['SendRunLogToEmail']:
        SendRunLogToEmail(Email)

#*********************************************************************************************************
#  End of Program
#*********************************************************************************************************

