#=========================================================================================================
#                                      CreateShortTable
#=========================================================================================================
#
# Author(s)         :  Clover Point Cartographics (Mike Neal)
#
# Date              :  2009-04-08
#
# Purpose           :  This script creates the detailed polygons feature class with geometry identical to
#                      the Long Table feature class, but using logic embedded in the code this script
#                      concatenates the Long Table's mostly one- and two- character detailed attributes
#                      into longer standard map label strings which ecologists are most accustomed to
#                      reading.
#
# Arguments         :  argv(1) = INPUT_MLT  (Long Table feature class used to create Short Table)
#                      argv(2) = INPUT_PD   (Project Details table - attributes used to create Short Table)
#
# Example Input     :  X:\fullpath\CreateShortTable.py X:\fullpath\TEI_Long_Tbl X:\fullpath\TEI_Project_Details
# Example Input     :  X:\fullpath\CreateShortTable.py X:\fullpath\TEIS_Master_Long_Tbl X:\fullpath\BAPID_Tracker_BE.mdb\tblProjects
#
# Outputs           :  TEI_Short_Tbl feature class in the Operational_Data.gdb file geodatabase
#
# Dependencies      :  None
# 
# Expected Duration :  < 2 hours for approximately 100,000 records
#                      30 hours for 24 million records
#
# History           :  2012-09-17 (JK): Rewrote to use cursors to process one row at a time (read one long
#                                       table row, process it and insert a new row to the short table).
#                                       Detects if current machine uses ArcGIS 10.1 and if so, uses the new
#                                       Data Access Module cursor object.
#
#                   :  2012-10-12 (JK): Update to CPMessage routine: only append messages to PROCESS_LOG
#                                       if the e-mail function is being used (otherwise it can fill memory).
#                                       Added code to read Domain_Project_Type table and record the TER_Label_Type
#                                       field values to decide type of Terrain Label based on Project Type.
#                                       Replaced all "gp" references to "arcpy" to save having to test
#                                       current Arc version during every CPMessage call.
#
#                   :  2012-10-15 (JK): Remove stra() function on POLY_COM field values, which may contain
#                                       unicode characters which cause error on stra().
#
#                   :  2012-10-16 (JK): Bug fix: need to use stra() function when reading text fields from
#                                       Project Details and Project Tracker tables, which may have null values
#
#                   :  2013-01-02 (JK): Modified to read all project details attributes from the specified
#                                       Project Details table (which may be a TEI_Project_Details FGDB table
#                                       or the BAPID Tracker table), as the Project Tracker table is now gone.
#
#                   :  2013-01-30 (JK): Added code to check that all BAPIDs in the long table feature class
#                                       are present in the project details table.
#
#                   :  2013-04-23 (JK): At Deepa's request, removed SDEC_# from the construction of SEILBL_# 
#                                       values
#
#                   :  2013-05-23 (JK): Update "Example Input" comments above (ODBC connection to BAPID Tracker
#                                       mdb file not required; you can specify the Windows path to it).
#
#                   :  2013-06-19 (JK): Added examples to message explaining script usage (displays if
#                                       incorrect number of arguments are provided). Added a check for whether
#                                       the datasets provided as arguments actually exist.
#
#=========================================================================================================
#=========================================================================================================
#  Process Status and Return Codes:
#      -1   - Unexpected error
#       0   - Completed Successful Run
#       1   - In progress
#       100 - Invalid Arguments
#       101 - ESRI License Error
#
#=========================================================================================================

#*********************************************************************************************************
#  Program Initialization
#*********************************************************************************************************

#---------------------------------------------------------------------------------------------------------
#  Python and System Modules
#---------------------------------------------------------------------------------------------------------

from __future__ import division

import sys
import string
import os
import time
import random
import pdb

#---------------------------------------------------------------------------------------------------------
#  Arguement Validation
#---------------------------------------------------------------------------------------------------------

# Capture/Validate Parameters
#  Test that the right number of paramters were provided and validated
#  Note that 1 must be added because the Python script is an arguement (0) supplied to the
#  Python Environment.

# If long text parameters are required, use:  any_value = arcpy.GetParameterAsText(0)

if len(sys.argv) < 3 or len(sys.argv) > 3 :

    print "\nInvalid number of arguments (" + str(len(sys.argv) - 1) + " supplied) to:"
    print "    " + sys.argv[0] + "\n"
    print " Usage:   " + sys.argv[0] + " [full path to long table feature class] [full path to tblProjects or TEI_Project_Details table]"
    print " Example 1:   " + sys.argv[0] + r" X:\fullpath\Operational_Data.gdb\TEIS_Master_Long_Tbl Y:\fullpath\BAPID_Tracker_BE.mdb\tblProjects"
    print " Example 2:   " + sys.argv[0] + r" X:\fullpath\Operational_Data.gdb\TEI_Long_Tbl X:\fullpath\Operational_Data.gdb\TEI_Project_Details"

    sys.exit(100)

PYTHON_SCRIPT     = sys.argv[0]
INPUT_MLT         = sys.argv[1]
INPUT_PD          = sys.argv[2]

#---------------------------------------------------------------------------------------------------------
#  Standard/Global Variables/Constants
#---------------------------------------------------------------------------------------------------------

# Global Constants
COMPUTERNAME     = os.environ['COMPUTERNAME']
USERNAME         = os.environ['USERNAME']
START_TIME       = time.ctime(time.time())
START_TIME_SEC   = time.time()
START_TIME_SQL   = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
BEEP             = chr(7)


# Global Variables
#   The Processing_Variables is the collection of messaging and script specific information needed
#   throughout this program.  It is constantly updated, and generally includes all paths and configuration
#   information.  For parameters that are passed in to the script, they should generally be assigned
#   to a Processing_Variable through the Initialize() routine.

Processing_Variables = {}



#---------------------------------------------------------------------------------------------------------
#  CPC Variables/Constants
#---------------------------------------------------------------------------------------------------------




#---------------------------------------------------------------------------------------------------------
#  Set up the ArcGIS Geoprocessing Environment
#---------------------------------------------------------------------------------------------------------

try:
    import arcgisscripting
    arcpy = arcgisscripting.create(93)
except:
    print '**** ArcGIS Failed to initialize a GeoProcessor Object'
    sys.exit(101)

d = arcpy.GetInstallInfo('desktop')
if d['Version'][:4] == '10.1':
    del arcpy
    import arcpy

#---------------------------------------------------------------------------------------------------------
#  Script Specific Variables/Constants
#---------------------------------------------------------------------------------------------------------

#### BAPID_LIST = '1060;4343'
BAPID_LIST = 'ALL'


#*********************************************************************************************************
#  Routines
#*********************************************************************************************************

#---------------------------------------------------------------------------------------------------------
# TemplateRoutine(Input_FC)
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def TemplateRoutine(Input_FC):
    CPMessage('    - Template Routine', 2)
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "TemplateRoutine" 

    result = arcpy.GetCount_management(Input_FC)
    Number_of_Features = int(result.GetOutput(0))
    Number_Processed = 0
    
    CPMessage('        - Calculating individual values for' + str(Number_of_Features) + ' features.', 4)
    rows = arcpy.SearchCursor(Input_FC)
    row = rows.Next()
    
    while row:
        if Number_Processed % 1000 == 0:
            CPMessage('            - Processed ' + str(Number_Processed), 4)

        Number_Processed = Number_Processed + 1
        row = rows.Next()
    del row
    del rows

    return(Processing_Variables['PROCESS_STATUS'])

#=========================================================================================================
#  Standard Routines
#=========================================================================================================

#---------------------------------------------------------------------------------------------------------
# stra(value)
#
#       Converts any value to a string. Replaces Python's stra() which incurs an error if the value
#       happens to have a unicode character in it. Unicode characters will be replaced with ?.
#
#---------------------------------------------------------------------------------------------------------
def stra(value):
    return unicode(value).encode('ascii','replace')

#---------------------------------------------------------------------------------------------------------
# Initialize93()
#
#       Set up all of the paths and variables that we need to run this script.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def Initialize93():

    print '    - Initializing Script'
    
    Processing_Variables['Script_Directory_Path'] = os.path.split(PYTHON_SCRIPT)[0]
    Processing_Variables['Script_Name'] = os.path.split(PYTHON_SCRIPT)[1]
    Processing_Variables['Log_Directory_Path'] = Processing_Variables['Script_Directory_Path'] + r'\log'

    Processing_Variables['Project_Base_Path']      = os.path.split(Processing_Variables['Script_Directory_Path'])[0]

    
    Processing_Variables['ToolBoxRun'] = 'FALSE'
    # Processing_Variables['SendRunLogToEmail'] = ['mneal@cloverpoint.com', 'chaos_mike@hotmail.com']
    Processing_Variables['Log_Level'] = 4
    Processing_Variables['PROCESS_INFO']   = "RUNNING " + PYTHON_SCRIPT
    Processing_Variables['PROCESS_STATUS'] = 1
    Processing_Variables['PROCESS_LOG'] = []
    Processing_Variables['MessageQueueFile']   = Processing_Variables['Log_Directory_Path'] + '\\' + Processing_Variables['Script_Name'] + '.' + COMPUTERNAME + '.log'

    #### Adding a FGDB Log will significantly slow down performance.
    # Processing_Variables['Log_FGDB_Table'] = Processing_Variables['Project_Base_Path'] + r"\Operational.gdb\Processing_Log"

    if not arcpy.Exists(Processing_Variables['Log_Directory_Path']):
        os.mkdir(Processing_Variables['Log_Directory_Path'])
        CPMessage(r'Created \Scripts\log folder', 1)

    CPMessage('Executing: ' + PYTHON_SCRIPT, 1)
    CPMessage('    Start Time: ' + START_TIME, 2)
    CPMessage('    Python ' + sys.version, 4)

    # Load required toolboxes...
    # arcpy.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Data Management Tools.tbx")
    
    # Set the Geoprocessing environment...
    arcpy.OverwriteOutput = 1
    # arcpy.cellSize = "MINOF"
    # arcpy.workspace = INPUT_LOCATION
        
    
    # CPC Naming Standard: {variable}_Path is the fully qualified path, otherwise it is just the name of the item.
  
    # Processing Variables
   
    # Processing_Variables['OperationalFGDB_Path']   = Processing_Variables['Project_Base_Path'] + r'\Operational_Data.gdb'
    Processing_Variables['OperationalFGDB_Path']   = os.path.split(INPUT_MLT)[0]
    Processing_Variables['Scratch_Path']           = Processing_Variables['Project_Base_Path'] + r'\Scratch'
    Processing_Variables['ScratchFGDB_Path']           = Processing_Variables['Scratch_Path'] + r'\Scratch.gdb'
    arcpy.scratchworkspace = Processing_Variables['ScratchFGDB_Path']
    
    Processing_Variables['Tools_and_Templates_FGDB_Path']   = Processing_Variables['Project_Base_Path'] + r'\Tools_and_Templates.gdb'
    
    # Processing_Variables['TEIS_Project_Details']   = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Project_Details'
    Processing_Variables['Project_Details']   = INPUT_PD
    Processing_Variables['Long_Tbl']   = INPUT_MLT
    Processing_Variables['SummaryErrorTable']   = INPUT_MLT + '_SummaryErrors'
    Processing_Variables['Project_Boundaries']   = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Project_Boundaries'

    Processing_Variables['Short_Tbl']   = INPUT_MLT.replace('Long','Short')
    if Processing_Variables['Short_Tbl'] == Processing_Variables['Long_Tbl']:
        Processing_Variables['Short_Tbl']    = INPUT_MLT + "_Short"

    Processing_Variables['Template_Short_Tbl']   = Processing_Variables['Tools_and_Templates_FGDB_Path'] + r'\Template_FGDB_TEIS_Master_Short_Tbl'
  
    Processing_Variables['BAPID_LIST'] = []
    
    for BAPID in BAPID_LIST.split(';'):
        Processing_Variables['BAPID_LIST'].append(BAPID)
    
    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# Initialize101()
#
#       Set up all of the paths and variables that we need to run this script.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def Initialize101():

    print '    - Initializing Script'
    
    Processing_Variables['Script_Directory_Path'] = os.path.split(PYTHON_SCRIPT)[0]
    Processing_Variables['Script_Name'] = os.path.split(PYTHON_SCRIPT)[1]
    Processing_Variables['Log_Directory_Path'] = Processing_Variables['Script_Directory_Path'] + r'\log'

    Processing_Variables['Project_Base_Path']      = os.path.split(Processing_Variables['Script_Directory_Path'])[0]

    
    Processing_Variables['ToolBoxRun'] = 'FALSE'
    # Processing_Variables['SendRunLogToEmail'] = ['mneal@cloverpoint.com', 'chaos_mike@hotmail.com']
    Processing_Variables['Log_Level'] = 4
    Processing_Variables['PROCESS_INFO']   = "RUNNING " + PYTHON_SCRIPT
    Processing_Variables['PROCESS_STATUS'] = 1
    Processing_Variables['PROCESS_LOG'] = []
    Processing_Variables['MessageQueueFile']   = Processing_Variables['Log_Directory_Path'] + '\\' + Processing_Variables['Script_Name'] + '.' + COMPUTERNAME + '.log'

    #### Adding a FGDB Log will significantly slow down performance.
    # Processing_Variables['Log_FGDB_Table'] = Processing_Variables['Project_Base_Path'] + r"\Operational.gdb\Processing_Log"

    if not arcpy.Exists(Processing_Variables['Log_Directory_Path']):
        os.mkdir(Processing_Variables['Log_Directory_Path'])
        CPMessage(r'Created \Scripts\log folder', 1)

    CPMessage('Executing: ' + PYTHON_SCRIPT, 1)
    CPMessage('    Start Time: ' + START_TIME, 2)
    CPMessage('    Python ' + sys.version, 4)

    # Load required toolboxes...
    # arcpy.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Data Management Tools.tbx")
    
    # Set the Geoprocessing environment...
    arcpy.OverwriteOutput = 1
    # arcpy.cellSize = "MINOF"
    # arcpy.workspace = INPUT_LOCATION
        
    
    # CPC Naming Standard: {variable}_Path is the fully qualified path, otherwise it is just the name of the item.
  
    # Processing Variables
   
    # Processing_Variables['OperationalFGDB_Path']   = Processing_Variables['Project_Base_Path'] + r'\Operational_Data.gdb'
    Processing_Variables['OperationalFGDB_Path']   = os.path.split(INPUT_MLT)[0]
    Processing_Variables['Scratch_Path']           = Processing_Variables['Project_Base_Path'] + r'\Scratch'
    Processing_Variables['ScratchFGDB_Path']           = Processing_Variables['Scratch_Path'] + r'\Scratch.gdb'
    arcpy.env.scratchWorkspace = Processing_Variables['ScratchFGDB_Path']
    
    Processing_Variables['Tools_and_Templates_FGDB_Path']   = Processing_Variables['Project_Base_Path'] + r'\Tools_and_Templates.gdb'
    
    # Processing_Variables['TEIS_Project_Details']   = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Project_Details'
    Processing_Variables['Project_Details']   = INPUT_PD
    Processing_Variables['Long_Tbl']   = INPUT_MLT
    Processing_Variables['SummaryErrorTable']   = INPUT_MLT + '_SummaryErrors'
    Processing_Variables['Project_Boundaries']   = Processing_Variables['OperationalFGDB_Path'] + r'\TEIS_Project_Boundaries'

    Processing_Variables['Short_Tbl']   = INPUT_MLT.replace('Long','Short')
    if Processing_Variables['Short_Tbl'] == Processing_Variables['Long_Tbl']:
        Processing_Variables['Short_Tbl']    = INPUT_MLT + "_Short"

    Processing_Variables['Template_Short_Tbl']   = Processing_Variables['Tools_and_Templates_FGDB_Path'] + r'\Template_FGDB_TEIS_Master_Short_Tbl'
  
    Processing_Variables['BAPID_LIST'] = []
    
    for BAPID in BAPID_LIST.split(';'):
        Processing_Variables['BAPID_LIST'].append(BAPID)
    
    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# CPMessage(Message)
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def CPMessage(Message, MessageLevel):
    # Based on the logging level, add messages to the message queues.
    #   Text Run Log   : Implimented
    #   Console        : Implimented
    #   GeoProcessor   : Implimented
    #   Processing Log : Implimented (with e-mail)
    #   FGDB           : Implimented
    
    # Message Levels:
    #   0    : Fatal Errors
    #   1    : Top Level Program
    #   2    : Main routines and events
    #   3    : Warnings
    #   4    : General Information
    #   5    : Debug level detail
    
    # Send to the screen
    if MessageLevel <= Processing_Variables['Log_Level']:
        WriteOutputToScreen(Message, MessageLevel)    
    
    # Send to the Processing Log
    if 'SendRunLogToEmail' in Processing_Variables:
        Processing_Variables['PROCESS_LOG'].append((Message, MessageLevel))
    
    # Send to the Log File
    if 'MessageQueueFile' in Processing_Variables and MessageLevel <= Processing_Variables['Log_Level']:
        WriteOutputToFile(Message, Processing_Variables['MessageQueueFile'])

    # Send to the GeoProcessor Message Queue
    #### At ArcGIS 9.3 SP1 there is a bug that also causes any GeoProcessor messages to be echoed to the console.
    #    Added the ToolBoxRun parameter to allow the script to add messages to the GeoProcessor Messaging properly
    if MessageLevel < 4 and MessageLevel <= Processing_Variables['Log_Level'] and Processing_Variables['ToolBoxRun'] == 'TRUE':
        if MessageLevel == 0:
            arcpy.AddError(Message)
        if MessageLevel == 1 or MessageLevel == 2:
            arcpy.AddMessage(Message)
        if MessageLevel == 3:
            arcpy.AddWarning(Message)

    # Send to the FGDB Processing Log
    if MessageLevel < 4 and MessageLevel <= Processing_Variables['Log_Level'] and 'Log_FGDB_Table' in Processing_Variables:
        rows = arcpy.InsertCursor(Processing_Variables['Log_FGDB_Table'])

        row = rows.NewRow()
        row.SetValue("Computer_Name", COMPUTERNAME)
        row.SetValue("User_Name", USERNAME)
        row.SetValue("Script_Name", PYTHON_SCRIPT)
        row.SetValue("Event_Time", time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
        row.SetValue("Message_Level", MessageLevel)
        row.SetValue("Message", Message)
        row.SetValue("Start_Time", START_TIME_SQL)
        
        rows.InsertRow(row)
        del row
        del rows
    

    return(1)


#---------------------------------------------------------------------------------------------------------
# WriteOutputToScreen(Output_Comment, Header_Style):
#
#   Prints the Output_Comment to the screen with the appropriate formatting
#       0 : "******" Used for Error Sections
#       1 : "======" Used for the Program's Header and Footer
#       2 : "------" Used for New Sections
#       3+:          No delimitation
#   
#---------------------------------------------------------------------------------------------------------
def WriteOutputToScreen(Output_Comment, Header_Style):
    if Header_Style == 0:
        # Errors get beeps, too!
        print BEEP * 3 + "\n" * 2 + "*" * 79 + "\n" * 2
    if Header_Style == 2:
        print "\n" + "-" * 79 + "\n"
    if Header_Style == 1:
        print "\n" + "=" * 79 + "\n"
        
    #print Output_Comment
    arcpy.AddMessage(Output_Comment)
    
    if Header_Style == 0:
        print "\n" * 2 + "*" * 79 + "\n" * 2
    if Header_Style == 1:
        print "\n" + "=" * 79 + "\n"


#---------------------------------------------------------------------------------------------------------
# WriteOutputToFile(Output_Comment, Run_Log)
#   Record to Log 
#
#
#---------------------------------------------------------------------------------------------------------
def WriteOutputToFile(Output_Comment, Run_Log):
# Run logs are not compatible with concurrency - only one process is expected in debug mode at a time!
# First try to append to an existing file - if that fails create a new file.
    ReturnValue = 1
    try:
        if os.path.isfile(Run_Log) == 1:
            # try to open up the fileName 
            file = open(Run_Log,'a') 

            # write String to the fileName
            file.write(COMPUTERNAME + " " + START_TIME_SQL + ' (' + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()) + ') : ' + Output_Comment + '\n')

            # Close the file
            file.close()
            Processing_Variables['PROCESS_STATUS'] = 1
            Processing_Variables['PROCESS_INFO']   = "Appended comment to the file"
        else:
            # try to open up the fileName 
            file = open(Run_Log,'w') 

            # write String to the fileName
            file.write(COMPUTERNAME + " " + START_TIME_SQL + ' (' + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()) + ') : ' + Output_Comment + '\n')

            # Close the file
            file.close()
            Processing_Variables['PROCESS_STATUS'] = 1
            Processing_Variables['PROCESS_INFO']   = "Created new file and put in the comment"
    except:
        # All failures are unexpected in this case...  
        # Maybe an invalid drive name, read only file or something?
        Processing_Variables['PROCESS_STATUS'] = -1
        Processing_Variables['PROCESS_INFO']   = "**ERROR: Can't write to file: " + Run_Log   
        print BEEP + BEEP + BEEP
        ReturnValue = -1
    return(ReturnValue)
        

#---------------------------------------------------------------------------------------------------------
# SetupWorkspace93()
#
#       Test if we need to create the workspace file geodatabases, and then do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def SetupWorkspace93():
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "SetupWorkspace93"  

    CPMessage('    - Setting Up Workspace', 2)
    
    if not arcpy.Exists(INPUT_MLT):
        CPMessage(' **** Specified Master Long Table feature class does not exist: ' + INPUT_MLT, 0)
        sys.exit(100)

    if not arcpy.Exists(INPUT_PD):
        CPMessage(' **** Specified project details table does not exist: ' + INPUT_PD, 0)
        sys.exit(100)

    if not arcpy.Exists(Processing_Variables['ScratchFGDB_Path']):
        CPMessage('        -- Creating Scratch FGDB', 4)
        try:
            arcpy.CreateFileGDB_management(os.path.split(Processing_Variables['ScratchFGDB_Path'])[0], os.path.split(Processing_Variables['ScratchFGDB_Path'])[1])
            Processing_Variables['PROCESS_STATUS'] = 1
            Processing_Variables['PROCESS_INFO']   = "Created Scratch FGDB"
        except:
            Processing_Variables['PROCESS_STATUS'] = -1
            Processing_Variables['PROCESS_INFO']   = "Error Creating Scratch FGDB"
            CPMessage('** Can not find or create ' + Processing_Variables['ScratchFGDB_Path'], 0)
            sys.exit(100)
    else:
        CPMessage('        -- Scratch FGDB exists, nothing to do.', 4)
       
    if arcpy.Exists(Processing_Variables['OperationalFGDB_Path']):
        CPMessage('        -- Operational FGDB exists.', 4)
    else:
        CPMessage(' **** Operational FGDB does not exist: ' + Processing_Variables['OperationalFGDB_Path'], 0)
        sys.exit(100)

    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path']):
        CPMessage('        -- Tools and Templates FGDB exists.', 4)
    else:
        CPMessage(' **** Tools and Templates FGDB does not exist: ' + Processing_Variables['Tools_and_Templates_FGDB_Path'], 0)
        sys.exit(100)

    if arcpy.Exists(Processing_Variables['Short_Tbl']):
        CPMessage('        -- Short Table exists. Deleting.', 4)
        while not arcpy.TestSchemaLock(Processing_Variables['Short_Tbl']):
            CPMessage('** Schema Locked - waiting 10 seconds: ' + Processing_Variables['Short_Tbl'], 3)
            time.sleep(10)        
        arcpy.Delete_management(Processing_Variables['Short_Tbl'])
    CPMessage('        -- Copying short table template to new feature class.', 4)
    arcpy.Copy_management(Processing_Variables['Template_Short_Tbl'], Processing_Variables['Short_Tbl'])

    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# SetupWorkspace101()
#
#       Test if we need to create the workspace file geodatabases, and then do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def SetupWorkspace101():
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "SetupWorkspace101"  

    CPMessage('    - Setting Up Workspace', 2)
    
    if not arcpy.Exists(INPUT_MLT):
        CPMessage(' **** Specified Master Long Table feature class does not exist: ' + INPUT_MLT, 0)
        sys.exit(100)

    if not arcpy.Exists(INPUT_PD):
        CPMessage(' **** Specified project details table does not exist: ' + INPUT_PD, 0)
        sys.exit(100)

    if not arcpy.Exists(Processing_Variables['ScratchFGDB_Path']):
        CPMessage('        -- Creating Scratch FGDB', 4)
        try:
            arcpy.CreateFileGDB_management(os.path.split(Processing_Variables['ScratchFGDB_Path'])[0], os.path.split(Processing_Variables['ScratchFGDB_Path'])[1])
            Processing_Variables['PROCESS_STATUS'] = 1
            Processing_Variables['PROCESS_INFO']   = "Created Scratch FGDB"
        except:
            Processing_Variables['PROCESS_STATUS'] = -1
            Processing_Variables['PROCESS_INFO']   = "Error Creating Scratch FGDB"
            CPMessage('** Can not find or create ' + Processing_Variables['ScratchFGDB_Path'], 0)
            sys.exit(100)
    else:
        CPMessage('        -- Scratch FGDB exists, nothing to do.', 4)
       
    if arcpy.Exists(Processing_Variables['OperationalFGDB_Path']):
        CPMessage('        -- Operational FGDB exists.', 4)
    else:
        CPMessage(' **** Operational FGDB does not exist: ' + Processing_Variables['OperationalFGDB_Path'], 0)
        sys.exit(100)

    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path']):
        CPMessage('        -- Tools and Templates FGDB exists.', 4)
    else:
        CPMessage(' **** Tools and Templates FGDB does not exist: ' + Processing_Variables['Tools_and_Templates_FGDB_Path'], 0)
        sys.exit(100)

    if arcpy.Exists(Processing_Variables['Short_Tbl']):
        CPMessage('        -- Short Table exists. Deleting.', 4)
        while not arcpy.TestSchemaLock(Processing_Variables['Short_Tbl']):
            CPMessage('** Schema Locked - waiting 10 seconds: ' + Processing_Variables['Short_Tbl'], 3)
            time.sleep(10)        
        arcpy.Delete_management(Processing_Variables['Short_Tbl'])
    CPMessage('        -- Copying short table template to new feature class.', 4)
    arcpy.Copy_management(Processing_Variables['Template_Short_Tbl'], Processing_Variables['Short_Tbl'])

    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# SendRunLogToEmail()
#
#       Test if we need to create the workspace file geodatabases, and then do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def SendRunLogToEmail(mailto):

    import smtplib
    CPMessage('  * Sending log to ' + mailto, 2)
    mailfrom = 'ssmith@cloverpoint.com'
    # mailto = 'ssmith@cloverpoint.com'
    mailsubject = 'Run Log: ' + os.path.split(PYTHON_SCRIPT)[1] + ' (' + COMPUTERNAME + '\\' + USERNAME + ')'
    mailbody = "This is an automatic run log.\n\n"
    
    for LogEntry in Processing_Variables['PROCESS_LOG']:
        mailbody = mailbody + '\n' + LogEntry[0]

    mailmsg = ("From: " + mailfrom + "\r\nTo: " + mailto + "\r\nSubject: " + mailsubject + "\r\n" + mailbody)

    mailserver = smtplib.SMTP('shawmail.gv.shawcable.net')

    # SMTP.login(user, password)

    mailserver.sendmail(mailfrom, mailto, mailmsg)
    mailserver.quit()
    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# SanitizeElapsedTime(number_of_seconds)
#
# routine to convert a raw number of seconds into a human-friendly statement of duration
#---------------------------------------------------------------------------------------------------------

def SanitizeElapsedTime(dtInput):

    if dtInput < 120:
        strElapsedTime = stra(int(dtInput)) + ' seconds.'
        
    elif dtInput < 3600:
        strElapsedTime = stra(int(dtInput)) + ' seconds (' + stra(round(dtInput / 60, 1)) + ' minutes).'
        
    else:
        strElapsedTime = stra(int(dtInput)) + ' seconds (' + stra(round(dtInput / 3600, 2)) + ' hours).'
        
    return strElapsedTime



#=========================================================================================================
#  Special Routines
#=========================================================================================================

#---------------------------------------------------------------------------------------------------------
# PopulateTEISShortTable93()
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def PopulateTEISShortTable93():

    CPMessage('        - Populate TEIS Short Table', 2)
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "PopulateTEISShortTable93" 

    ProjDetails_Dictionary = {}
    result = arcpy.GetCount_management(Processing_Variables['Project_Details'])
    Number_of_Features = int(result.GetOutput(0))
    Number_Processed = 0
    
    CPMessage('            - Reading Project Details table for ' + stra(Number_of_Features) + ' items.', 4)
    
    rows = arcpy.SearchCursor(Processing_Variables['Project_Details'])
    row = rows.Next()
    while row:
        BAPID = row.GetValue('BAPID')
        MOD_DATE = row.GetValue('MOD_DATE')
        PROJ_TYPE = stra(row.GetValue('PROJ_TYPE')).rstrip(' ').replace('None','')
        PROJ_SCALE = row.GetValue('PROJ_SCALE')
        PROJ_NAME = stra(row.GetValue('PROJ_NAME')).rstrip(' ').replace('None','')
        LEGEND_COM = stra(row.GetValue('LEGEND_COM')).rstrip(' ').replace('None','')
        LEGEND_URL = stra(row.GetValue('LEGEND_URL')).rstrip(' ').replace('None','')
        ProjDetails_Dictionary[BAPID] = {'MOD_DATE': MOD_DATE, 'PROJ_TYPE': PROJ_TYPE, 'PROJ_SCALE': PROJ_SCALE, 'PROJ_NAME': PROJ_NAME, 'LEGEND_COM': LEGEND_COM, 'LEGEND_URL': LEGEND_URL}

        Number_Processed = Number_Processed + 1
        row = rows.Next()

    CPMessage('            - Reading BAPID numbers in Master Long Table.', 4)
    row_count = 0
    bapid_dict = {}
    bapid_not_found_dict = {}
    rows = arcpy.SearchCursor(Processing_Variables['Long_Tbl'])
    row = rows.Next()
    while row:
        row_count += 1
        BAPID = row.GetValue('BAPID')
        try:
            ProjDetails_Dictionary[BAPID]
        except:
            try:
                bapid_not_found_dict[BAPID]
            except:
                bapid_not_found_dict[BAPID] = True
        try:
            bapid_dict[BAPID]
        except:
            bapid_dict[BAPID] = True
        if row_count % 1000000 == 0:
            CPMessage('                - Read ' + stra(row_count) + ' rows (' + stra(len(bapid_dict.keys())) + ' unique BAPID(s) found)', 4)
        row = rows.Next()
    if len(bapid_not_found_dict.keys()) > 0:
        bapids_not_found_string = ''
        bapid_not_found_list = bapid_not_found_dict.keys()
        bapid_not_found_list.sort()
        for bapid in bapid_not_found_list:
            bapids_not_found_string += stra(bapid) + ', '
        bapids_not_found_string = bapids_not_found_string[:-2]
        CPMessage('    ***** BAPIDs not found in BAPID Tracker/Project Details table: ' + bapids_not_found_string, 0)
        sys.exit()
    CPMessage('                - Read ' + stra(row_count) + ' rows. All ' + stra(len(bapid_dict.keys())) + ' unique BAPIDs found in BAPID Tracker/Project Details table.', 4)

    ErrorSum_Dictionary = {}
    if arcpy.Exists(Processing_Variables['SummaryErrorTable']):
        result = arcpy.GetCount_management(Processing_Variables['SummaryErrorTable'])
        Number_of_Features = int(result.GetOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Error Summary table for ' + stra(Number_of_Features) + ' items.', 4)

        rows = arcpy.SearchCursor(Processing_Variables['SummaryErrorTable'])
        row = rows.Next()
        while row:
            if Number_Processed % 10000 == 0:
                CPMessage('                - Read ' + stra(Number_Processed) + ' of ' + stra(Number_of_Features), 4)
            FEATURE_ID = row.GetValue('FEATURE_ID')
            PROBLEMS = stra(row.GetValue('PROBLEMS')).rstrip(' ').replace('None','')
            ErrorSum_Dictionary[FEATURE_ID] = PROBLEMS

            Number_Processed = Number_Processed + 1
            row = rows.Next()
        CPMessage('                - Read ' + stra(Number_Processed) + ' of ' + stra(Number_of_Features), 4)


    DPT_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Project_Type"):
        result = arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Project_Type")
        Number_of_Features = int(result.GetOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Project Type domain table for ' + stra(Number_of_Features) + ' items.', 4)

        rows = arcpy.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Project_Type")
        row = rows.Next()
        while row:
            CODE = stra(row.GetValue('Code')).rstrip(' ').replace('None','')
            TER_Label_Type = stra(row.GetValue('TER_Label_Type')).rstrip(' ').replace('None','')
            DPT_Dictionary[CODE] = TER_Label_Type

            Number_Processed = Number_Processed + 1
            row = rows.Next()

    SSC_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class_with_Roads_Qualifier_Flag"):
        result = arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class_with_Roads_Qualifier_Flag")
        Number_of_Features = int(result.GetOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Slope Stability Class domain table for ' + stra(Number_of_Features) + ' items.', 4)

        rows = arcpy.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class_with_Roads_Qualifier_Flag")
        row = rows.Next()
        while row:
            CODE = stra(row.GetValue('Code')).rstrip(' ').replace('None','')
            SSC_TXT = stra(row.GetValue('SSC_TXT')).rstrip(' ').replace('None','')
            SSC_Dictionary[CODE] = SSC_TXT

            Number_Processed = Number_Processed + 1
            row = rows.Next()

    SEP_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Potential"):
        result = arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Potential")
        Number_of_Features = int(result.GetOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Surface Erosion Potential domain table for ' + stra(Number_of_Features) + ' items.', 4)

        rows = arcpy.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Potential")
        row = rows.Next()
        while row:
            CODE = stra(row.GetValue('Code')).rstrip(' ').replace('None','')
            SEP_TXT = stra(row.GetValue('SEP_TXT')).rstrip(' ').replace('None','')
            SEP_Dictionary[CODE] = SEP_TXT

            Number_Processed = Number_Processed + 1
            row = rows.Next()

    LISSC_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Landslide_Induced_Stream_Sedimentation"):
        result = arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Landslide_Induced_Stream_Sedimentation")
        Number_of_Features = int(result.GetOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Landslide Induced Stream Sedimentation domain table for ' + stra(Number_of_Features) + ' items.', 4)

        rows = arcpy.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Landslide_Induced_Stream_Sedimentation")
        row = rows.Next()
        while row:
            CODE = stra(row.GetValue('Code')).rstrip(' ').replace('None','')
            LISSC_TXT = stra(row.GetValue('LISSC_TXT')).rstrip(' ').replace('None','')
            LISSC_Dictionary[CODE] = LISSC_TXT

            Number_Processed = Number_Processed + 1
            row = rows.Next()

    PSTBCLS_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class"):
        result = arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class")
        Number_of_Features = int(result.GetOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Slope Stability Class domain table for ' + stra(Number_of_Features) + ' items.', 4)

        rows = arcpy.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class")
        row = rows.Next()
        while row:
            CODE = stra(row.GetValue('Code')).rstrip(' ').replace('None','')
            PSTBCLS_TP = stra(row.GetValue('PSTBCLS_TP')).rstrip(' ').replace('None','')
            PSTBCLS_Dictionary[CODE] = PSTBCLS_TP

            Number_Processed = Number_Processed + 1
            row = rows.Next()

    SESC_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Sedimentation"):
        result = arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Sedimentation")
        Number_of_Features = int(result.GetOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Surface Erosion Sedimentation domain table for ' + stra(Number_of_Features) + ' items.', 4)

        rows = arcpy.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Sedimentation")
        row = rows.Next()
        while row:
            CODE = stra(row.GetValue('Code')).rstrip(' ').replace('None','')
            SESC_TXT = stra(row.GetValue('SESC_TXT')).rstrip(' ').replace('None','')
            SESC_Dictionary[CODE] = SESC_TXT

            Number_Processed = Number_Processed + 1
            row = rows.Next()

    SURFM_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surficial_Material"):
        result = arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surficial_Material")
        Number_of_Features = int(result.GetOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Surficial Material domain table for ' + stra(Number_of_Features) + ' items.', 4)

        rows = arcpy.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surficial_Material")
        row = rows.Next()
        while row:
            CODE = stra(row.GetValue('Code')).rstrip(' ').replace('None','')
            SURFM_DOM = stra(row.GetValue('Short_Name')).rstrip(' ').replace('None','')
            SURFM_Dictionary[CODE] = SURFM_DOM

            Number_Processed = Number_Processed + 1
            row = rows.Next()

    result = arcpy.GetCount_management(Processing_Variables['Long_Tbl'])
    Number_of_Features = int(result.GetOutput(0))
    Number_Processed = 0

    desc = arcpy.Describe(Processing_Variables['Long_Tbl'])
    Long_Tbl_Shape_Field_Name = desc.ShapeFieldName
    
    desc = arcpy.Describe(Processing_Variables['Short_Tbl'])
    Short_Tbl_Shape_Field_Name = desc.ShapeFieldName

    CPMessage('            - Temporarily removing attribute indexes from Short Table feature class', 4)
    idx_list = [idx for idx in arcpy.ListIndexes(Processing_Variables['Short_Tbl'])]
    for idx in idx_list:
        field_list = [f for f in idx.fields]
        index_can_be_removed = True
        for field in field_list:
            if field.type in ['OID','Geometry']:
                index_can_be_removed = False
        if index_can_be_removed:
            field_name_list = [f.name for f in idx.fields]
            field_name_list_string = ''
            for field_name in field_name_list:
                field_name_list_string += field_name + ", "
            field_name_list_string = field_name_list_string[:-2]
            CPMessage('                - Removing attribute index ' + idx.name + ' on field(s) ' + field_name_list_string, 4)
            arcpy.RemoveIndex_management(Processing_Variables['Short_Tbl'],idx.name)
            
    CPMessage('            - Reading Long Table and writing Short Table for ' + stra(Number_of_Features) + ' items.', 4)
    long_rows = arcpy.SearchCursor(Processing_Variables['Long_Tbl'])
    long_row = long_rows.Next()

    short_rows = arcpy.InsertCursor(Processing_Variables['Short_Tbl'])

    dtCalcStart = time.time()
    while long_row:
        if (0 < Number_Processed <= 10000 and Number_Processed % 1000 == 0) or (10000 < Number_Processed <= 100000 and Number_Processed % 10000 == 0) or (100000 < Number_Processed and Number_Processed % 100000 == 0):
            dtCalcNow = time.time()
            dtCalcElapsed = dtCalcNow - dtCalcStart
            dtCalcRemaining = (dtCalcElapsed / Number_Processed) * (Number_of_Features - Number_Processed)
            CPMessage('                - Processed ' + stra(Number_Processed) + ' of ' + stra(Number_of_Features) + ' (' + SanitizeElapsedTime(dtCalcElapsed)[:-1] + ' elapsed, ' + SanitizeElapsedTime(dtCalcRemaining)[:-1] + ' remaining)', 4)

        BAPID = long_row.GetValue('BAPID')
        if BAPID in Processing_Variables['BAPID_LIST'] or Processing_Variables['BAPID_LIST'] == ['ALL']:

            short_row = short_rows.NewRow()
            
            input_feature_geometry = long_row.GetValue(Long_Tbl_Shape_Field_Name)
            
            short_row.SetValue(Short_Tbl_Shape_Field_Name, input_feature_geometry)

            short_row.SetValue("BAPID", BAPID)

            TEIS_ID = long_row.GetValue("TEIS_ID")
            short_row.SetValue("TEIS_ID", TEIS_ID)

            PROJPOLYID = long_row.GetValue("PROJPOLYID")
            short_row.SetValue("PROJPOLYID", PROJPOLYID)

            if TEIS_ID in ErrorSum_Dictionary:
                short_row.SetValue("ERROR_SUM", ErrorSum_Dictionary[TEIS_ID][:500])
            else:
                short_row.SetValue("ERROR_SUM", '')

            if BAPID in ProjDetails_Dictionary:
                if stra(ProjDetails_Dictionary[BAPID]['MOD_DATE']) not in ['',' ','None']:
                    short_row.SetValue("MOD_DATE", ProjDetails_Dictionary[BAPID]['MOD_DATE'])
                short_row.SetValue("PROJ_TYPE", ProjDetails_Dictionary[BAPID]['PROJ_TYPE'])
                if stra(ProjDetails_Dictionary[BAPID]['PROJ_SCALE']) not in ['',' ','None']:
                    short_row.SetValue("PROJ_SCALE", ProjDetails_Dictionary[BAPID]['PROJ_SCALE'])
                short_row.SetValue("PROJ_NAME", ProjDetails_Dictionary[BAPID]['PROJ_NAME'])
                short_row.SetValue("LEGEND_COM", ProjDetails_Dictionary[BAPID]['LEGEND_COM'])
                short_row.SetValue("LEGEND_URL", ProjDetails_Dictionary[BAPID]['LEGEND_URL'])
            else:
                short_row.SetNull("MOD_DATE")
                short_row.SetValue("PROJ_TYPE", '')
                short_row.SetNull("PROJ_SCALE")
                short_row.SetValue("PROJ_NAME", '')
                short_row.SetValue("LEGEND_COM", '')
                short_row.SetValue("LEGEND_URL", '')

            FCODE = stra(long_row.GetValue("FCODE"))
            short_row.SetValue("FCODE", FCODE)

            ECO_SEC = stra(long_row.GetValue("ECO_SEC"))
            short_row.SetValue("ECO_SEC", ECO_SEC)

            # BGC_LBL - Equals <Bgc_zone> + <Bgc_Subzon> + <Bgc_Vrt> + <Bgc_Phase>
            BGC_ZONE = stra(long_row.GetValue("BGC_ZONE")).rstrip(' ').replace('None','')
            BGC_SUBZON = stra(long_row.GetValue("BGC_SUBZON")).rstrip(' ').replace('None','')
            BGC_VRT = stra(long_row.GetValue("BGC_VRT")).rstrip(' ').replace('None','')
            BGC_PHASE = stra(long_row.GetValue("BGC_PHASE")).rstrip(' ').replace('None','')

            BGC_LBL = BGC_ZONE + BGC_SUBZON + BGC_VRT + BGC_PHASE
            BGC_LBL = BGC_LBL.replace(' ', '')
            BGC_LBL = BGC_LBL.replace('None', '')        

            short_row.SetValue("BGC_LBL", BGC_LBL)

            # ECOLBL_1 - Equals <Sdec1> + <SiteMC_S1> + "[" + <Site_S1> + "]" + <Site_M1a> + <Site_M1b> + <Strct_S1> + <Strct_M1> + <Stand_A1> + ":" + <Seral_1> 
            SDEC_1 = stra(long_row.GetValue("SDEC_1")).rstrip(' ').replace('None','')
            SITEMC_S1 = stra(long_row.GetValue("SITEMC_S1")).rstrip(' ').replace('None','')
            SITE_S1 = stra(long_row.GetValue("SITE_S1")).rstrip(' ').replace('None','')
            SITE_M1A = stra(long_row.GetValue("SITE_M1A")).rstrip(' ').replace('None','')
            SITE_M1B = stra(long_row.GetValue("SITE_M1B")).rstrip(' ').replace('None','')
            STRCT_S1 = stra(long_row.GetValue("STRCT_S1")).rstrip(' ').replace('None','')
            STRCT_M1 = stra(long_row.GetValue("STRCT_M1")).rstrip(' ').replace('None','')
            STAND_A1 = stra(long_row.GetValue("STAND_A1")).rstrip(' ').replace('None','')
            SERAL_1 = stra(long_row.GetValue("SERAL_1")).rstrip(' ').replace('None','')

            if SDEC_1 in ['1','2','3','4','5','6','7','8','9','10']:
                short_row.SetValue("SDEC_1", int(SDEC_1))
            else:
                short_row.SetNull("SDEC_1")

            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S1 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S1 = '0' + SITE_S1

            ECOLBL_1 = SITEMC_S1 + r"[" + SITE_S1 + r"]" + SITE_M1A + SITE_M1B + STRCT_S1 + STRCT_M1 + STAND_A1 + r":" + SERAL_1
            ECOLBL_1 = ECOLBL_1.replace(' ', '')
            ECOLBL_1 = ECOLBL_1.replace('None', '')        
            ECOLBL_1 = ECOLBL_1.replace(r'[]', '')
            ECOLBL_1 = ECOLBL_1.strip(r':')
            short_row.SetValue("ECOLBL_1", ECOLBL_1)

            # ECOLBL_2 - Equals <Sdec2> + <SiteMC_S2> + "[" + <Site_S2> + "]" + <Site_M2a> + <Site_M2b> + <Strct_S2> + <Strct_M2> + <Stand_A2> + ":" + <Seral_2> 
            SDEC_2 = stra(long_row.GetValue("SDEC_2")).rstrip(' ').replace('None','')
            SITEMC_S2 = stra(long_row.GetValue("SITEMC_S2")).rstrip(' ').replace('None','')
            SITE_S2 = stra(long_row.GetValue("SITE_S2")).rstrip(' ').replace('None','')
            SITE_M2A = stra(long_row.GetValue("SITE_M2A")).rstrip(' ').replace('None','')
            SITE_M2B = stra(long_row.GetValue("SITE_M2B")).rstrip(' ').replace('None','')
            STRCT_S2 = stra(long_row.GetValue("STRCT_S2")).rstrip(' ').replace('None','')
            STRCT_M2 = stra(long_row.GetValue("STRCT_M2")).rstrip(' ').replace('None','')
            STAND_A2 = stra(long_row.GetValue("STAND_A2")).rstrip(' ').replace('None','')
            SERAL_2 = stra(long_row.GetValue("SERAL_2")).rstrip(' ').replace('None','')

            if SDEC_2 in ['1','2','3','4','5','6','7','8','9','10']:
                short_row.SetValue("SDEC_2", int(SDEC_2))
            else:
                short_row.SetNull("SDEC_2")

            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S2 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S2 = '0' + SITE_S2

            ECOLBL_2 = SITEMC_S2 + r"[" + SITE_S2 + r"]" + SITE_M2A + SITE_M2B + STRCT_S2 + STRCT_M2 + STAND_A2 + r":" + SERAL_2
            ECOLBL_2 = ECOLBL_2.replace(' ', '')
            ECOLBL_2 = ECOLBL_2.replace('None', '')        
            ECOLBL_2 = ECOLBL_2.replace(r'[]', '')
            ECOLBL_2 = ECOLBL_2.strip(r':')
            short_row.SetValue("ECOLBL_2", ECOLBL_2)


            # ECOLBL_3 - Equals <Sdec3> + <SiteMC_S3> + "[" + <Site_S3> + "]" + <Site_M3a> + <Site_M3b> + <Strct_S3> + <Strct_M3> + <Stand_A3> + ":" + <Seral_3> 
            SDEC_3 = stra(long_row.GetValue("SDEC_3")).rstrip(' ').replace('None','')
            SITEMC_S3 = stra(long_row.GetValue("SITEMC_S3")).rstrip(' ').replace('None','')
            SITE_S3 = stra(long_row.GetValue("SITE_S3")).rstrip(' ').replace('None','')
            SITE_M3A = stra(long_row.GetValue("SITE_M3A")).rstrip(' ').replace('None','')
            SITE_M3B = stra(long_row.GetValue("SITE_M3B")).rstrip(' ').replace('None','')
            STRCT_S3 = stra(long_row.GetValue("STRCT_S3")).rstrip(' ').replace('None','')
            STRCT_M3 = stra(long_row.GetValue("STRCT_M3")).rstrip(' ').replace('None','')
            STAND_A3 = stra(long_row.GetValue("STAND_A3")).rstrip(' ').replace('None','')
            SERAL_3 = stra(long_row.GetValue("SERAL_3")).rstrip(' ').replace('None','')

            if SDEC_3 in ['1','2','3','4','5','6','7','8','9','10']:
                short_row.SetValue("SDEC_3", int(SDEC_3))
            else:
                short_row.SetNull("SDEC_3")

            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if  SITE_S3 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S3 = '0' + SITE_S3


            ECOLBL_3 = SITEMC_S3 + r"[" + SITE_S3 + r"]" + SITE_M3A + SITE_M3B + STRCT_S3 + STRCT_M3 + STAND_A3 + r":" + SERAL_3
            ECOLBL_3 = ECOLBL_3.replace(' ', '')
            ECOLBL_3 = ECOLBL_3.replace('None', '')        
            ECOLBL_3 = ECOLBL_3.replace(r'[]', '')
            ECOLBL_3 = ECOLBL_3.strip(r':')
            short_row.SetValue("ECOLBL_3", ECOLBL_3)

            # SITES_LBL1 - Equals "[ "+ <Site_S1> + "]" + ":"+ <Seral_1> 
            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S1 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S1 = '0' + SITE_S1

            # used to be: SITES_LBL1 = r"[" + SITE_S1 + r"]" + r":" + SERAL_1  (changed March 28 2011 by Jeff K)
            SITES_LBL1 = SITE_S1 + r":" + SERAL_1
            SITES_LBL1 = SITES_LBL1.replace(' ', '')
            SITES_LBL1 = SITES_LBL1.replace('None', '')        
            SITES_LBL1 = SITES_LBL1.replace(r'[]', '')
            SITES_LBL1 = SITES_LBL1.strip(r':')
            short_row.SetValue("SITES_LBL1", SITES_LBL1)

            # SITES_LBL2 - Equals "[ "+ <Site_S2> + "]" + ":"+ <Seral_2> 
            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S2 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S2 = '0' + SITE_S2        

            # used to be: SITES_LBL2 = r"[" + SITE_S2 + r"]" + r":" + SERAL_2  (changed March 28 2011 by Jeff K)
            SITES_LBL2 = SITE_S2 + r":" + SERAL_2
            SITES_LBL2 = SITES_LBL2.replace(' ', '')
            SITES_LBL2 = SITES_LBL2.replace('None', '')        
            SITES_LBL2 = SITES_LBL2.replace(r'[]', '')
            SITES_LBL2 = SITES_LBL2.strip(r':')
            short_row.SetValue("SITES_LBL2", SITES_LBL2)

            # SITES_LBL3 - Equals "[ "+ <Site_S3> + "]" + ":"+ <Seral_3>  
            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if  SITE_S3 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S3 = '0' + SITE_S3        

            # used to be: SITES_LBL3 = r"[" + SITE_S3 + r"]" + r":" + SERAL_3  (changed March 28 2011 by Jeff K)
            SITES_LBL3 = SITE_S3 + r":" + SERAL_3
            SITES_LBL3 = SITES_LBL3.replace(' ', '')
            SITES_LBL3 = SITES_LBL3.replace('None', '')        
            SITES_LBL3 = SITES_LBL3.replace(r'[]', '')
            SITES_LBL3 = SITES_LBL3.strip(r':')
            short_row.SetValue("SITES_LBL3", SITES_LBL3)


            # SITEMCLBL1 - Equals <siteMC_1 > + ":" + <Seral_1 >
            ## Actually SITEMC_S1
            # used to be: SITEMCLBL1 = r"[" + SITEMC_S1 + r"]" + r":" + SERAL_1  (changed March 28 2011 by Jeff K)
            SITEMCLBL1 = SITEMC_S1 + r":" + SERAL_1
            SITEMCLBL1 = SITEMCLBL1.replace(' ', '')
            SITEMCLBL1 = SITEMCLBL1.replace('None', '')        
            SITEMCLBL1 = SITEMCLBL1.replace(r'[]', '')
            SITEMCLBL1 = SITEMCLBL1.strip(r':')
            short_row.SetValue("SITEMCLBL1", SITEMCLBL1)

            # SITEMCLBL2 - Equals <siteMC_2 > + ":" + <Seral_2 >
            ## Actually SITEMC_S2
            # used to be: SITEMCLBL2 = r"[" + SITEMC_S2 + r"]" + r":" + SERAL_2 (changed March 28 2011 by Jeff K)
            SITEMCLBL2 = SITEMC_S2 + r":" + SERAL_2
            SITEMCLBL2 = SITEMCLBL2.replace(' ', '')
            SITEMCLBL2 = SITEMCLBL2.replace('None', '')        
            SITEMCLBL2 = SITEMCLBL2.replace(r'[]', '')
            SITEMCLBL2 = SITEMCLBL2.strip(r':')
            short_row.SetValue("SITEMCLBL2", SITEMCLBL2)

            # SITEMCLBL3 - Equals <siteMC_3 > + ":" + <Seral_3 >
            ## Actually SITEMC_S3
            # used to be: SITEMCLBL3 = r"[" + SITEMC_S3 + r"]" + r":" + SERAL_3 (changed March 28 2011 by Jeff K)
            SITEMCLBL3 = SITEMC_S3 + r":" + SERAL_3
            SITEMCLBL3 = SITEMCLBL3.replace(' ', '')
            SITEMCLBL3 = SITEMCLBL3.replace('None', '')        
            SITEMCLBL3 = SITEMCLBL3.replace(r'[]', '')
            SITEMCLBL3 = SITEMCLBL3.strip(r':')
            short_row.SetValue("SITEMCLBL3", SITEMCLBL3)        


            # SEILBL_1 - Equals <SECl_1> + ":"+ <SEsubcl_1> 
            SECL_1 = stra(long_row.GetValue("SECL_1")).rstrip(' ').replace('None','')
            SESUBCL_1 = stra(long_row.GetValue("SESUBCL_1")).rstrip(' ').replace('None','')

            SEILBL_1 = SECL_1 + r":" + SESUBCL_1
            SEILBL_1 = SEILBL_1.replace(' ', '')
            SEILBL_1 = SEILBL_1.replace('None', '')        
            SEILBL_1 = SEILBL_1.strip(r':')
            short_row.SetValue("SEILBL_1", SEILBL_1)    
            short_row.SetValue("SECL_1", SECL_1)   
            short_row.SetValue("SESUBCL_1", SESUBCL_1)   

            # SEILBL_2 - Equals <SECl_2> + ":"+ <SEsubcl_2> 
            SECL_2 = stra(long_row.GetValue("SECL_2")).rstrip(' ').replace('None','')
            SESUBCL_2 = stra(long_row.GetValue("SESUBCL_2")).rstrip(' ').replace('None','')

            SEILBL_2 = SECL_2 + r":" + SESUBCL_2
            SEILBL_2 = SEILBL_2.replace(' ', '')
            SEILBL_2 = SEILBL_2.replace('None', '')        
            SEILBL_2 = SEILBL_2.strip(r':')
            short_row.SetValue("SEILBL_2", SEILBL_2)   
            short_row.SetValue("SECL_2", SECL_2)   
            short_row.SetValue("SESUBCL_2", SESUBCL_2)   


            # SEILBL_3 - Equals <SECl_1> + ":"+ <SEsubcl_3> 
            SECL_3 = stra(long_row.GetValue("SECL_3")).rstrip(' ').replace('None','')
            SESUBCL_3 = stra(long_row.GetValue("SESUBCL_3")).rstrip(' ').replace('None','')

            SEILBL_3 = SECL_3 + r":" + SESUBCL_3
            SEILBL_3 = SEILBL_3.replace(' ', '')
            SEILBL_3 = SEILBL_3.replace('None', '')        
            SEILBL_3 = SEILBL_3.strip(r':')
            short_row.SetValue("SEILBL_3", SEILBL_3)   
            short_row.SetValue("SECL_3", SECL_3)   
            short_row.SetValue("SESUBCL_3", SESUBCL_3)   

            COND_1 = stra(long_row.GetValue("COND_1")).rstrip(' ').replace('None','')
            VIAB_1 = stra(long_row.GetValue("VIAB_1")).rstrip(' ').replace('None','')
            COND_2 = stra(long_row.GetValue("COND_2")).rstrip(' ').replace('None','')
            VIAB_2 = stra(long_row.GetValue("VIAB_2")).rstrip(' ').replace('None','')
            COND_3 = stra(long_row.GetValue("COND_3")).rstrip(' ').replace('None','')
            VIAB_3 = stra(long_row.GetValue("VIAB_3")).rstrip(' ').replace('None','')

            short_row.SetValue("COND_1", COND_1)   
            short_row.SetValue("VIAB_1", VIAB_1)   
            short_row.SetValue("COND_2", COND_2)   
            short_row.SetValue("VIAB_2", VIAB_2)   
            short_row.SetValue("COND_3", COND_3)   
            short_row.SetValue("VIAB_3", VIAB_3)   


            # STRCTLBL_1 - <Strct_S1> + <Strct_M1>+ <Stand_A1>
            STRCTLBL_1 = STRCT_S1 + STRCT_M1 + STAND_A1
            STRCTLBL_1 = STRCTLBL_1.replace(' ', '')
            STRCTLBL_1 = STRCTLBL_1.replace('None', '')        
            short_row.SetValue("STRCTLBL_1", STRCTLBL_1) 

            # STRCTLBL_2 - <Strct_S2> + <Strct_M2>+ <Stand_A2>
            STRCTLBL_2 = STRCT_S2 + STRCT_M2 + STAND_A2
            STRCTLBL_2 = STRCTLBL_2.replace(' ', '')
            STRCTLBL_2 = STRCTLBL_2.replace('None', '')        
            short_row.SetValue("STRCTLBL_2", STRCTLBL_2) 

            # STRCTLBL_3 - <Strct_S3> + <Strct_M3>+ <Stand_A3>
            STRCTLBL_3 = STRCT_S3 + STRCT_M3 + STAND_A3
            STRCTLBL_3 = STRCTLBL_3.replace(' ', '')
            STRCTLBL_3 = STRCTLBL_3.replace('None', '')        
            short_row.SetValue("STRCTLBL_3", STRCTLBL_3) 


            # TERLBL_1A (Detailed Polys - Calculated)
            # Equals <Prtflg_1> + <Ttex_1> + <Surfm_1> + "(" + <Surfm_Q1>+ ")" + <Surfm_St1> + <Surf_E1>
            PRTFLG_1   = stra(long_row.GetValue("PRTFLG_1")).rstrip(' ').replace('None','')
            ## TTEX_1 will be discontinued from the data model in the future.
            ## TTEX_1     = stra(long_row.GetValue("TTEX_1"))
            TTEX_1A     = stra(long_row.GetValue("TTEX_1A")).rstrip(' ').replace('None','')
            TTEX_1B     = stra(long_row.GetValue("TTEX_1B")).rstrip(' ').replace('None','')
            TTEX_1C     = stra(long_row.GetValue("TTEX_1C")).rstrip(' ').replace('None','')
            TTEX_1 = TTEX_1C + TTEX_1B + TTEX_1A
            SURFM_1    = stra(long_row.GetValue("SURFM_1")).rstrip(' ').replace('None','')
            SURFM_Q1   = stra(long_row.GetValue("SURFM_Q1")).rstrip(' ').replace('None','')
            SURFM_ST1  = stra(long_row.GetValue("SURFM_ST1")).rstrip(' ').replace('None','')
            ## SURF_E1 will be discontinued from the data model in the future.
            ## SURF_E1    = stra(long_row.GetValue("SURF_E1"))
            SURF_E1A    = stra(long_row.GetValue("SURF_E1A")).rstrip(' ').replace('None','')
            SURF_E1B    = stra(long_row.GetValue("SURF_E1B")).rstrip(' ').replace('None','')
            SURF_E1C    = stra(long_row.GetValue("SURF_E1C")).rstrip(' ').replace('None','')
            SURF_E1     = SURF_E1A + SURF_E1B + SURF_E1C

            TERLBL_1A = PRTFLG_1 + TTEX_1 + SURFM_1 + "(" + SURFM_Q1 + ")" + SURFM_ST1 + SURF_E1

            TERLBL_1A = TERLBL_1A.replace(' ', '')
            TERLBL_1A = TERLBL_1A.replace('None', '')
            TERLBL_1A = TERLBL_1A.replace('()', '')
            short_row.SetValue("TERLBL_1A", TERLBL_1A)


            # TERLBL_1B (Detailed Polys - Calculated)
            # Equals <Sttex_1> + <Ssurfm_1> + "(" + <Ssurfm_Q1> + ")" + <Ssurfm_St1> + <Ssurf_E1>
            ## STTEX_1 will be discontinued from the data model in the future.
            ## STTEX_1 = stra(long_row.GetValue("STTEX_1"))
            STTEX_1A = stra(long_row.GetValue("STTEX_1A")).rstrip(' ').replace('None','')
            STTEX_1B = stra(long_row.GetValue("STTEX_1B")).rstrip(' ').replace('None','')
            STTEX_1C = stra(long_row.GetValue("STTEX_1C")).rstrip(' ').replace('None','')
            STTEX_1 = STTEX_1C + STTEX_1B + STTEX_1A
            SSURFM_1 = stra(long_row.GetValue("SSURFM_1")).rstrip(' ').replace('None','')
            SSURFM_Q1 = stra(long_row.GetValue("SSURFM_Q1")).rstrip(' ').replace('None','')
            SSURFM_ST1 = stra(long_row.GetValue("SSURFM_ST1")).rstrip(' ').replace('None','')
            ## SSURF_E1 will be discontinued from the data model in the future.
            ## SSURF_E1 = stra(long_row.GetValue("SSURF_E1"))
            SSURF_E1A = stra(long_row.GetValue("SSURF_E1A")).rstrip(' ').replace('None','')
            SSURF_E1B = stra(long_row.GetValue("SSURF_E1B")).rstrip(' ').replace('None','')
            SSURF_E1C = stra(long_row.GetValue("SSURF_E1C")).rstrip(' ').replace('None','')
            SSURF_E1 = SSURF_E1A + SSURF_E1B + SSURF_E1C

            TERLBL_1B = STTEX_1 + SSURFM_1 + "(" + SSURFM_Q1 + ")" + SSURFM_ST1 + SSURF_E1

            TERLBL_1B = TERLBL_1B.replace(' ', '')
            TERLBL_1B = TERLBL_1B.replace('None', '')
            TERLBL_1B = TERLBL_1B.replace('()', '')
            short_row.SetValue("TERLBL_1B", TERLBL_1B)


            # TERLBL_2A (Detailed Polys - Calculated)
            # Equals <Prtflg_2> + <Ttex_2> + <Surfm_2> +  "(" + <Surfm_Q2> + ")" + <Surfm_St2> + <Surf_E2>
            PRTFLG_2 = stra(long_row.GetValue("PRTFLG_2")).rstrip(' ').replace('None','')
            ## TTEX_2 will be discontinued from the data model in the future.
            ## TTEX_2 = stra(long_row.GetValue("TTEX_2"))
            TTEX_2A = stra(long_row.GetValue("TTEX_2A")).rstrip(' ').replace('None','')
            TTEX_2B = stra(long_row.GetValue("TTEX_2B")).rstrip(' ').replace('None','')
            TTEX_2C = stra(long_row.GetValue("TTEX_2C")).rstrip(' ').replace('None','')
            TTEX_2 = TTEX_2C + TTEX_2B + TTEX_2A
            SURFM_2 = stra(long_row.GetValue("SURFM_2")).rstrip(' ').replace('None','')
            SURFM_Q2 = stra(long_row.GetValue("SURFM_Q2")).rstrip(' ').replace('None','')
            SURFM_ST2 = stra(long_row.GetValue("SURFM_ST2")).rstrip(' ').replace('None','')
            ## SURF_E2 will be discontinued from the data model in the future.
            ## SURF_E2 = stra(long_row.GetValue("SURF_E2"))
            SURF_E2A = stra(long_row.GetValue("SURF_E2A")).rstrip(' ').replace('None','')
            SURF_E2B = stra(long_row.GetValue("SURF_E2B")).rstrip(' ').replace('None','')
            SURF_E2C = stra(long_row.GetValue("SURF_E2C")).rstrip(' ').replace('None','')
            SURF_E2 = SURF_E2A + SURF_E2B + SURF_E2C

            TERLBL_2A = PRTFLG_2 + TTEX_2 + SURFM_2 +  "(" + SURFM_Q2 + ")" + SURFM_ST2 + SURF_E2

            TERLBL_2A = TERLBL_2A.replace(' ', '')
            TERLBL_2A = TERLBL_2A.replace('None', '')
            TERLBL_2A = TERLBL_2A.replace('()', '')
            short_row.SetValue("TERLBL_2A", TERLBL_2A)


            # TERLBL_2B (Detailed Polys - Calculated)
            # Equals <Sttex_2> + <Ssurfm_2> + "(" + <Ssurfm_Q2>+ ")" + <Ssurfm_St2> + <Ssurf_E2>
            ## STTEX_2 will be discontinued from the data model in the future.
            ## STTEX_2 = stra(long_row.GetValue("STTEX_2"))
            STTEX_2A = stra(long_row.GetValue("STTEX_2A")).rstrip(' ').replace('None','')
            STTEX_2B = stra(long_row.GetValue("STTEX_2B")).rstrip(' ').replace('None','')
            STTEX_2C = stra(long_row.GetValue("STTEX_2C")).rstrip(' ').replace('None','')
            STTEX_2 = STTEX_2C + STTEX_2B + STTEX_2A
            SSURFM_2 = stra(long_row.GetValue("SSURFM_2")).rstrip(' ').replace('None','')
            SSURFM_Q2 = stra(long_row.GetValue("SSURFM_Q2")).rstrip(' ').replace('None','')
            SSURFM_ST2 = stra(long_row.GetValue("SSURFM_ST2")).rstrip(' ').replace('None','')
            ## SSURF_E2 will be discontinued from the data model in the future.
            ## SSURF_E2 = stra(long_row.GetValue("SSURF_E2"))
            SSURF_E2A = stra(long_row.GetValue("SSURF_E2A")).rstrip(' ').replace('None','')
            SSURF_E2B = stra(long_row.GetValue("SSURF_E2B")).rstrip(' ').replace('None','')
            SSURF_E2C = stra(long_row.GetValue("SSURF_E2C")).rstrip(' ').replace('None','')
            SSURF_E2 = SSURF_E2A + SSURF_E2B + SSURF_E2C

            TERLBL_2B = STTEX_2 + SSURFM_2 + "(" + SSURFM_Q2 + ")" + SSURFM_ST2 + SSURF_E2

            TERLBL_2B = TERLBL_2B.replace(' ', '')
            TERLBL_2B = TERLBL_2B.replace('None', '')
            TERLBL_2B = TERLBL_2B.replace('()', '')
            short_row.SetValue("TERLBL_2B", TERLBL_2B)        


            # TERLBL_3A (Detailed Polys - Calculated)
            # Equals <Prtflg_3> + <Ttex_3> + <Surfm_3> + "(" + <Surfm_Q3> + ")"+ <Surfm_St3> + <Surf_E3>

            PRTFLG_3   = stra(long_row.GetValue("PRTFLG_3")).rstrip(' ').replace('None','')
            ## TTEX_3 will be discontinued from the data model in the future.
            ## TTEX_3     = stra(long_row.GetValue("TTEX_3"))
            TTEX_3A     = stra(long_row.GetValue("TTEX_3A")).rstrip(' ').replace('None','')
            TTEX_3B     = stra(long_row.GetValue("TTEX_3B")).rstrip(' ').replace('None','')
            TTEX_3C     = stra(long_row.GetValue("TTEX_3C")).rstrip(' ').replace('None','')
            TTEX_3     = TTEX_3C + TTEX_3B + TTEX_3A
            SURFM_3    = stra(long_row.GetValue("SURFM_3")).rstrip(' ').replace('None','')
            SURFM_Q3   = stra(long_row.GetValue("SURFM_Q3")).rstrip(' ').replace('None','')
            SURFM_ST3  = stra(long_row.GetValue("SURFM_ST3")).rstrip(' ').replace('None','')
            ## SURF_E3 will be discontinued from the data model in the future.
            ## SURF_E3    = stra(long_row.GetValue("SURF_E3"))
            SURF_E3A    = stra(long_row.GetValue("SURF_E3A")).rstrip(' ').replace('None','')
            SURF_E3B    = stra(long_row.GetValue("SURF_E3B")).rstrip(' ').replace('None','')
            SURF_E3C    = stra(long_row.GetValue("SURF_E3C")).rstrip(' ').replace('None','')
            SURF_E3    = SURF_E3A + SURF_E3B + SURF_E3C

            TERLBL_3A = PRTFLG_3 + TTEX_3 + SURFM_3 + "(" + SURFM_Q3 + ")" + SURFM_ST3 + SURF_E3

            TERLBL_3A = TERLBL_3A.replace(' ', '')
            TERLBL_3A = TERLBL_3A.replace('None', '')
            TERLBL_3A = TERLBL_3A.replace('()', '')
            short_row.SetValue("TERLBL_3A", TERLBL_3A)


            # TERLBL_3B (Detailed Polys - Calculated)
            # Equals <Sttex_3> + <Ssurfm_3> + "(" + <Ssurfm_Q3> + ")"+ <Ssurfm_St3> + <Ssurf_E3>
            ## STTEX_3 will be discontinued from the data model in the future.
            ## STTEX_3 = stra(long_row.GetValue("STTEX_3"))
            STTEX_3A = stra(long_row.GetValue("STTEX_3A")).rstrip(' ').replace('None','')
            STTEX_3B = stra(long_row.GetValue("STTEX_3B")).rstrip(' ').replace('None','')
            STTEX_3C = stra(long_row.GetValue("STTEX_3C")).rstrip(' ').replace('None','')
            STTEX_3 = STTEX_3C + STTEX_3B + STTEX_3A
            SSURFM_3 = stra(long_row.GetValue("SSURFM_3")).rstrip(' ').replace('None','')
            SSURFM_Q3 = stra(long_row.GetValue("SSURFM_Q3")).rstrip(' ').replace('None','')
            SSURFM_ST3 = stra(long_row.GetValue("SSURFM_ST3")).rstrip(' ').replace('None','')
            ## SSURF_E3 will be discontinued from the data model in the future.
            ## SSURF_E3 = stra(long_row.GetValue("SSURF_E3"))
            SSURF_E3A = stra(long_row.GetValue("SSURF_E3A")).rstrip(' ').replace('None','')
            SSURF_E3B = stra(long_row.GetValue("SSURF_E3B")).rstrip(' ').replace('None','')
            SSURF_E3C = stra(long_row.GetValue("SSURF_E3C")).rstrip(' ').replace('None','')
            SSURF_E3 = SSURF_E3A + SSURF_E3B + SSURF_E3C

            TERLBL_3B = STTEX_3 + SSURFM_3 + "(" + SSURFM_Q3 + ")" + SSURFM_ST3 + SSURF_E3

            TERLBL_3B = TERLBL_3B.replace(' ', '')
            TERLBL_3B = TERLBL_3B.replace('None', '')
            TERLBL_3B = TERLBL_3B.replace('()', '')
            short_row.SetValue("TERLBL_3B", TERLBL_3B)


            # GEOP_LBL (Detailed Polys - Calculated)
            # Equals <Geop_1> + <Geop_Q1> + <Geop_St1> + <Geop_InZ1> + <Geop_Scm1> + <Geop_2> + <Geop_Q2> + <Geop_St2> + <Geop_InZ2> + <Geop_Scm> + <Geop_3> + + <Geop_Q3> + <Geop_St3> + <Geop_InZ3> + <Geop_Scm3>
            #                                                                                                                               ^^^^ GEOP_SCM2
            GEOP_1 = stra(long_row.GetValue("GEOP_1")).rstrip(' ').replace('None','')
            GEOP_Q1 = stra(long_row.GetValue("GEOP_Q1")).rstrip(' ').replace('None','')
            GEOP_ST1 = stra(long_row.GetValue("GEOP_ST1")).rstrip(' ').replace('None','')
            GEOP_INZ1 = stra(long_row.GetValue("GEOP_INZ1")).rstrip(' ').replace('None','')
            ## GEOP_SCM1 will be discontinued from the data model in the future.
            ## GEOP_SCM1 = stra(long_row.GetValue("GEOP_SCM1"))
            GEOP_SCM1A = stra(long_row.GetValue("GEOP_SCM1A")).rstrip(' ').replace('None','')
            GEOP_SCM1B = stra(long_row.GetValue("GEOP_SCM1B")).rstrip(' ').replace('None','')
            GEOP_SCM1C = stra(long_row.GetValue("GEOP_SCM1C")).rstrip(' ').replace('None','')
            GEOP_SCM1 = GEOP_SCM1A + GEOP_SCM1B + GEOP_SCM1C
            GEOP_2 = stra(long_row.GetValue("GEOP_2")).rstrip(' ').replace('None','')
            GEOP_Q2 = stra(long_row.GetValue("GEOP_Q2")).rstrip(' ').replace('None','')
            GEOP_ST2 = stra(long_row.GetValue("GEOP_ST2")).rstrip(' ').replace('None','')
            GEOP_INZ2 = stra(long_row.GetValue("GEOP_INZ2")).rstrip(' ').replace('None','')
            ## GEOP_SCM2 will be discontinued from the data model in the future.
            ## GEOP_SCM2 = stra(long_row.GetValue("GEOP_SCM2"))
            GEOP_SCM2A = stra(long_row.GetValue("GEOP_SCM2A")).rstrip(' ').replace('None','')
            GEOP_SCM2B = stra(long_row.GetValue("GEOP_SCM2B")).rstrip(' ').replace('None','')
            GEOP_SCM2C = stra(long_row.GetValue("GEOP_SCM2C")).rstrip(' ').replace('None','')
            GEOP_SCM2 = GEOP_SCM2A + GEOP_SCM2B + GEOP_SCM2C
            GEOP_3 = stra(long_row.GetValue("GEOP_3")).rstrip(' ').replace('None','')
            GEOP_Q3 = stra(long_row.GetValue("GEOP_Q3")).rstrip(' ').replace('None','')
            GEOP_ST3 = stra(long_row.GetValue("GEOP_ST3")).rstrip(' ').replace('None','')
            GEOP_INZ3 = stra(long_row.GetValue("GEOP_INZ3")).rstrip(' ').replace('None','')
            ## GEOP_SCM3 will be discontinued from the data model in the future.
            ## GEOP_SCM3 = stra(long_row.GetValue("GEOP_SCM3"))
            GEOP_SCM3A = stra(long_row.GetValue("GEOP_SCM3A")).rstrip(' ').replace('None','')
            GEOP_SCM3B = stra(long_row.GetValue("GEOP_SCM3B")).rstrip(' ').replace('None','')
            GEOP_SCM3C = stra(long_row.GetValue("GEOP_SCM3C")).rstrip(' ').replace('None','')
            GEOP_SCM3 = GEOP_SCM3A + GEOP_SCM3B + GEOP_SCM3C

            GEOP_LBL = GEOP_1 + GEOP_Q1 + GEOP_ST1 + GEOP_INZ1 + GEOP_SCM1 + GEOP_2 + GEOP_Q2 + GEOP_ST2 + GEOP_INZ2 + GEOP_SCM2 + GEOP_3 + GEOP_Q3 + GEOP_ST3 + GEOP_INZ3 + GEOP_SCM3

            GEOP_LBL = GEOP_LBL.replace(' ', '')
            GEOP_LBL = GEOP_LBL.replace('None', '')
            GEOP_LBL = GEOP_LBL.replace('()', '')
            short_row.SetValue("GEOP_LBL", GEOP_LBL)


            # DRAIN_LBL (Detailed Polys - Calculated)
            # Equals <Drain_1> + <Drain_Sep> + <Drain_2>
            DRAIN_1 = stra(long_row.GetValue("DRAIN_1")).rstrip(' ').replace('None','')
            DRAIN_SEP = stra(long_row.GetValue("DRAIN_SEP")).rstrip(' ').replace('None','')
            DRAIN_2 = stra(long_row.GetValue("DRAIN_2")).rstrip(' ').replace('None','')

            DRAIN_LBL = DRAIN_1 + DRAIN_SEP + DRAIN_2

            DRAIN_LBL = DRAIN_LBL.replace(' ', '')
            DRAIN_LBL = DRAIN_LBL.replace('None', '')
            DRAIN_LBL = DRAIN_LBL.replace('()', '')
            short_row.SetValue("DRAIN_LBL", DRAIN_LBL)


            # SLOPE_LBL (Detailed Polys - Calculated)
            # Equals <Slpll_1> + "-" + <Slpul_1> + "," + <Slpll_2> + "-" + <Slpul_2>
            SLPLL_1 = stra(long_row.GetValue("SLPLL_1")).rstrip(' ').replace('None','')
            if SLPLL_1 == None:
                SLPLL_1 = 0
            SLPUL_1 = stra(long_row.GetValue("SLPUL_1")).rstrip(' ').replace('None','')
            if SLPUL_1 == None:
                SLPUL_1 = 0
            SLPLL_2 = stra(long_row.GetValue("SLPLL_2")).rstrip(' ').replace('None','')
            if SLPLL_2 == None:
                SLPLL_2 = 0
            SLPUL_2 = stra(long_row.GetValue("SLPUL_2")).rstrip(' ').replace('None','')
            if SLPUL_2 == None:
                SLPUL_2 = 0
            SLOPE_LBL = SLPLL_1 + "-" + SLPUL_1 + "," + SLPLL_2 + "-" + SLPUL_2
            SLOPE_LBL = SLOPE_LBL.replace(' ', '')
            SLOPE_LBL = SLOPE_LBL.replace('None', '')
            SLOPE_LBL = SLOPE_LBL.replace('()', '')
            SLOPE_LBL = SLOPE_LBL.strip('-,')
            # If there is no real slope information, set it to a blank label
            if SLOPE_LBL in ["0-0,0-0", "0-0"]:
                SLOPE_LBL = ""
            short_row.SetValue("SLOPE_LBL", SLOPE_LBL)

            # SLPSTBCLSR (Detailed Polys - Calculated)
            # Equals <Slpstb_Cls> + <Rdstb_Flg>
            SLPSTB_CLS = stra(long_row.GetValue("SLPSTB_CLS")).rstrip(' ').replace('None','')
            RDSTB_FLG = stra(long_row.GetValue("RDSTB_FLG")).rstrip(' ').replace('None','')

            SLPSTBCLSR = SLPSTB_CLS + RDSTB_FLG

            SLPSTBCLSR = SLPSTBCLSR.replace(' ', '')
            SLPSTBCLSR = SLPSTBCLSR.replace('None', '')
            SLPSTBCLSR = SLPSTBCLSR.replace('()', '')
            short_row.SetValue("SLPSTBCLSR", SLPSTBCLSR)

            if SLPSTBCLSR in SSC_Dictionary:
                short_row.SetValue("SSC_TXT", SSC_Dictionary[SLPSTBCLSR])
            else:
                short_row.SetValue("SSC_TXT", '')

            if SLPSTB_CLS in PSTBCLS_Dictionary:
                short_row.SetValue("PSTBCLS_TP", PSTBCLS_Dictionary[SLPSTB_CLS])
            else:
                short_row.SetValue("PSTBCLS_TP", '')

            # TER_LBL (Detailed Polys - Calculated)
            # If TER or TES Equals  <TERLBL_1A> + [ <TERLBL_1B> ] + <Comrel1_2> + <TERLBL_2A> + [ <TERLBL_2B> ] <TERLBL_2A> + [<TERLBL_2B>] + <Comrel2_3> + <TERLBL_3A> + [ <TERLBL_3B> ] <TERLBL_3A> + [<TERLBL_3B>] + '-' + <GEOP_LBL> IF BTM equils <Tdec_1> + <TERLBL_1A> + [ <TERLBL_1B> ] + <Tdec_2> + <TERLBL_2A> + [ <TERLBL_2B> ] <TERLBL_2A> + [<TERLBL_2B>] + <Tdec_3> + <TERLBL_3A> + [ <TERLBL_3B> ] <TERLBL_3A> + [<TERLBL_3B>] + '-' + <GEOP_LBL>
            PROJ_TYPE = ProjDetails_Dictionary[BAPID]['PROJ_TYPE']
            COMREL1_2 = stra(long_row.GetValue("COMREL1_2")).rstrip(' ').replace('None','')
            COMREL2_3 = stra(long_row.GetValue("COMREL2_3")).rstrip(' ').replace('None','')
            TDEC_1 = stra(long_row.GetValue("TDEC_1")).rstrip(' ').replace('None','')
            TDEC_2 = stra(long_row.GetValue("TDEC_2")).rstrip(' ').replace('None','')
            TDEC_3 = stra(long_row.GetValue("TDEC_3")).rstrip(' ').replace('None','')

            if TDEC_1 in ['1','2','3','4','5','6','7','8','9','10']:
                short_row.SetValue("TDEC_1", int(TDEC_1))
            else:
                short_row.SetNull("TDEC_1")

            if TDEC_2 in ['1','2','3','4','5','6','7','8','9','10']:
                short_row.SetValue("TDEC_2", int(TDEC_2))
            else:
                short_row.SetNull("TDEC_2")

            if TDEC_3 in ['1','2','3','4','5','6','7','8','9','10']:
                short_row.SetValue("TDEC_3", int(TDEC_3))
            else:
                short_row.SetNull("TDEC_3")

            short_row.SetValue("COMREL1_2", COMREL1_2)
            short_row.SetValue("COMREL2_3", COMREL2_3)

            BEDROCK_1 = stra(long_row.GetValue("BEDROCK_1")).rstrip(' ').replace('None','')
            BEDROCK_2 = stra(long_row.GetValue("BEDROCK_2")).rstrip(' ').replace('None','')
            BEDROCK_3 = stra(long_row.GetValue("BEDROCK_3")).rstrip(' ').replace('None','')
            short_row.SetValue("BEDROCK_1", BEDROCK_1)
            short_row.SetValue("BEDROCK_2", BEDROCK_2)
            short_row.SetValue("BEDROCK_3", BEDROCK_3)

            #### We must add logic for ALL project types here.  The Terrain Label Type (TER_Label_Type) indicator in the Domain_Project_Type table?

            TER_LBL = 'None'

            if PROJ_TYPE in DPT_Dictionary:
                if DPT_Dictionary[PROJ_TYPE] == 'Delimiter':
                    TER_LBL = TERLBL_1A + "[" + TERLBL_1B + "]" + COMREL1_2 + TERLBL_2A + "[" + TERLBL_2B + "]" + COMREL2_3 + TERLBL_3A + "[" + TERLBL_3B + "]"  + '-' + GEOP_LBL
                elif DPT_Dictionary[PROJ_TYPE] == 'Decile':
                    TER_LBL = TDEC_1 + TERLBL_1A + "[" + TERLBL_1B + "]" + TDEC_2 + TERLBL_2A + "[" + TERLBL_2B + "]" + TDEC_3 + TERLBL_3A + "[" + TERLBL_3B + "]" + '-' + GEOP_LBL

            #if TER_LBL == 'None':
            #    CPMessage(' ** WARNING: PROJ_TYPE (' + TER_LBL + ') in TEIS_ID ' + stra(TEIS_ID)  + ' not mapped to a TER_LBL ** ' + TER_LBL, 3)    

            TER_LBL = TER_LBL.replace(' ', '')
            TER_LBL = TER_LBL.replace('()', '')
            TER_LBL = TER_LBL.replace('None', '')
            TER_LBL = TER_LBL.replace('[]', '')
            TER_LBL = TER_LBL.strip('-')
            short_row.SetValue("TER_LBL", TER_LBL)
            CPMessage('                -- ' + TER_LBL, 5)

            # ORIG_LBL (Detailed Polys)
            # Equals <Orig_Lbl>; Called Map_Label in Compilation Project
            ## Changed to be the best corrected version that matches the original hard copy map with clarifications/QC
            #### Change metadata definition to match this (Deepa to provide)
            CAPTURED_LABEL = stra(long_row.GetValue("CAPLBL")).rstrip(' ').replace('None','')
            CAPTURED_LABEL = CAPTURED_LABEL.replace(' ', '')
            CAPTURED_LABEL = CAPTURED_LABEL.replace('None', '')

            CAPTURED_LABEL_CORRECTED = stra(long_row.GetValue("CAPLBL_COR")).rstrip(' ').replace('None','')
            CAPTURED_LABEL_CORRECTED = CAPTURED_LABEL_CORRECTED.replace(' ', '')
            CAPTURED_LABEL_CORRECTED = CAPTURED_LABEL_CORRECTED.replace('None', '')

            if stra(CAPTURED_LABEL_CORRECTED) != '':
                ORIG_LBL = CAPTURED_LABEL_CORRECTED
            else:
                ORIG_LBL = CAPTURED_LABEL
            short_row.SetValue("ORIG_LBL", ORIG_LBL)

            #POLY_COM = stra(long_row.GetValue("POLY_COM")).rstrip(' ').replace('None','')
            # normally we would "stra()" this value, but POLY_COM sometimes contains unicode characters which causes error

            POLY_COM = long_row.GetValue("POLY_COM")
            if POLY_COM in (' ',None):
                POLY_COM = ''
            s_POLY_COM = POLY_COM

            short_row.SetValue("POLY_COM", POLY_COM)

            SMPL_TYPE = stra(long_row.GetValue("SMPL_TYPE")).rstrip(' ').replace('None','')
            short_row.SetValue("SMPL_TYPE", SMPL_TYPE)

            short_row.SetValue("POLY_TEX", '')
            short_row.SetValue("POLY_THK", '')
            short_row.SetValue("THEME_1", '')
            short_row.SetValue("THEME_2", '')
            short_row.SetValue("THEME_3", '')

            SFCERO_POT = stra(long_row.GetValue("SFCERO_POT")).rstrip(' ').replace('None','')
            short_row.SetValue("SFCERO_POT", SFCERO_POT)

            if SFCERO_POT in SEP_Dictionary:
                short_row.SetValue("SEP_TXT", SEP_Dictionary[SFCERO_POT])
            else:
                short_row.SetValue("SEP_TXT", '')
    
            LSSED_CLS = stra(long_row.GetValue("LSSED_CLS")).rstrip(' ').replace('None','')
            if LSSED_CLS <> '':
                short_row.SetValue("LSSED_CLS", int(LSSED_CLS))

                if LSSED_CLS in LISSC_Dictionary:
                    short_row.SetValue("LISSC_TXT", LISSC_Dictionary[LSSED_CLS])
                else:
                    short_row.SetValue("LISSC_TXT", '')
            else:
                short_row.SetNull("LSSED_CLS")
                short_row.SetValue("LISSC_TXT", '')

            SESED_CLS = stra(long_row.GetValue("SESED_CLS")).rstrip(' ').replace('None','')
            short_row.SetValue("SESED_CLS", SESED_CLS)

            if SESED_CLS in SESC_Dictionary:
                short_row.SetValue("SESC_TXT", SESC_Dictionary[SESED_CLS])
            else:
                short_row.SetValue("SESC_TXT", '')

            if SURFM_1 in SURFM_Dictionary:
                short_row.SetValue("SURFM_DOM", SURFM_Dictionary[SURFM_1])
            else:
                short_row.SetValue("SURFM_DOM", '')


            #### POLY_TEX (Data Definition Doesn't Exist Yet, Multifield) - NO ACTION
            # CPMessage('        - Working on POLY_TEX', 3)
            # CPMessage('            -- Nothing to do', 4)    

            #### POLY_THK (Data Definition Doesn't Exist Yet, Multifield) - NO ACTION 
            # CPMessage('        - Working on POLY_THK', 3)
            # CPMessage('            -- Nothing to do', 4)

            try:
                short_rows.InsertRow(short_row)
            except:
                CPMessage(' ** Error in row: ' + stra(Number_Processed), 3)
            Number_Processed = Number_Processed + 1

        long_row = long_rows.Next()
    CPMessage('            - Processed all ' + stra(Number_Processed), 4)
    del long_row
    del long_rows
    del short_row
    del short_rows

    CPMessage('            - Creating attribute indexes in Short Table feature class', 4)
    for idx in idx_list:
        field_list = [f for f in idx.fields]
        index_should_be_added = True
        for field in field_list:
            if field.type in ['OID','Geometry']:
                index_should_be_added = False
        if index_should_be_added:
            field_name_list = [f.name for f in idx.fields]
            field_name_list_string = ''
            for field_name in field_name_list:
                field_name_list_string += field_name + ", "
            field_name_list_string = field_name_list_string[:-2]
            CPMessage('                - Adding attribute index ' + idx.name + ' on field(s) ' + field_name_list_string, 4)
            field_name_list_string = ''
            for field_name in field_name_list:
                field_name_list_string += field_name + ";"
            field_name_list_string = field_name_list_string[:-1]
            arcpy.AddIndex_management(Processing_Variables['Short_Tbl'],field_name_list_string,idx.name) ## index will be NON_UNIQUE and NON_ASCENDING by default
            
    return(Processing_Variables['PROCESS_STATUS'])


#---------------------------------------------------------------------------------------------------------
# PopulateTEISShortTable101()
#
#       Test if we need to do stuff, and do it if necessary.
#          Return 0 - nothing to do
#                 1 - successful
#                -1 - unknown failure
#
#---------------------------------------------------------------------------------------------------------
def PopulateTEISShortTable101():

    CPMessage('        - Populate TEIS Short Table', 2)
    Processing_Variables['PROCESS_STATUS'] = 0
    Processing_Variables['PROCESS_INFO']   = "PopulateTEISShortTable101" 

    ProjDetails_Dictionary = {}
    Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['Project_Details']).getOutput(0))
    Number_Processed = 0
    
    CPMessage('            - Reading Project Details table for ' + stra(Number_of_Features) + ' items.', 4)
    
    exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Project_Details'], ['BAPID','MOD_DATE','PROJ_TYPE','PROJ_SCALE','PROJ_NAME','LEGEND_COM','LEGEND_URL']) as cursor:
    for row in cursor:
        BAPID = row[0]
        MOD_DATE = row[1]
        PROJ_TYPE = row[2]
        PROJ_SCALE = row[3]
        PROJ_NAME = row[4]
        LEGEND_COM = row[5]
        LEGEND_URL = row[6]
        ProjDetails_Dictionary[BAPID] = {'MOD_DATE': MOD_DATE, 'PROJ_TYPE': PROJ_TYPE, 'PROJ_SCALE': PROJ_SCALE, 'PROJ_NAME': PROJ_NAME, 'LEGEND_COM': LEGEND_COM, 'LEGEND_URL': LEGEND_URL}
        Number_Processed = Number_Processed + 1'''
    exec(exec_string)

    CPMessage('            - Reading BAPID numbers in Master Long Table.', 4)
    row_count = 0
    bapid_dict = {}
    bapid_not_found_dict = {}
    exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Long_Tbl'], ['BAPID']) as cursor:
    for row in cursor:
        row_count += 1
        try:
            ProjDetails_Dictionary[row[0]]
        except:
            try:
                bapid_not_found_dict[row[0]]
            except:
                bapid_not_found_dict[row[0]] = True
        try:
            bapid_dict[row[0]]
        except:
            bapid_dict[row[0]] = True
        if row_count % 1000000 == 0:
            CPMessage('                - Read ' + stra(row_count) + ' rows (' + stra(len(bapid_dict.keys())) + ' unique BAPID(s) found)', 4)
    if len(bapid_not_found_dict.keys()) > 0:
        bapids_not_found_string = ''
        bapid_not_found_list = bapid_not_found_dict.keys()
        bapid_not_found_list.sort()
        for bapid in bapid_not_found_list:
            bapids_not_found_string += stra(bapid) + ', '
        bapids_not_found_string = bapids_not_found_string[:-2]
        CPMessage('    ***** BAPIDs not found in BAPID Tracker/Project Details table: ' + bapids_not_found_string, 0)
        sys.exit()
    CPMessage('                - Read ' + stra(row_count) + ' rows. All ' + stra(len(bapid_dict.keys())) + ' unique BAPIDs found in BAPID Tracker/Project Details table.', 4)'''
    exec(exec_string)

    ErrorSum_Dictionary = {}
    if arcpy.Exists(Processing_Variables['SummaryErrorTable']):
        Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['SummaryErrorTable']).getOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Error Summary table for ' + stra(Number_of_Features) + ' items.', 4)

        exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['SummaryErrorTable'], ['FEATURE_ID','PROBLEMS']) as cursor:
    for row in cursor:
        if Number_Processed % 10000 == 0:
            CPMessage('                - Read ' + stra(Number_Processed) + ' of ' + stra(Number_of_Features), 4)
        FEATURE_ID = row[0]
        PROBLEMS = row[1]
        ErrorSum_Dictionary[FEATURE_ID] = PROBLEMS
        Number_Processed = Number_Processed + 1'''
        exec(exec_string)
        CPMessage('                - Read ' + stra(Number_Processed) + ' of ' + stra(Number_of_Features), 4)


    DPT_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Project_Type"):
        Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Project_Type").getOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Project Type domain table for ' + stra(Number_of_Features) + ' items.', 4)

        exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Project_Type", ['Code','TER_Label_Type']) as cursor:
    for row in cursor:
        CODE = row[0]
        TER_Label_Type = row[1]
        DPT_Dictionary[CODE] = TER_Label_Type
        Number_Processed = Number_Processed + 1'''
        exec(exec_string)

    SSC_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class_with_Roads_Qualifier_Flag"):
        Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class_with_Roads_Qualifier_Flag").getOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Slope Stability Class domain table for ' + stra(Number_of_Features) + ' items.', 4)

        exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class_with_Roads_Qualifier_Flag", ['Code','SSC_TXT']) as cursor:
    for row in cursor:
        CODE = row[0]
        SSC_TXT = row[1]
        SSC_Dictionary[CODE] = SSC_TXT
        Number_Processed = Number_Processed + 1'''
        exec(exec_string)

    SEP_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Potential"):
        Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Potential").getOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Surface Erosion Potential domain table for ' + stra(Number_of_Features) + ' items.', 4)

        exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Potential", ['Code','SEP_TXT']) as cursor:
    for row in cursor:
        CODE = row[0]
        SEP_TXT = row[1]
        SEP_Dictionary[CODE] = SEP_TXT
        Number_Processed = Number_Processed + 1'''
        exec(exec_string)

    LISSC_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Landslide_Induced_Stream_Sedimentation"):
        Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Landslide_Induced_Stream_Sedimentation").getOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Landslide Induced Stream Sedimentation domain table for ' + stra(Number_of_Features) + ' items.', 4)

        exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Landslide_Induced_Stream_Sedimentation", ['Code','LISSC_TXT']) as cursor:
    for row in cursor:
        CODE = stra(row[0])
        LISSC_TXT = row[1]
        LISSC_Dictionary[CODE] = LISSC_TXT
        Number_Processed = Number_Processed + 1'''
        exec(exec_string)

    PSTBCLS_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class"):
        Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class").getOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Slope Stability Class domain table for ' + stra(Number_of_Features) + ' items.', 4)

        exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Slope_Stability_Class", ['Code','PSTBCLS_TP']) as cursor:
    for row in cursor:
        CODE = row[0]
        PSTBCLS_TP = row[1]
        PSTBCLS_Dictionary[CODE] = PSTBCLS_TP
        Number_Processed = Number_Processed + 1'''
        exec(exec_string)

    SESC_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Sedimentation"):
        Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Sedimentation").getOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Surface Erosion Sedimentation domain table for ' + stra(Number_of_Features) + ' items.', 4)

        exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surface_Erosion_Sedimentation", ['Code','SESC_TXT']) as cursor:
    for row in cursor:
        CODE = row[0]
        SESC_TXT = row[1]
        SESC_Dictionary[CODE] = SESC_TXT
        Number_Processed = Number_Processed + 1'''
        exec(exec_string)

    SURFM_Dictionary = {}
    if arcpy.Exists(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surficial_Material"):
        Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surficial_Material").getOutput(0))
        Number_Processed = 0
        CPMessage('            - Reading Surficial Material domain table for ' + stra(Number_of_Features) + ' items.', 4)

        exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Tools_and_Templates_FGDB_Path'] + r"\Domain_Surficial_Material", ['Code','Short_Name']) as cursor:
    for row in cursor:
        CODE = row[0]
        SURFM_DOM = row[1]
        SURFM_Dictionary[CODE] = SURFM_DOM
        Number_Processed = Number_Processed + 1'''
        exec(exec_string)

    Number_of_Features = int(arcpy.GetCount_management(Processing_Variables['Long_Tbl']).getOutput(0))
    Number_Processed = 0

    #desc = arcpy.Describe(Processing_Variables['Long_Tbl'])
    #Long_Tbl_Shape_Field_Name = desc.ShapeFieldName
    #Long_Tbl_OID_Field_Name = desc.OIDFieldName
    #long_tbl_field_list = arcpy.ListFields(Processing_Variables['Long_Tbl'])
    #long_tbl_cursor_field_list = []
    #for long_tbl_field in long_tbl_field_list:
    #    if long_tbl_field.name not in [Long_Tbl_Shape_Field_Name,Long_Tbl_OID_Field_Name,'Shape_Length','Shape_Area']:
    #        long_tbl_cursor_field_list.append(long_tbl_field.name)

    long_tbl_cursor_field_list = []
    long_tbl_cursor_field_list.append('SHAPE@')
    long_tbl_cursor_field_list.append('BAPID')
    long_tbl_cursor_field_list.append('TEIS_ID')
    long_tbl_cursor_field_list.append('PROJPOLYID')
    long_tbl_cursor_field_list.append('FCODE')
    long_tbl_cursor_field_list.append('ECO_SEC')
    long_tbl_cursor_field_list.append('BGC_ZONE')
    long_tbl_cursor_field_list.append('BGC_SUBZON')
    long_tbl_cursor_field_list.append('BGC_VRT')
    long_tbl_cursor_field_list.append('BGC_PHASE')
    long_tbl_cursor_field_list.append('SDEC_1')
    long_tbl_cursor_field_list.append('SITEMC_S1')
    long_tbl_cursor_field_list.append('SITE_S1')
    long_tbl_cursor_field_list.append('SITE_M1A')
    long_tbl_cursor_field_list.append('SITE_M1B')
    long_tbl_cursor_field_list.append('STRCT_S1')
    long_tbl_cursor_field_list.append('STRCT_M1')
    long_tbl_cursor_field_list.append('STAND_A1')
    long_tbl_cursor_field_list.append('SERAL_1')
    long_tbl_cursor_field_list.append('SDEC_2')
    long_tbl_cursor_field_list.append('SITEMC_S2')
    long_tbl_cursor_field_list.append('SITE_S2')
    long_tbl_cursor_field_list.append('SITE_M2A')
    long_tbl_cursor_field_list.append('SITE_M2B')
    long_tbl_cursor_field_list.append('STRCT_S2')
    long_tbl_cursor_field_list.append('STRCT_M2')
    long_tbl_cursor_field_list.append('STAND_A2')
    long_tbl_cursor_field_list.append('SERAL_2')
    long_tbl_cursor_field_list.append('SDEC_3')
    long_tbl_cursor_field_list.append('SITEMC_S3')
    long_tbl_cursor_field_list.append('SITE_S3')
    long_tbl_cursor_field_list.append('SITE_M3A')
    long_tbl_cursor_field_list.append('SITE_M3B')
    long_tbl_cursor_field_list.append('STRCT_S3')
    long_tbl_cursor_field_list.append('STRCT_M3')
    long_tbl_cursor_field_list.append('STAND_A3')
    long_tbl_cursor_field_list.append('SERAL_3')
    long_tbl_cursor_field_list.append('SECL_1')
    long_tbl_cursor_field_list.append('SESUBCL_1')
    long_tbl_cursor_field_list.append('SECL_2')
    long_tbl_cursor_field_list.append('SESUBCL_2')
    long_tbl_cursor_field_list.append('SECL_3')
    long_tbl_cursor_field_list.append('SESUBCL_3')
    long_tbl_cursor_field_list.append('COND_1')
    long_tbl_cursor_field_list.append('VIAB_1')
    long_tbl_cursor_field_list.append('COND_2')
    long_tbl_cursor_field_list.append('VIAB_2')
    long_tbl_cursor_field_list.append('COND_3')
    long_tbl_cursor_field_list.append('VIAB_3')
    long_tbl_cursor_field_list.append('PRTFLG_1')
    long_tbl_cursor_field_list.append('TTEX_1A')
    long_tbl_cursor_field_list.append('TTEX_1B')
    long_tbl_cursor_field_list.append('TTEX_1C')
    long_tbl_cursor_field_list.append('SURFM_1')
    long_tbl_cursor_field_list.append('SURFM_Q1')
    long_tbl_cursor_field_list.append('SURFM_ST1')
    long_tbl_cursor_field_list.append('SURF_E1A')
    long_tbl_cursor_field_list.append('SURF_E1B')
    long_tbl_cursor_field_list.append('SURF_E1C')
    long_tbl_cursor_field_list.append('STTEX_1A')
    long_tbl_cursor_field_list.append('STTEX_1B')
    long_tbl_cursor_field_list.append('STTEX_1C')
    long_tbl_cursor_field_list.append('SSURFM_1')
    long_tbl_cursor_field_list.append('SSURFM_Q1')
    long_tbl_cursor_field_list.append('SSURFM_ST1')
    long_tbl_cursor_field_list.append('SSURF_E1A')
    long_tbl_cursor_field_list.append('SSURF_E1B')
    long_tbl_cursor_field_list.append('SSURF_E1C')
    long_tbl_cursor_field_list.append('PRTFLG_2')
    long_tbl_cursor_field_list.append('TTEX_2A')
    long_tbl_cursor_field_list.append('TTEX_2B')
    long_tbl_cursor_field_list.append('TTEX_2C')
    long_tbl_cursor_field_list.append('SURFM_2')
    long_tbl_cursor_field_list.append('SURFM_Q2')
    long_tbl_cursor_field_list.append('SURFM_ST2')
    long_tbl_cursor_field_list.append('SURF_E2A')
    long_tbl_cursor_field_list.append('SURF_E2B')
    long_tbl_cursor_field_list.append('SURF_E2C')
    long_tbl_cursor_field_list.append('STTEX_2A')
    long_tbl_cursor_field_list.append('STTEX_2B')
    long_tbl_cursor_field_list.append('STTEX_2C')
    long_tbl_cursor_field_list.append('SSURFM_2')
    long_tbl_cursor_field_list.append('SSURFM_Q2')
    long_tbl_cursor_field_list.append('SSURFM_ST2')
    long_tbl_cursor_field_list.append('SSURF_E2A')
    long_tbl_cursor_field_list.append('SSURF_E2B')
    long_tbl_cursor_field_list.append('SSURF_E2C')
    long_tbl_cursor_field_list.append('PRTFLG_3')
    long_tbl_cursor_field_list.append('TTEX_3A')
    long_tbl_cursor_field_list.append('TTEX_3B')
    long_tbl_cursor_field_list.append('TTEX_3C')
    long_tbl_cursor_field_list.append('SURFM_3')
    long_tbl_cursor_field_list.append('SURFM_Q3')
    long_tbl_cursor_field_list.append('SURFM_ST3')
    long_tbl_cursor_field_list.append('SURF_E3A')
    long_tbl_cursor_field_list.append('SURF_E3B')
    long_tbl_cursor_field_list.append('SURF_E3C')
    long_tbl_cursor_field_list.append('STTEX_3A')
    long_tbl_cursor_field_list.append('STTEX_3B')
    long_tbl_cursor_field_list.append('STTEX_3C')
    long_tbl_cursor_field_list.append('SSURFM_3')
    long_tbl_cursor_field_list.append('SSURFM_Q3')
    long_tbl_cursor_field_list.append('SSURFM_ST3')
    long_tbl_cursor_field_list.append('SSURF_E3A')
    long_tbl_cursor_field_list.append('SSURF_E3B')
    long_tbl_cursor_field_list.append('SSURF_E3C')
    long_tbl_cursor_field_list.append('GEOP_1')
    long_tbl_cursor_field_list.append('GEOP_Q1')
    long_tbl_cursor_field_list.append('GEOP_ST1')
    long_tbl_cursor_field_list.append('GEOP_INZ1')
    long_tbl_cursor_field_list.append('GEOP_SCM1A')
    long_tbl_cursor_field_list.append('GEOP_SCM1B')
    long_tbl_cursor_field_list.append('GEOP_SCM1C')
    long_tbl_cursor_field_list.append('GEOP_2')
    long_tbl_cursor_field_list.append('GEOP_Q2')
    long_tbl_cursor_field_list.append('GEOP_ST2')
    long_tbl_cursor_field_list.append('GEOP_INZ2')
    long_tbl_cursor_field_list.append('GEOP_SCM2A')
    long_tbl_cursor_field_list.append('GEOP_SCM2B')
    long_tbl_cursor_field_list.append('GEOP_SCM2C')
    long_tbl_cursor_field_list.append('GEOP_3')
    long_tbl_cursor_field_list.append('GEOP_Q3')
    long_tbl_cursor_field_list.append('GEOP_ST3')
    long_tbl_cursor_field_list.append('GEOP_INZ3')
    long_tbl_cursor_field_list.append('GEOP_SCM3A')
    long_tbl_cursor_field_list.append('GEOP_SCM3B')
    long_tbl_cursor_field_list.append('GEOP_SCM3C')
    long_tbl_cursor_field_list.append('DRAIN_1')
    long_tbl_cursor_field_list.append('DRAIN_SEP')
    long_tbl_cursor_field_list.append('DRAIN_2')
    long_tbl_cursor_field_list.append('SLPLL_1')
    long_tbl_cursor_field_list.append('SLPUL_1')
    long_tbl_cursor_field_list.append('SLPLL_2')
    long_tbl_cursor_field_list.append('SLPUL_2')
    long_tbl_cursor_field_list.append('SLPSTB_CLS')
    long_tbl_cursor_field_list.append('RDSTB_FLG')
    long_tbl_cursor_field_list.append('COMREL1_2')
    long_tbl_cursor_field_list.append('COMREL2_3')
    long_tbl_cursor_field_list.append('TDEC_1')
    long_tbl_cursor_field_list.append('TDEC_2')
    long_tbl_cursor_field_list.append('TDEC_3')
    long_tbl_cursor_field_list.append('BEDROCK_1')
    long_tbl_cursor_field_list.append('BEDROCK_2')
    long_tbl_cursor_field_list.append('BEDROCK_3')
    long_tbl_cursor_field_list.append('CAPLBL')
    long_tbl_cursor_field_list.append('CAPLBL_COR')
    long_tbl_cursor_field_list.append('POLY_COM')
    long_tbl_cursor_field_list.append('SMPL_TYPE')
    long_tbl_cursor_field_list.append('SFCERO_POT')
    long_tbl_cursor_field_list.append('LSSED_CLS')
    long_tbl_cursor_field_list.append('SESED_CLS')

    Long_Tbl_FieldName_Dict = {}
    x = 0
    for CurrentFieldName in long_tbl_cursor_field_list:
        Long_Tbl_FieldName_Dict[CurrentFieldName] = x
        x += 1

    #long_tbl_field_list = arcpy.ListFields(Processing_Variables['Long_Tbl'])
    #long_tbl_field_name_list = []
    #for long_tbl_field in long_tbl_field_list:
    #    long_tbl_field_name_list.append(long_tbl_field.name)
    #for field_name in long_tbl_cursor_field_list:
    #    if field_name not in long_tbl_field_name_list:
    #        print "Cursor field not found in long tbl: " + field_name

    #desc = arcpy.Describe(Processing_Variables['Short_Tbl'])
    #Short_Tbl_Shape_Field_Name = desc.ShapeFieldName
    #Short_Tbl_OID_Field_Name = desc.OIDFieldName
    #short_tbl_field_list = arcpy.ListFields(Processing_Variables['Short_Tbl'])
    #short_tbl_cursor_field_list = ['SHAPE@']
    #for short_tbl_field in short_tbl_field_list:
    #    if short_tbl_field.name not in [Short_Tbl_Shape_Field_Name,Short_Tbl_OID_Field_Name,'Shape_Length','Shape_Area']:
    #        short_tbl_cursor_field_list.append(short_tbl_field.name)

    short_tbl_cursor_field_list = []
    short_tbl_cursor_field_list.append('SHAPE@')
    short_tbl_cursor_field_list.append('TEIS_ID')
    short_tbl_cursor_field_list.append('PROJPOLYID')
    short_tbl_cursor_field_list.append('BAPID')
    short_tbl_cursor_field_list.append('MOD_DATE')
    short_tbl_cursor_field_list.append('PROJ_TYPE')
    short_tbl_cursor_field_list.append('PROJ_NAME')
    short_tbl_cursor_field_list.append('PROJ_SCALE')
    short_tbl_cursor_field_list.append('ECO_SEC')
    short_tbl_cursor_field_list.append('BGC_LBL')
    short_tbl_cursor_field_list.append('ECOLBL_1')
    short_tbl_cursor_field_list.append('SDEC_1')
    short_tbl_cursor_field_list.append('SITES_LBL1')
    short_tbl_cursor_field_list.append('SITEMCLBL1')
    short_tbl_cursor_field_list.append('SEILBL_1')
    short_tbl_cursor_field_list.append('SECL_1')
    short_tbl_cursor_field_list.append('SESUBCL_1')
    short_tbl_cursor_field_list.append('COND_1')
    short_tbl_cursor_field_list.append('VIAB_1')
    short_tbl_cursor_field_list.append('STRCTLBL_1')
    short_tbl_cursor_field_list.append('ECOLBL_2')
    short_tbl_cursor_field_list.append('SDEC_2')
    short_tbl_cursor_field_list.append('SITES_LBL2')
    short_tbl_cursor_field_list.append('SITEMCLBL2')
    short_tbl_cursor_field_list.append('SEILBL_2')
    short_tbl_cursor_field_list.append('SECL_2')
    short_tbl_cursor_field_list.append('SESUBCL_2')
    short_tbl_cursor_field_list.append('COND_2')
    short_tbl_cursor_field_list.append('VIAB_2')
    short_tbl_cursor_field_list.append('STRCTLBL_2')
    short_tbl_cursor_field_list.append('ECOLBL_3')
    short_tbl_cursor_field_list.append('SDEC_3')
    short_tbl_cursor_field_list.append('SITES_LBL3')
    short_tbl_cursor_field_list.append('SITEMCLBL3')
    short_tbl_cursor_field_list.append('SEILBL_3')
    short_tbl_cursor_field_list.append('SECL_3')
    short_tbl_cursor_field_list.append('SESUBCL_3')
    short_tbl_cursor_field_list.append('COND_3')
    short_tbl_cursor_field_list.append('VIAB_3')
    short_tbl_cursor_field_list.append('STRCTLBL_3')
    short_tbl_cursor_field_list.append('TER_LBL')
    short_tbl_cursor_field_list.append('TDEC_1')
    short_tbl_cursor_field_list.append('TERLBL_1A')
    short_tbl_cursor_field_list.append('TERLBL_1B')
    short_tbl_cursor_field_list.append('BEDROCK_1')
    short_tbl_cursor_field_list.append('COMREL1_2')
    short_tbl_cursor_field_list.append('TDEC_2')
    short_tbl_cursor_field_list.append('TERLBL_2A')
    short_tbl_cursor_field_list.append('TERLBL_2B')
    short_tbl_cursor_field_list.append('BEDROCK_2')
    short_tbl_cursor_field_list.append('COMREL2_3')
    short_tbl_cursor_field_list.append('TDEC_3')
    short_tbl_cursor_field_list.append('TERLBL_3A')
    short_tbl_cursor_field_list.append('TERLBL_3B')
    short_tbl_cursor_field_list.append('BEDROCK_3')
    short_tbl_cursor_field_list.append('GEOP_LBL')
    short_tbl_cursor_field_list.append('DRAIN_LBL')
    short_tbl_cursor_field_list.append('SLOPE_LBL')
    short_tbl_cursor_field_list.append('SLPSTBCLSR')
    short_tbl_cursor_field_list.append('SFCERO_POT')
    short_tbl_cursor_field_list.append('LSSED_CLS')
    short_tbl_cursor_field_list.append('SESED_CLS')
    short_tbl_cursor_field_list.append('SSC_TXT')
    short_tbl_cursor_field_list.append('SEP_TXT')
    short_tbl_cursor_field_list.append('LISSC_TXT')
    short_tbl_cursor_field_list.append('SESC_TXT')
    short_tbl_cursor_field_list.append('POLY_TEX')
    short_tbl_cursor_field_list.append('POLY_THK')
    short_tbl_cursor_field_list.append('SURFM_DOM')
    short_tbl_cursor_field_list.append('THEME_1')
    short_tbl_cursor_field_list.append('THEME_2')
    short_tbl_cursor_field_list.append('THEME_3')
    short_tbl_cursor_field_list.append('POLY_COM')
    short_tbl_cursor_field_list.append('FCODE')
    short_tbl_cursor_field_list.append('SMPL_TYPE')
    short_tbl_cursor_field_list.append('LEGEND_COM')
    short_tbl_cursor_field_list.append('LEGEND_URL')
    short_tbl_cursor_field_list.append('ORIG_LBL')
    short_tbl_cursor_field_list.append('PSTBCLS_TP')
    short_tbl_cursor_field_list.append('ERROR_SUM')

    CPMessage('            - Temporarily removing attribute indexes from Short Table feature class', 4)
    idx_list = [idx for idx in arcpy.ListIndexes(Processing_Variables['Short_Tbl'])]
    for idx in idx_list:
        field_list = [f for f in idx.fields]
        index_can_be_removed = True
        for field in field_list:
            if field.type in ['OID','Geometry']:
                index_can_be_removed = False
        if index_can_be_removed:
            field_name_list = [f.name for f in idx.fields]
            field_name_list_string = ''
            for field_name in field_name_list:
                field_name_list_string += field_name + ", "
            field_name_list_string = field_name_list_string[:-2]
            CPMessage('                - Removing attribute index ' + idx.name + ' on field(s) ' + field_name_list_string, 4)
            arcpy.RemoveIndex_management(Processing_Variables['Short_Tbl'],idx.name)
            
    short_cursor = arcpy.da.InsertCursor(Processing_Variables['Short_Tbl'],short_tbl_cursor_field_list)
    
    CPMessage('            - Reading Long Table and writing Short Table for ' + stra(Number_of_Features) + ' items.', 4)

    dtCalcStart = time.time()
    exec_string = '''with arcpy.da.SearchCursor(Processing_Variables['Long_Tbl'], long_tbl_cursor_field_list) as cursor:
    for long_row in cursor:
        if (0 < Number_Processed <= 10000 and Number_Processed % 1000 == 0) or (10000 < Number_Processed <= 100000 and Number_Processed % 10000 == 0) or (100000 < Number_Processed and Number_Processed % 100000 == 0):
            dtCalcNow = time.time()
            dtCalcElapsed = dtCalcNow - dtCalcStart
            dtCalcRemaining = (dtCalcElapsed / Number_Processed) * (Number_of_Features - Number_Processed)
            CPMessage('                - Processed ' + stra(Number_Processed) + ' of ' + stra(Number_of_Features) + ' (' + SanitizeElapsedTime(dtCalcElapsed)[:-1] + ' elapsed, ' + SanitizeElapsedTime(dtCalcRemaining)[:-1] + ' remaining)', 4)

        BAPID = long_row[Long_Tbl_FieldName_Dict['BAPID']]
        if BAPID in Processing_Variables['BAPID_LIST'] or Processing_Variables['BAPID_LIST'] == ['ALL']:

            s_SHAPE = long_row[Long_Tbl_FieldName_Dict['SHAPE@']]

            s_BAPID = BAPID

            TEIS_ID = long_row[Long_Tbl_FieldName_Dict['TEIS_ID']]
            s_TEIS_ID = TEIS_ID

            PROJPOLYID = stra(long_row[Long_Tbl_FieldName_Dict['PROJPOLYID']]).rstrip(' ').replace('None','')
            s_PROJPOLYID = PROJPOLYID

            if TEIS_ID in ErrorSum_Dictionary:
                s_ERROR_SUM = ErrorSum_Dictionary[TEIS_ID][:500]
            else:
                s_ERROR_SUM = ''

            if BAPID in ProjDetails_Dictionary:
                if stra(ProjDetails_Dictionary[BAPID]['MOD_DATE']) not in ['',' ','None']:
                    s_MOD_DATE = ProjDetails_Dictionary[BAPID]['MOD_DATE']
                else:
                    s_MOD_DATE = None
                s_PROJ_TYPE = stra(ProjDetails_Dictionary[BAPID]['PROJ_TYPE']).rstrip(' ').replace('None','')
                if stra(ProjDetails_Dictionary[BAPID]['PROJ_SCALE']) not in ['',' ','None']:
                    s_PROJ_SCALE = ProjDetails_Dictionary[BAPID]['PROJ_SCALE']
                else:
                    s_PROJ_SCALE = None
                s_PROJ_NAME = stra(ProjDetails_Dictionary[BAPID]['PROJ_NAME']).rstrip(' ').replace('None','')
                s_LEGEND_COM = stra(ProjDetails_Dictionary[BAPID]['LEGEND_COM']).rstrip(' ').replace('None','')
                s_LEGEND_URL = stra(ProjDetails_Dictionary[BAPID]['LEGEND_URL']).rstrip(' ').replace('None','')
            else:
                s_MOD_DATE = None
                s_PROJ_TYPE = ''
                s_PROJ_SCALE = None
                s_PROJ_NAME = ''
                s_LEGEND_COM = ''
                s_LEGEND_URL = ''

            FCODE = stra(long_row[Long_Tbl_FieldName_Dict['FCODE']]).rstrip(' ').replace('None','')
            s_FCODE = FCODE

            ECO_SEC = stra(long_row[Long_Tbl_FieldName_Dict['ECO_SEC']]).rstrip(' ').replace('None','')
            s_ECO_SEC = ECO_SEC

            # BGC_LBL - Equals <Bgc_zone> + <Bgc_Subzon> + <Bgc_Vrt> + <Bgc_Phase>
            BGC_ZONE = stra(long_row[Long_Tbl_FieldName_Dict['BGC_ZONE']]).rstrip(' ').replace('None','')
            BGC_SUBZON = stra(long_row[Long_Tbl_FieldName_Dict['BGC_SUBZON']]).rstrip(' ').replace('None','')
            BGC_VRT = stra(long_row[Long_Tbl_FieldName_Dict['BGC_VRT']]).rstrip(' ').replace('None','')
            BGC_PHASE = stra(long_row[Long_Tbl_FieldName_Dict['BGC_PHASE']]).rstrip(' ').replace('None','')

            s_BGC_LBL = BGC_ZONE + BGC_SUBZON + BGC_VRT + BGC_PHASE
            s_BGC_LBL = s_BGC_LBL.replace(' ', '')
            s_BGC_LBL = s_BGC_LBL.replace('None', '')        

            # ECOLBL_1 - Equals <Sdec1> + <SiteMC_S1> + "[" + <Site_S1> + "]" + <Site_M1a> + <Site_M1b> + <Strct_S1> + <Strct_M1> + <Stand_A1> + ":" + <Seral_1> 
            SDEC_1 = stra(long_row[Long_Tbl_FieldName_Dict['SDEC_1']]).rstrip(' ').replace('None','')
            SITEMC_S1 = stra(long_row[Long_Tbl_FieldName_Dict['SITEMC_S1']]).rstrip(' ').replace('None','')
            SITE_S1 = stra(long_row[Long_Tbl_FieldName_Dict['SITE_S1']]).rstrip(' ').replace('None','')
            SITE_M1A = stra(long_row[Long_Tbl_FieldName_Dict['SITE_M1A']]).rstrip(' ').replace('None','')
            SITE_M1B = stra(long_row[Long_Tbl_FieldName_Dict['SITE_M1B']]).rstrip(' ').replace('None','')
            STRCT_S1 = stra(long_row[Long_Tbl_FieldName_Dict['STRCT_S1']]).rstrip(' ').replace('None','')
            STRCT_M1 = stra(long_row[Long_Tbl_FieldName_Dict['STRCT_M1']]).rstrip(' ').replace('None','')
            STAND_A1 = stra(long_row[Long_Tbl_FieldName_Dict['STAND_A1']]).rstrip(' ').replace('None','')
            SERAL_1 = stra(long_row[Long_Tbl_FieldName_Dict['SERAL_1']]).rstrip(' ').replace('None','')

            if SDEC_1 in ['1','2','3','4','5','6','7','8','9','10']:
                s_SDEC_1 = int(SDEC_1)
            else:
                s_SDEC_1 = None

            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S1 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S1 = '0' + SITE_S1

            s_ECOLBL_1 = SITEMC_S1 + r"[" + SITE_S1 + r"]" + SITE_M1A + SITE_M1B + STRCT_S1 + STRCT_M1 + STAND_A1 + r":" + SERAL_1
            s_ECOLBL_1 = s_ECOLBL_1.replace(' ', '')
            s_ECOLBL_1 = s_ECOLBL_1.replace('None', '')        
            s_ECOLBL_1 = s_ECOLBL_1.replace(r'[]', '')
            s_ECOLBL_1 = s_ECOLBL_1.strip(r':')

            # ECOLBL_2 - Equals <Sdec2> + <SiteMC_S2> + "[" + <Site_S2> + "]" + <Site_M2a> + <Site_M2b> + <Strct_S2> + <Strct_M2> + <Stand_A2> + ":" + <Seral_2> 
            SDEC_2 = stra(long_row[Long_Tbl_FieldName_Dict['SDEC_2']]).rstrip(' ').replace('None','')
            SITEMC_S2 = stra(long_row[Long_Tbl_FieldName_Dict['SITEMC_S2']]).rstrip(' ').replace('None','')
            SITE_S2 = stra(long_row[Long_Tbl_FieldName_Dict['SITE_S2']]).rstrip(' ').replace('None','')
            SITE_M2A = stra(long_row[Long_Tbl_FieldName_Dict['SITE_M2A']]).rstrip(' ').replace('None','')
            SITE_M2B = stra(long_row[Long_Tbl_FieldName_Dict['SITE_M2B']]).rstrip(' ').replace('None','')
            STRCT_S2 = stra(long_row[Long_Tbl_FieldName_Dict['STRCT_S2']]).rstrip(' ').replace('None','')
            STRCT_M2 = stra(long_row[Long_Tbl_FieldName_Dict['STRCT_M2']]).rstrip(' ').replace('None','')
            STAND_A2 = stra(long_row[Long_Tbl_FieldName_Dict['STAND_A2']]).rstrip(' ').replace('None','')
            SERAL_2 = stra(long_row[Long_Tbl_FieldName_Dict['SERAL_2']]).rstrip(' ').replace('None','')

            if SDEC_2 in ['1','2','3','4','5','6','7','8','9','10']:
                s_SDEC_2 = int(SDEC_2)
            else:
                s_SDEC_2 = None

            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S2 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S2 = '0' + SITE_S2

            s_ECOLBL_2 = SITEMC_S2 + r"[" + SITE_S2 + r"]" + SITE_M2A + SITE_M2B + STRCT_S2 + STRCT_M2 + STAND_A2 + r":" + SERAL_2
            s_ECOLBL_2 = s_ECOLBL_2.replace(' ', '')
            s_ECOLBL_2 = s_ECOLBL_2.replace('None', '')        
            s_ECOLBL_2 = s_ECOLBL_2.replace(r'[]', '')
            s_ECOLBL_2 = s_ECOLBL_2.strip(r':')

            # ECOLBL_3 - Equals <Sdec3> + <SiteMC_S3> + "[" + <Site_S3> + "]" + <Site_M3a> + <Site_M3b> + <Strct_S3> + <Strct_M3> + <Stand_A3> + ":" + <Seral_3> 
            SDEC_3 = stra(long_row[Long_Tbl_FieldName_Dict['SDEC_3']]).rstrip(' ').replace('None','')
            SITEMC_S3 = stra(long_row[Long_Tbl_FieldName_Dict['SITEMC_S3']]).rstrip(' ').replace('None','')
            SITE_S3 = stra(long_row[Long_Tbl_FieldName_Dict['SITE_S3']]).rstrip(' ').replace('None','')
            SITE_M3A = stra(long_row[Long_Tbl_FieldName_Dict['SITE_M3A']]).rstrip(' ').replace('None','')
            SITE_M3B = stra(long_row[Long_Tbl_FieldName_Dict['SITE_M3B']]).rstrip(' ').replace('None','')
            STRCT_S3 = stra(long_row[Long_Tbl_FieldName_Dict['STRCT_S3']]).rstrip(' ').replace('None','')
            STRCT_M3 = stra(long_row[Long_Tbl_FieldName_Dict['STRCT_M3']]).rstrip(' ').replace('None','')
            STAND_A3 = stra(long_row[Long_Tbl_FieldName_Dict['STAND_A3']]).rstrip(' ').replace('None','')
            SERAL_3 = stra(long_row[Long_Tbl_FieldName_Dict['SERAL_3']]).rstrip(' ').replace('None','')

            if SDEC_3 in ['1','2','3','4','5','6','7','8','9','10']:
                s_SDEC_3 = int(SDEC_3)
            else:
                s_SDEC_3 = None

            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S3 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S3 = '0' + SITE_S3


            s_ECOLBL_3 = SITEMC_S3 + r"[" + SITE_S3 + r"]" + SITE_M3A + SITE_M3B + STRCT_S3 + STRCT_M3 + STAND_A3 + r":" + SERAL_3
            s_ECOLBL_3 = s_ECOLBL_3.replace(' ', '')
            s_ECOLBL_3 = s_ECOLBL_3.replace('None', '')        
            s_ECOLBL_3 = s_ECOLBL_3.replace(r'[]', '')
            s_ECOLBL_3 = s_ECOLBL_3.strip(r':')

            # SITES_LBL1 - Equals "[ "+ <Site_S1> + "]" + ":"+ <Seral_1> 
            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S1 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S1 = '0' + SITE_S1

            # used to be: SITES_LBL1 = r"[" + SITE_S1 + r"]" + r":" + SERAL_1  (changed March 28 2011 by Jeff K)
            s_SITES_LBL1 = SITE_S1 + r":" + SERAL_1
            s_SITES_LBL1 = s_SITES_LBL1.replace(' ', '')
            s_SITES_LBL1 = s_SITES_LBL1.replace('None', '')        
            s_SITES_LBL1 = s_SITES_LBL1.replace(r'[]', '')
            s_SITES_LBL1 = s_SITES_LBL1.strip(r':')

            # SITES_LBL2 - Equals "[ "+ <Site_S2> + "]" + ":"+ <Seral_2> 
            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S2 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S2 = '0' + SITE_S2        

            # used to be: SITES_LBL2 = r"[" + SITE_S2 + r"]" + r":" + SERAL_2  (changed March 28 2011 by Jeff K)
            s_SITES_LBL2 = SITE_S2 + r":" + SERAL_2
            s_SITES_LBL2 = s_SITES_LBL2.replace(' ', '')
            s_SITES_LBL2 = s_SITES_LBL2.replace('None', '')        
            s_SITES_LBL2 = s_SITES_LBL2.replace(r'[]', '')
            s_SITES_LBL2 = s_SITES_LBL2.strip(r':')

            # SITES_LBL3 - Equals "[ "+ <Site_S3> + "]" + ":"+ <Seral_3>  
            # Padded zeros.  There will likely be a domain clean up in the future that will make this irrelivant.
            if SITE_S3 in ['0','1', '2', '3', '4', '5', '6', '7', '8', '9']:
                SITE_S3 = '0' + SITE_S3        

            # used to be: SITES_LBL3 = r"[" + SITE_S3 + r"]" + r":" + SERAL_3  (changed March 28 2011 by Jeff K)
            s_SITES_LBL3 = SITE_S3 + r":" + SERAL_3
            s_SITES_LBL3 = s_SITES_LBL3.replace(' ', '')
            s_SITES_LBL3 = s_SITES_LBL3.replace('None', '')        
            s_SITES_LBL3 = s_SITES_LBL3.replace(r'[]', '')
            s_SITES_LBL3 = s_SITES_LBL3.strip(r':')


            # SITEMCLBL1 - Equals <siteMC_1 > + ":" + <Seral_1 >
            ## Actually SITEMC_S1
            # used to be: SITEMCLBL1 = r"[" + SITEMC_S1 + r"]" + r":" + SERAL_1  (changed March 28 2011 by Jeff K)
            s_SITEMCLBL1 = SITEMC_S1 + r":" + SERAL_1
            s_SITEMCLBL1 = s_SITEMCLBL1.replace(' ', '')
            s_SITEMCLBL1 = s_SITEMCLBL1.replace('None', '')        
            s_SITEMCLBL1 = s_SITEMCLBL1.replace(r'[]', '')
            s_SITEMCLBL1 = s_SITEMCLBL1.strip(r':')

            # SITEMCLBL2 - Equals <siteMC_2 > + ":" + <Seral_2 >
            ## Actually SITEMC_S2
            # used to be: SITEMCLBL2 = r"[" + SITEMC_S2 + r"]" + r":" + SERAL_2 (changed March 28 2011 by Jeff K)
            s_SITEMCLBL2 = SITEMC_S2 + r":" + SERAL_2
            s_SITEMCLBL2 = s_SITEMCLBL2.replace(' ', '')
            s_SITEMCLBL2 = s_SITEMCLBL2.replace('None', '')        
            s_SITEMCLBL2 = s_SITEMCLBL2.replace(r'[]', '')
            s_SITEMCLBL2 = s_SITEMCLBL2.strip(r':')

            # SITEMCLBL3 - Equals <siteMC_3 > + ":" + <Seral_3 >
            ## Actually SITEMC_S3
            # used to be: SITEMCLBL3 = r"[" + SITEMC_S3 + r"]" + r":" + SERAL_3 (changed March 28 2011 by Jeff K)
            s_SITEMCLBL3 = SITEMC_S3 + r":" + SERAL_3
            s_SITEMCLBL3 = s_SITEMCLBL3.replace(' ', '')
            s_SITEMCLBL3 = s_SITEMCLBL3.replace('None', '')        
            s_SITEMCLBL3 = s_SITEMCLBL3.replace(r'[]', '')
            s_SITEMCLBL3 = s_SITEMCLBL3.strip(r':')


            # SEILBL_1 - Equals <SECl_1> + ":"+ <SEsubcl_1> 
            SECL_1 = stra(long_row[Long_Tbl_FieldName_Dict['SECL_1']]).rstrip(' ').replace('None','')
            SESUBCL_1 = stra(long_row[Long_Tbl_FieldName_Dict['SESUBCL_1']]).rstrip(' ').replace('None','')

            s_SEILBL_1 = SECL_1 + r":" + SESUBCL_1
            s_SEILBL_1 = s_SEILBL_1.replace(' ', '')
            s_SEILBL_1 = s_SEILBL_1.replace('None', '')        
            s_SEILBL_1 = s_SEILBL_1.strip(r':')
            s_SECL_1 = SECL_1
            s_SESUBCL_1 = SESUBCL_1

            # SEILBL_2 - Equals <SECl_2> + ":"+ <SEsubcl_2> 
            SECL_2 = stra(long_row[Long_Tbl_FieldName_Dict['SECL_2']]).rstrip(' ').replace('None','')
            SESUBCL_2 = stra(long_row[Long_Tbl_FieldName_Dict['SESUBCL_2']]).rstrip(' ').replace('None','')

            s_SEILBL_2 = SECL_2 + r":" + SESUBCL_2
            s_SEILBL_2 = s_SEILBL_2.replace(' ', '')
            s_SEILBL_2 = s_SEILBL_2.replace('None', '')        
            s_SEILBL_2 = s_SEILBL_2.strip(r':')
            s_SECL_2 = SECL_2
            s_SESUBCL_2 = SESUBCL_2


            # SEILBL_3 - Equals <SECl_1> + ":"+ <SEsubcl_3> 
            SECL_3 = stra(long_row[Long_Tbl_FieldName_Dict['SECL_3']]).rstrip(' ').replace('None','')
            SESUBCL_3 = stra(long_row[Long_Tbl_FieldName_Dict['SESUBCL_3']]).rstrip(' ').replace('None','')

            s_SEILBL_3 = SECL_3 + r":" + SESUBCL_3
            s_SEILBL_3 = s_SEILBL_3.replace(' ', '')
            s_SEILBL_3 = s_SEILBL_3.replace('None', '')        
            s_SEILBL_3 = s_SEILBL_3.strip(r':')
            s_SECL_3 = SECL_3
            s_SESUBCL_3 = SESUBCL_3

            s_COND_1 = stra(long_row[Long_Tbl_FieldName_Dict['COND_1']]).rstrip(' ').replace('None','')
            s_VIAB_1 = stra(long_row[Long_Tbl_FieldName_Dict['VIAB_1']]).rstrip(' ').replace('None','')
            s_COND_2 = stra(long_row[Long_Tbl_FieldName_Dict['COND_2']]).rstrip(' ').replace('None','')
            s_VIAB_2 = stra(long_row[Long_Tbl_FieldName_Dict['VIAB_2']]).rstrip(' ').replace('None','')
            s_COND_3 = stra(long_row[Long_Tbl_FieldName_Dict['COND_3']]).rstrip(' ').replace('None','')
            s_VIAB_3 = stra(long_row[Long_Tbl_FieldName_Dict['VIAB_3']]).rstrip(' ').replace('None','')


            # STRCTLBL_1 - <Strct_S1> + <Strct_M1>+ <Stand_A1>
            s_STRCTLBL_1 = STRCT_S1 + STRCT_M1 + STAND_A1
            s_STRCTLBL_1 = s_STRCTLBL_1.replace(' ', '')
            s_STRCTLBL_1 = s_STRCTLBL_1.replace('None', '')        

            # STRCTLBL_2 - <Strct_S2> + <Strct_M2>+ <Stand_A2>
            s_STRCTLBL_2 = STRCT_S2 + STRCT_M2 + STAND_A2
            s_STRCTLBL_2 = s_STRCTLBL_2.replace(' ', '')
            s_STRCTLBL_2 = s_STRCTLBL_2.replace('None', '')        

            # STRCTLBL_3 - <Strct_S3> + <Strct_M3>+ <Stand_A3>
            s_STRCTLBL_3 = STRCT_S3 + STRCT_M3 + STAND_A3
            s_STRCTLBL_3 = s_STRCTLBL_3.replace(' ', '')
            s_STRCTLBL_3 = s_STRCTLBL_3.replace('None', '')        


            # TERLBL_1A (Detailed Polys - Calculated)
            # Equals <Prtflg_1> + <Ttex_1> + <Surfm_1> + "(" + <Surfm_Q1>+ ")" + <Surfm_St1> + <Surf_E1>
            PRTFLG_1   = stra(long_row[Long_Tbl_FieldName_Dict['PRTFLG_1']]).rstrip(' ').replace('None','')
            ## TTEX_1 will be discontinued from the data model in the future.
            ## TTEX_1     = stra(long_row.GetValue("TTEX_1"))
            TTEX_1A     = stra(long_row[Long_Tbl_FieldName_Dict['TTEX_1A']]).rstrip(' ').replace('None','')
            TTEX_1B     = stra(long_row[Long_Tbl_FieldName_Dict['TTEX_1B']]).rstrip(' ').replace('None','')
            TTEX_1C     = stra(long_row[Long_Tbl_FieldName_Dict['TTEX_1C']]).rstrip(' ').replace('None','')
            TTEX_1 = TTEX_1C + TTEX_1B + TTEX_1A
            SURFM_1    = stra(long_row[Long_Tbl_FieldName_Dict['SURFM_1']]).rstrip(' ').replace('None','')
            SURFM_Q1   = stra(long_row[Long_Tbl_FieldName_Dict['SURFM_Q1']]).rstrip(' ').replace('None','')
            SURFM_ST1  = stra(long_row[Long_Tbl_FieldName_Dict['SURFM_ST1']]).rstrip(' ').replace('None','')
            ## SURF_E1 will be discontinued from the data model in the future.
            ## SURF_E1    = stra(long_row.GetValue("SURF_E1"))
            SURF_E1A    = stra(long_row[Long_Tbl_FieldName_Dict['SURF_E1A']]).rstrip(' ').replace('None','')
            SURF_E1B    = stra(long_row[Long_Tbl_FieldName_Dict['SURF_E1B']]).rstrip(' ').replace('None','')
            SURF_E1C    = stra(long_row[Long_Tbl_FieldName_Dict['SURF_E1C']]).rstrip(' ').replace('None','')
            SURF_E1     = SURF_E1A + SURF_E1B + SURF_E1C

            TERLBL_1A = PRTFLG_1 + TTEX_1 + SURFM_1 + "(" + SURFM_Q1 + ")" + SURFM_ST1 + SURF_E1

            TERLBL_1A = TERLBL_1A.replace(' ', '')
            TERLBL_1A = TERLBL_1A.replace('None', '')
            TERLBL_1A = TERLBL_1A.replace('()', '')
            s_TERLBL_1A = TERLBL_1A

            # TERLBL_1B (Detailed Polys - Calculated)
            # Equals <Sttex_1> + <Ssurfm_1> + "(" + <Ssurfm_Q1> + ")" + <Ssurfm_St1> + <Ssurf_E1>
            ## STTEX_1 will be discontinued from the data model in the future.
            ## STTEX_1 = stra(long_row.GetValue("STTEX_1"))
            STTEX_1A = stra(long_row[Long_Tbl_FieldName_Dict['STTEX_1A']]).rstrip(' ').replace('None','')
            STTEX_1B = stra(long_row[Long_Tbl_FieldName_Dict['STTEX_1B']]).rstrip(' ').replace('None','')
            STTEX_1C = stra(long_row[Long_Tbl_FieldName_Dict['STTEX_1C']]).rstrip(' ').replace('None','')
            STTEX_1 = STTEX_1C + STTEX_1B + STTEX_1A
            SSURFM_1 = stra(long_row[Long_Tbl_FieldName_Dict['SSURFM_1']]).rstrip(' ').replace('None','')
            SSURFM_Q1 = stra(long_row[Long_Tbl_FieldName_Dict['SSURFM_Q1']]).rstrip(' ').replace('None','')
            SSURFM_ST1 = stra(long_row[Long_Tbl_FieldName_Dict['SSURFM_ST1']]).rstrip(' ').replace('None','')
            ## SSURF_E1 will be discontinued from the data model in the future.
            ## SSURF_E1 = stra(long_row.GetValue("SSURF_E1"))
            SSURF_E1A = stra(long_row[Long_Tbl_FieldName_Dict['SSURF_E1A']]).rstrip(' ').replace('None','')
            SSURF_E1B = stra(long_row[Long_Tbl_FieldName_Dict['SSURF_E1B']]).rstrip(' ').replace('None','')
            SSURF_E1C = stra(long_row[Long_Tbl_FieldName_Dict['SSURF_E1C']]).rstrip(' ').replace('None','')
            SSURF_E1 = SSURF_E1A + SSURF_E1B + SSURF_E1C

            TERLBL_1B = STTEX_1 + SSURFM_1 + "(" + SSURFM_Q1 + ")" + SSURFM_ST1 + SSURF_E1

            TERLBL_1B = TERLBL_1B.replace(' ', '')
            TERLBL_1B = TERLBL_1B.replace('None', '')
            TERLBL_1B = TERLBL_1B.replace('()', '')
            s_TERLBL_1B = TERLBL_1B


            # TERLBL_2A (Detailed Polys - Calculated)
            # Equals <Prtflg_2> + <Ttex_2> + <Surfm_2> +  "(" + <Surfm_Q2> + ")" + <Surfm_St2> + <Surf_E2>
            PRTFLG_2 = stra(long_row[Long_Tbl_FieldName_Dict['PRTFLG_2']]).rstrip(' ').replace('None','')
            ## TTEX_2 will be discontinued from the data model in the future.
            ## TTEX_2 = stra(long_row.GetValue("TTEX_2"))
            TTEX_2A = stra(long_row[Long_Tbl_FieldName_Dict['TTEX_2A']]).rstrip(' ').replace('None','')
            TTEX_2B = stra(long_row[Long_Tbl_FieldName_Dict['TTEX_2B']]).rstrip(' ').replace('None','')
            TTEX_2C = stra(long_row[Long_Tbl_FieldName_Dict['TTEX_2C']]).rstrip(' ').replace('None','')
            TTEX_2 = TTEX_2C + TTEX_2B + TTEX_2A
            SURFM_2 = stra(long_row[Long_Tbl_FieldName_Dict['SURFM_2']]).rstrip(' ').replace('None','')
            SURFM_Q2 = stra(long_row[Long_Tbl_FieldName_Dict['SURFM_Q2']]).rstrip(' ').replace('None','')
            SURFM_ST2 = stra(long_row[Long_Tbl_FieldName_Dict['SURFM_ST2']]).rstrip(' ').replace('None','')
            ## SURF_E2 will be discontinued from the data model in the future.
            ## SURF_E2 = stra(long_row.GetValue("SURF_E2"))
            SURF_E2A = stra(long_row[Long_Tbl_FieldName_Dict['SURF_E2A']]).rstrip(' ').replace('None','')
            SURF_E2B = stra(long_row[Long_Tbl_FieldName_Dict['SURF_E2B']]).rstrip(' ').replace('None','')
            SURF_E2C = stra(long_row[Long_Tbl_FieldName_Dict['SURF_E2C']]).rstrip(' ').replace('None','')
            SURF_E2 = SURF_E2A + SURF_E2B + SURF_E2C

            TERLBL_2A = PRTFLG_2 + TTEX_2 + SURFM_2 +  "(" + SURFM_Q2 + ")" + SURFM_ST2 + SURF_E2

            TERLBL_2A = TERLBL_2A.replace(' ', '')
            TERLBL_2A = TERLBL_2A.replace('None', '')
            TERLBL_2A = TERLBL_2A.replace('()', '')
            s_TERLBL_2A = TERLBL_2A


            # TERLBL_2B (Detailed Polys - Calculated)
            # Equals <Sttex_2> + <Ssurfm_2> + "(" + <Ssurfm_Q2>+ ")" + <Ssurfm_St2> + <Ssurf_E2>
            ## STTEX_2 will be discontinued from the data model in the future.
            ## STTEX_2 = stra(long_row.GetValue("STTEX_2"))
            STTEX_2A = stra(long_row[Long_Tbl_FieldName_Dict['STTEX_2A']]).rstrip(' ').replace('None','')
            STTEX_2B = stra(long_row[Long_Tbl_FieldName_Dict['STTEX_2B']]).rstrip(' ').replace('None','')
            STTEX_2C = stra(long_row[Long_Tbl_FieldName_Dict['STTEX_2C']]).rstrip(' ').replace('None','')
            STTEX_2 = STTEX_2C + STTEX_2B + STTEX_2A
            SSURFM_2 = stra(long_row[Long_Tbl_FieldName_Dict['SSURFM_2']]).rstrip(' ').replace('None','')
            SSURFM_Q2 = stra(long_row[Long_Tbl_FieldName_Dict['SSURFM_Q2']]).rstrip(' ').replace('None','')
            SSURFM_ST2 = stra(long_row[Long_Tbl_FieldName_Dict['SSURFM_ST2']]).rstrip(' ').replace('None','')
            ## SSURF_E2 will be discontinued from the data model in the future.
            ## SSURF_E2 = stra(long_row.GetValue("SSURF_E2"))
            SSURF_E2A = stra(long_row[Long_Tbl_FieldName_Dict['SSURF_E2A']]).rstrip(' ').replace('None','')
            SSURF_E2B = stra(long_row[Long_Tbl_FieldName_Dict['SSURF_E2B']]).rstrip(' ').replace('None','')
            SSURF_E2C = stra(long_row[Long_Tbl_FieldName_Dict['SSURF_E2C']]).rstrip(' ').replace('None','')
            SSURF_E2 = SSURF_E2A + SSURF_E2B + SSURF_E2C

            TERLBL_2B = STTEX_2 + SSURFM_2 + "(" + SSURFM_Q2 + ")" + SSURFM_ST2 + SSURF_E2

            TERLBL_2B = TERLBL_2B.replace(' ', '')
            TERLBL_2B = TERLBL_2B.replace('None', '')
            TERLBL_2B = TERLBL_2B.replace('()', '')
            s_TERLBL_2B = TERLBL_2B


            # TERLBL_3A (Detailed Polys - Calculated)
            # Equals <Prtflg_3> + <Ttex_3> + <Surfm_3> + "(" + <Surfm_Q3> + ")"+ <Surfm_St3> + <Surf_E3>

            PRTFLG_3   = stra(long_row[Long_Tbl_FieldName_Dict['PRTFLG_2']]).rstrip(' ').replace('None','')
            ## TTEX_3 will be discontinued from the data model in the future.
            ## TTEX_3     = stra(long_row.GetValue("TTEX_3"))
            TTEX_3A     = stra(long_row[Long_Tbl_FieldName_Dict['TTEX_3A']]).rstrip(' ').replace('None','')
            TTEX_3B     = stra(long_row[Long_Tbl_FieldName_Dict['TTEX_3B']]).rstrip(' ').replace('None','')
            TTEX_3C     = stra(long_row[Long_Tbl_FieldName_Dict['TTEX_3C']]).rstrip(' ').replace('None','')
            TTEX_3     = TTEX_3C + TTEX_3B + TTEX_3A
            SURFM_3    = stra(long_row[Long_Tbl_FieldName_Dict['SURFM_3']]).rstrip(' ').replace('None','')
            SURFM_Q3   = stra(long_row[Long_Tbl_FieldName_Dict['SURFM_Q3']]).rstrip(' ').replace('None','')
            SURFM_ST3  = stra(long_row[Long_Tbl_FieldName_Dict['SURFM_ST3']]).rstrip(' ').replace('None','')
            ## SURF_E3 will be discontinued from the data model in the future.
            ## SURF_E3    = stra(long_row.GetValue("SURF_E3"))
            SURF_E3A    = stra(long_row[Long_Tbl_FieldName_Dict['SURF_E3A']]).rstrip(' ').replace('None','')
            SURF_E3B    = stra(long_row[Long_Tbl_FieldName_Dict['SURF_E3B']]).rstrip(' ').replace('None','')
            SURF_E3C    = stra(long_row[Long_Tbl_FieldName_Dict['SURF_E3C']]).rstrip(' ').replace('None','')
            SURF_E3    = SURF_E3A + SURF_E3B + SURF_E3C

            TERLBL_3A = PRTFLG_3 + TTEX_3 + SURFM_3 + "(" + SURFM_Q3 + ")" + SURFM_ST3 + SURF_E3

            TERLBL_3A = TERLBL_3A.replace(' ', '')
            TERLBL_3A = TERLBL_3A.replace('None', '')
            TERLBL_3A = TERLBL_3A.replace('()', '')
            s_TERLBL_3A = TERLBL_3A


            # TERLBL_3B (Detailed Polys - Calculated)
            # Equals <Sttex_3> + <Ssurfm_3> + "(" + <Ssurfm_Q3> + ")"+ <Ssurfm_St3> + <Ssurf_E3>
            ## STTEX_3 will be discontinued from the data model in the future.
            ## STTEX_3 = stra(long_row.GetValue("STTEX_3"))
            STTEX_3A = stra(long_row[Long_Tbl_FieldName_Dict['STTEX_3A']]).rstrip(' ').replace('None','')
            STTEX_3B = stra(long_row[Long_Tbl_FieldName_Dict['STTEX_3B']]).rstrip(' ').replace('None','')
            STTEX_3C = stra(long_row[Long_Tbl_FieldName_Dict['STTEX_3C']]).rstrip(' ').replace('None','')
            STTEX_3 = STTEX_3C + STTEX_3B + STTEX_3A
            SSURFM_3 = stra(long_row[Long_Tbl_FieldName_Dict['SSURFM_3']]).rstrip(' ').replace('None','')
            SSURFM_Q3 = stra(long_row[Long_Tbl_FieldName_Dict['SSURFM_Q3']]).rstrip(' ').replace('None','')
            SSURFM_ST3 = stra(long_row[Long_Tbl_FieldName_Dict['SSURFM_ST3']]).rstrip(' ').replace('None','')
            ## SSURF_E3 will be discontinued from the data model in the future.
            ## SSURF_E3 = stra(long_row.GetValue("SSURF_E3"))
            SSURF_E3A = stra(long_row[Long_Tbl_FieldName_Dict['SSURF_E3A']]).rstrip(' ').replace('None','')
            SSURF_E3B = stra(long_row[Long_Tbl_FieldName_Dict['SSURF_E3B']]).rstrip(' ').replace('None','')
            SSURF_E3C = stra(long_row[Long_Tbl_FieldName_Dict['SSURF_E3C']]).rstrip(' ').replace('None','')
            SSURF_E3 = SSURF_E3A + SSURF_E3B + SSURF_E3C

            TERLBL_3B = STTEX_3 + SSURFM_3 + "(" + SSURFM_Q3 + ")" + SSURFM_ST3 + SSURF_E3

            TERLBL_3B = TERLBL_3B.replace(' ', '')
            TERLBL_3B = TERLBL_3B.replace('None', '')
            TERLBL_3B = TERLBL_3B.replace('()', '')
            s_TERLBL_3B = TERLBL_3B


            # GEOP_LBL (Detailed Polys - Calculated)
            # Equals <Geop_1> + <Geop_Q1> + <Geop_St1> + <Geop_InZ1> + <Geop_Scm1> + <Geop_2> + <Geop_Q2> + <Geop_St2> + <Geop_InZ2> + <Geop_Scm> + <Geop_3> + + <Geop_Q3> + <Geop_St3> + <Geop_InZ3> + <Geop_Scm3>
            #                                                                                                                               ^^^^ GEOP_SCM2
            GEOP_1 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_1']]).rstrip(' ').replace('None','')
            GEOP_Q1 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_Q1']]).rstrip(' ').replace('None','')
            GEOP_ST1 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_ST1']]).rstrip(' ').replace('None','')
            GEOP_INZ1 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_INZ1']]).rstrip(' ').replace('None','')
            ## GEOP_SCM1 will be discontinued from the data model in the future.
            ## GEOP_SCM1 = stra(long_row.GetValue("GEOP_SCM1"))
            GEOP_SCM1A = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_SCM1A']]).rstrip(' ').replace('None','')
            GEOP_SCM1B = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_SCM1B']]).rstrip(' ').replace('None','')
            GEOP_SCM1C = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_SCM1C']]).rstrip(' ').replace('None','')
            GEOP_SCM1 = GEOP_SCM1A + GEOP_SCM1B + GEOP_SCM1C
            GEOP_2 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_2']]).rstrip(' ').replace('None','')
            GEOP_Q2 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_Q2']]).rstrip(' ').replace('None','')
            GEOP_ST2 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_ST2']]).rstrip(' ').replace('None','')
            GEOP_INZ2 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_INZ2']]).rstrip(' ').replace('None','')
            ## GEOP_SCM2 will be discontinued from the data model in the future.
            ## GEOP_SCM2 = stra(long_row.GetValue("GEOP_SCM2"))
            GEOP_SCM2A = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_SCM2A']]).rstrip(' ').replace('None','')
            GEOP_SCM2B = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_SCM2B']]).rstrip(' ').replace('None','')
            GEOP_SCM2C = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_SCM2C']]).rstrip(' ').replace('None','')
            GEOP_SCM2 = GEOP_SCM2A + GEOP_SCM2B + GEOP_SCM2C
            GEOP_3 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_3']]).rstrip(' ').replace('None','')
            GEOP_Q3 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_Q3']]).rstrip(' ').replace('None','')
            GEOP_ST3 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_ST3']]).rstrip(' ').replace('None','')
            GEOP_INZ3 = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_INZ3']]).rstrip(' ').replace('None','')
            ## GEOP_SCM3 will be discontinued from the data model in the future.
            ## GEOP_SCM3 = stra(long_row.GetValue("GEOP_SCM3"))
            GEOP_SCM3A = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_SCM3A']]).rstrip(' ').replace('None','')
            GEOP_SCM3B = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_SCM3B']]).rstrip(' ').replace('None','')
            GEOP_SCM3C = stra(long_row[Long_Tbl_FieldName_Dict['GEOP_SCM3C']]).rstrip(' ').replace('None','')
            GEOP_SCM3 = GEOP_SCM3A + GEOP_SCM3B + GEOP_SCM3C

            GEOP_LBL = GEOP_1 + GEOP_Q1 + GEOP_ST1 + GEOP_INZ1 + GEOP_SCM1 + GEOP_2 + GEOP_Q2 + GEOP_ST2 + GEOP_INZ2 + GEOP_SCM2 + GEOP_3 + GEOP_Q3 + GEOP_ST3 + GEOP_INZ3 + GEOP_SCM3

            GEOP_LBL = GEOP_LBL.replace(' ', '')
            GEOP_LBL = GEOP_LBL.replace('None', '')
            GEOP_LBL = GEOP_LBL.replace('()', '')
            s_GEOP_LBL = GEOP_LBL

            # DRAIN_LBL (Detailed Polys - Calculated)
            # Equals <Drain_1> + <Drain_Sep> + <Drain_2>
            DRAIN_1 = stra(long_row[Long_Tbl_FieldName_Dict['DRAIN_1']]).rstrip(' ').replace('None','')
            DRAIN_SEP = stra(long_row[Long_Tbl_FieldName_Dict['DRAIN_SEP']]).rstrip(' ').replace('None','')
            DRAIN_2 = stra(long_row[Long_Tbl_FieldName_Dict['DRAIN_2']]).rstrip(' ').replace('None','')

            s_DRAIN_LBL = DRAIN_1 + DRAIN_SEP + DRAIN_2

            s_DRAIN_LBL = s_DRAIN_LBL.replace(' ', '')
            s_DRAIN_LBL = s_DRAIN_LBL.replace('None', '')
            s_DRAIN_LBL = s_DRAIN_LBL.replace('()', '')


            # SLOPE_LBL (Detailed Polys - Calculated)
            # Equals <Slpll_1> + "-" + <Slpul_1> + "," + <Slpll_2> + "-" + <Slpul_2>
            SLPLL_1 = stra(long_row[Long_Tbl_FieldName_Dict['SLPLL_1']]).rstrip(' ').replace('None','')
            if SLPLL_1 == None:
                SLPLL_1 = 0
            SLPUL_1 = stra(long_row[Long_Tbl_FieldName_Dict['SLPUL_1']]).rstrip(' ').replace('None','')
            if SLPUL_1 == None:
                SLPUL_1 = 0
            SLPLL_2 = stra(long_row[Long_Tbl_FieldName_Dict['SLPLL_2']]).rstrip(' ').replace('None','')
            if SLPLL_2 == None:
                SLPLL_2 = 0
            SLPUL_2 = stra(long_row[Long_Tbl_FieldName_Dict['SLPUL_2']]).rstrip(' ').replace('None','')
            if SLPUL_2 == None:
                SLPUL_2 = 0
            s_SLOPE_LBL = SLPLL_1 + "-" + SLPUL_1 + "," + SLPLL_2 + "-" + SLPUL_2
            s_SLOPE_LBL = s_SLOPE_LBL.replace(' ', '')
            s_SLOPE_LBL = s_SLOPE_LBL.replace('None', '')
            s_SLOPE_LBL = s_SLOPE_LBL.replace('()', '')
            s_SLOPE_LBL = s_SLOPE_LBL.strip('-,')
            # If there is no real slope information, set it to a blank label
            if s_SLOPE_LBL in ["0-0,0-0", "0-0"]:
                s_SLOPE_LBL = ""

            # SLPSTBCLSR (Detailed Polys - Calculated)
            # Equals <Slpstb_Cls> + <Rdstb_Flg>
            SLPSTB_CLS = stra(long_row[Long_Tbl_FieldName_Dict['SLPSTB_CLS']]).rstrip(' ').replace('None','')
            RDSTB_FLG = stra(long_row[Long_Tbl_FieldName_Dict['RDSTB_FLG']]).rstrip(' ').replace('None','')

            s_SLPSTBCLSR = SLPSTB_CLS + RDSTB_FLG

            s_SLPSTBCLSR = s_SLPSTBCLSR.replace(' ', '')
            s_SLPSTBCLSR = s_SLPSTBCLSR.replace('None', '')
            s_SLPSTBCLSR = s_SLPSTBCLSR.replace('()', '')

            if s_SLPSTBCLSR in SSC_Dictionary:
                s_SSC_TXT = SSC_Dictionary[s_SLPSTBCLSR]
            else:
                s_SSC_TXT = ''

            if SLPSTB_CLS in PSTBCLS_Dictionary:
                s_PSTBCLS_TP = PSTBCLS_Dictionary[SLPSTB_CLS]
            else:
                s_PSTBCLS_TP = ''

            # TER_LBL (Detailed Polys - Calculated)
            # If TER or TES Equals  <TERLBL_1A> + [ <TERLBL_1B> ] + <Comrel1_2> + <TERLBL_2A> + [ <TERLBL_2B> ] <TERLBL_2A> + [<TERLBL_2B>] + <Comrel2_3> + <TERLBL_3A> + [ <TERLBL_3B> ] <TERLBL_3A> + [<TERLBL_3B>] + '-' + <GEOP_LBL> IF BTM equils <Tdec_1> + <TERLBL_1A> + [ <TERLBL_1B> ] + <Tdec_2> + <TERLBL_2A> + [ <TERLBL_2B> ] <TERLBL_2A> + [<TERLBL_2B>] + <Tdec_3> + <TERLBL_3A> + [ <TERLBL_3B> ] <TERLBL_3A> + [<TERLBL_3B>] + '-' + <GEOP_LBL>
            PROJ_TYPE = ProjDetails_Dictionary[BAPID]['PROJ_TYPE']
            COMREL1_2 = stra(long_row[Long_Tbl_FieldName_Dict['COMREL1_2']]).rstrip(' ').replace('None','')
            COMREL2_3 = stra(long_row[Long_Tbl_FieldName_Dict['COMREL2_3']]).rstrip(' ').replace('None','')
            TDEC_1 = stra(long_row[Long_Tbl_FieldName_Dict['TDEC_1']]).rstrip(' ').replace('None','')
            TDEC_2 = stra(long_row[Long_Tbl_FieldName_Dict['TDEC_2']]).rstrip(' ').replace('None','')
            TDEC_3 = stra(long_row[Long_Tbl_FieldName_Dict['TDEC_3']]).rstrip(' ').replace('None','')

            if TDEC_1 in ['1','2','3','4','5','6','7','8','9','10']:
                s_TDEC_1 = int(TDEC_1)
            else:
                s_TDEC_1 = None

            if TDEC_2 in ['1','2','3','4','5','6','7','8','9','10']:
                s_TDEC_2 = int(TDEC_2)
            else:
                s_TDEC_2 = None

            if TDEC_3 in ['1','2','3','4','5','6','7','8','9','10']:
                s_TDEC_3 = int(TDEC_3)
            else:
                s_TDEC_3 = None

            s_COMREL1_2 = COMREL1_2
            s_COMREL2_3 = COMREL2_3

            s_BEDROCK_1 = stra(long_row[Long_Tbl_FieldName_Dict['BEDROCK_1']]).rstrip(' ').replace('None','')
            s_BEDROCK_2 = stra(long_row[Long_Tbl_FieldName_Dict['BEDROCK_2']]).rstrip(' ').replace('None','')
            s_BEDROCK_3 = stra(long_row[Long_Tbl_FieldName_Dict['BEDROCK_3']]).rstrip(' ').replace('None','')

            #### We must add logic for ALL project types here.  The Terrain Label Type (TER_Label_Type) indicator in the Domain_Project_Type table?

            TER_LBL = 'None'

            if PROJ_TYPE in DPT_Dictionary:
                if DPT_Dictionary[PROJ_TYPE] == 'Delimiter':
                    TER_LBL = TERLBL_1A + "[" + TERLBL_1B + "]" + COMREL1_2 + TERLBL_2A + "[" + TERLBL_2B + "]" + COMREL2_3 + TERLBL_3A + "[" + TERLBL_3B + "]"  + '-' + GEOP_LBL
                elif DPT_Dictionary[PROJ_TYPE] == 'Decile':
                    TER_LBL = TDEC_1 + TERLBL_1A + "[" + TERLBL_1B + "]" + TDEC_2 + TERLBL_2A + "[" + TERLBL_2B + "]" + TDEC_3 + TERLBL_3A + "[" + TERLBL_3B + "]" + '-' + GEOP_LBL

            #if TER_LBL == 'None':
            #    CPMessage(' ** WARNING: PROJ_TYPE (' + TER_LBL + ') in TEIS_ID ' + stra(TEIS_ID)  + ' not mapped to a TER_LBL ** ' + TER_LBL, 3)    


            TER_LBL = TER_LBL.replace(' ', '')
            TER_LBL = TER_LBL.replace('()', '')
            TER_LBL = TER_LBL.replace('None', '')
            TER_LBL = TER_LBL.replace('[]', '')
            TER_LBL = TER_LBL.strip('-')
            s_TER_LBL = TER_LBL
            CPMessage('                -- ' + TER_LBL, 5)

            # ORIG_LBL (Detailed Polys)
            # Equals <Orig_Lbl>; Called Map_Label in Compilation Project
            ## Changed to be the best corrected version that matches the original hard copy map with clarifications/QC
            #### Change metadata definition to match this (Deepa to provide)
            CAPTURED_LABEL = stra(long_row[Long_Tbl_FieldName_Dict['CAPLBL']]).rstrip(' ').replace('None','')
            CAPTURED_LABEL = CAPTURED_LABEL.replace(' ', '')
            CAPTURED_LABEL = CAPTURED_LABEL.replace('None', '')

            CAPTURED_LABEL_CORRECTED = stra(long_row[Long_Tbl_FieldName_Dict['CAPLBL_COR']]).rstrip(' ').replace('None','')
            CAPTURED_LABEL_CORRECTED = CAPTURED_LABEL_CORRECTED.replace(' ', '')
            CAPTURED_LABEL_CORRECTED = CAPTURED_LABEL_CORRECTED.replace('None', '')

            if stra(CAPTURED_LABEL_CORRECTED) != '':
                s_ORIG_LBL = CAPTURED_LABEL_CORRECTED
            else:
                s_ORIG_LBL = CAPTURED_LABEL

            #POLY_COM = stra(long_row[Long_Tbl_FieldName_Dict['POLY_COM']]).rstrip(' ').replace('None','')
            # normally we would "stra()" this value, but POLY_COM sometimes contains unicode characters which causes error

            POLY_COM = long_row[Long_Tbl_FieldName_Dict['POLY_COM']]
            if POLY_COM in (' ',None):
                POLY_COM = ''
            s_POLY_COM = POLY_COM

            SMPL_TYPE = stra(long_row[Long_Tbl_FieldName_Dict['SMPL_TYPE']]).rstrip(' ').replace('None','')
            s_SMPL_TYPE = SMPL_TYPE

            s_POLY_TEX = ''
            s_POLY_THK = ''
            s_THEME_1 = ''
            s_THEME_2 = ''
            s_THEME_3 = ''

            SFCERO_POT = stra(long_row[Long_Tbl_FieldName_Dict['SFCERO_POT']]).rstrip(' ').replace('None','')
            s_SFCERO_POT = SFCERO_POT

            if SFCERO_POT in SEP_Dictionary:
                s_SEP_TXT = SEP_Dictionary[SFCERO_POT]
            else:
                s_SEP_TXT = ''

            LSSED_CLS = stra(long_row[Long_Tbl_FieldName_Dict['LSSED_CLS']]).rstrip(' ').replace('None','')
            if LSSED_CLS <> '':
                s_LSSED_CLS = int(LSSED_CLS)

                if LSSED_CLS in LISSC_Dictionary:
                    s_LISSC_TXT = LISSC_Dictionary[LSSED_CLS]
                else:
                    s_LISSC_TXT = ''
            else:
                s_LSSED_CLS = None
                s_LISSC_TXT = ''

            SESED_CLS = stra(long_row[Long_Tbl_FieldName_Dict['SESED_CLS']]).rstrip(' ').replace('None','')
            s_SESED_CLS = SESED_CLS

            if SESED_CLS in SESC_Dictionary:
                s_SESC_TXT = SESC_Dictionary[SESED_CLS]
            else:
                s_SESC_TXT = ''

            if SURFM_1 in SURFM_Dictionary:
                s_SURFM_DOM = SURFM_Dictionary[SURFM_1]
            else:
                s_SURFM_DOM = ''


            #### POLY_TEX (Data Definition Doesn't Exist Yet, Multifield) - NO ACTION
            # CPMessage('        - Working on POLY_TEX', 3)
            # CPMessage('            -- Nothing to do', 4)    

            #### POLY_THK (Data Definition Doesn't Exist Yet, Multifield) - NO ACTION 
            # CPMessage('        - Working on POLY_THK', 3)
            # CPMessage('            -- Nothing to do', 4)

            new_short_row = [s_SHAPE,s_TEIS_ID,s_PROJPOLYID,s_BAPID,s_MOD_DATE,s_PROJ_TYPE,s_PROJ_NAME,s_PROJ_SCALE,s_ECO_SEC,s_BGC_LBL,s_ECOLBL_1,s_SDEC_1,s_SITES_LBL1,s_SITEMCLBL1,s_SEILBL_1,s_SECL_1,s_SESUBCL_1,s_COND_1,s_VIAB_1,s_STRCTLBL_1,s_ECOLBL_2,s_SDEC_2,s_SITES_LBL2,s_SITEMCLBL2,s_SEILBL_2,s_SECL_2,s_SESUBCL_2,s_COND_2,s_VIAB_2,s_STRCTLBL_2,s_ECOLBL_3,s_SDEC_3,s_SITES_LBL3,s_SITEMCLBL3,s_SEILBL_3,s_SECL_3,s_SESUBCL_3,s_COND_3,s_VIAB_3,s_STRCTLBL_3,s_TER_LBL,s_TDEC_1,s_TERLBL_1A,s_TERLBL_1B,s_BEDROCK_1,s_COMREL1_2,s_TDEC_2,s_TERLBL_2A,s_TERLBL_2B,s_BEDROCK_2,s_COMREL2_3,s_TDEC_3,s_TERLBL_3A,s_TERLBL_3B,s_BEDROCK_3,s_GEOP_LBL,s_DRAIN_LBL,s_SLOPE_LBL,s_SLPSTBCLSR,s_SFCERO_POT,s_LSSED_CLS,s_SESED_CLS,s_SSC_TXT,s_SEP_TXT,s_LISSC_TXT,s_SESC_TXT,s_POLY_TEX,s_POLY_THK,s_SURFM_DOM,s_THEME_1,s_THEME_2,s_THEME_3,s_POLY_COM,s_FCODE,s_SMPL_TYPE,s_LEGEND_COM,s_LEGEND_URL,s_ORIG_LBL,s_PSTBCLS_TP,s_ERROR_SUM]
            #new_short_row = [s_SHAPE,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None,None]

            try:
                short_cursor.insertRow(new_short_row)
            except:
                CPMessage(' ** Error in row: ' + stra(Number_Processed), 3)
            Number_Processed = Number_Processed + 1'''
    exec(exec_string)

    del cursor
    del short_cursor
    
    CPMessage('            - Processed all ' + stra(Number_Processed), 4)

    CPMessage('            - Creating attribute indexes in Short Table feature class', 4)
    for idx in idx_list:
        field_list = [f for f in idx.fields]
        index_should_be_added = True
        for field in field_list:
            if field.type in ['OID','Geometry']:
                index_should_be_added = False
        if index_should_be_added:
            field_name_list = [f.name for f in idx.fields]
            field_name_list_string = ''
            for field_name in field_name_list:
                field_name_list_string += field_name + ", "
            field_name_list_string = field_name_list_string[:-2]
            CPMessage('                - Adding attribute index ' + idx.name + ' on field(s) ' + field_name_list_string, 4)
            field_name_list_string = ''
            for field_name in field_name_list:
                field_name_list_string += field_name + ";"
            field_name_list_string = field_name_list_string[:-1]
            arcpy.AddIndex_management(Processing_Variables['Short_Tbl'],field_name_list_string,idx.name) ## index will be NON_UNIQUE and NON_ASCENDING by default
            
    return(Processing_Variables['PROCESS_STATUS'])


    
#*********************************************************************************************************
#  Main Program
#*********************************************************************************************************

#---------------------------------------------------------------------------------------------------------
# Run Main for ArcGIS 9.3 and 10.0
#
#   Main Method manages the overall program
#---------------------------------------------------------------------------------------------------------
def runMain93():

    Initialize93()
    SetupWorkspace93()
    
    PopulateTEISShortTable93()

    return Processing_Variables['PROCESS_STATUS']


#---------------------------------------------------------------------------------------------------------
# Run Main for ArcGIS 10.1
#
#   Main Method manages the overall program
#---------------------------------------------------------------------------------------------------------
def runMain101():

    Initialize101()
    SetupWorkspace101()
    
    PopulateTEISShortTable101()

    return Processing_Variables['PROCESS_STATUS']

#*********************************************************************************************************
#  Execution
#*********************************************************************************************************



d = arcpy.GetInstallInfo('desktop')
if len(d['Version'].split('.')) > 2:
    dv = d['Version'].rsplit('.',len(d['Version'].split('.'))-2)[0]
    del arcpy
    import arcpy
    runMain101()
else:
    dv = d['Version']
    if float(dv) <10.1:
        runMain93()
    else:
        print '** Unsupported ArcGIS version (' + str(d['Version']) + ') running on this machine.'

#---------------------------------------------------------------------------------------------------------
#  Clean-up
#---------------------------------------------------------------------------------------------------------

END_TIME       = time.ctime(time.time())
END_TIME_SEC   = time.time()
END_TIME_SQL   = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
EXECUTION_TIME = END_TIME_SEC - START_TIME_SEC

if EXECUTION_TIME < 120:
    EXECUTION_TIME_STRING = stra(int(EXECUTION_TIME)) + ' seconds.'
elif EXECUTION_TIME < 3600:
    EXECUTION_TIME_STRING = stra(round(EXECUTION_TIME / 60, 1)) + ' minutes.'
else:
    EXECUTION_TIME_STRING = stra(round(EXECUTION_TIME / 3600, 2)) + ' hours.'

CPMessage('    End Time: ' + END_TIME, 2)
CPMessage('    Execution Time: ' + EXECUTION_TIME_STRING, 4)
CPMessage('Complete: ' + PYTHON_SCRIPT, 1)
CPMessage('Next Script: CreateProjectBoundariesForWebsite',4)
CPMessage('Preconditions for next step: Maps folder and PDF_Map_Data.gdb exist in TEIS Environment & Tools_and_Templates.gdb\LUT_Broad_Project_Types is updated with any new project types and PDF_Type is filled in.',4)

# arcpy.CheckInExtension("spatial")
#del gp

# Report Program Execution Status back to the calling program

if 'SendRunLogToEmail' in Processing_Variables:
    for Email in Processing_Variables['SendRunLogToEmail']:
        SendRunLogToEmail(Email)

# sys.exit(Processing_Variables['PROCESS_STATUS'])

#*********************************************************************************************************
#  End of Program
#*********************************************************************************************************